--  ________________________________________________________________________  --
-- |                                                                        | --
-- |          _______ _______                               __              | --
-- |         |   _   |   |   |.-----.-----.---.-.----.----.|  |--.          | --
-- |         |       |       ||__ --|  -__|  _  |   _|  __||     |          | --
-- |         |___|___|___|___||_____|_____|___._|__| |____||__|__|          | --
-- |                                                                        | --
-- |________________________________________________________________________| --
--  ________________________________________________________________________  --
-- |                                                                        | --
-- |                           AHsearch by kunda                            | --
-- |________________________________________________________________________| --
-- |                                                                        | --
-- |                                                                        | --
-- | AHsearch is a World of Warcraft AddOn that saves Auction House search  | --
-- | queries.                                                               | --
-- |                                                                        | --
-- | AHsearch adds a 20x20 pixel 'Arrow Down' Button ('AHsearch Button') to | --
-- | the right side of the Auction House searchfield.                       | --
-- |                                                                        | --
-- | Features:                                                              | --
-- | - Right-click 'AHsearch Button' for configuration.                     | --
-- |   (Or: Right-click on category or search query for configuration.)     | --
-- | - Left-click 'AHsearch Button' for your saved Search Queries.          | --
-- | - Cycle Search: ( Next / Previous / Random ) [DISABLED by DEFAULT]     | --
-- |   - Key Bindings [ESC -> Key bindings -> AHsearch]                     | --
-- |                                                                        | --
-- | How To:                                                                | --
-- | Step 1: Right-click 'AHsearch Button'.                                 | --
-- |         Click 'Categories'-Tab, enter a name and click 'Create New'.   | --
-- | Step 2: Click 'Search Queries'-Tab, select a category, enter a title   | --
-- |         for your new search, set up your search criteria and click     | --
-- |         'Create New'.                                                  | --
-- | Step 3: Click 'Close Configuration' or Right-click 'AHsearch Button'.  | --
-- | Step 4: Left-click 'AHsearch Button' for your search queries.          | --
-- |                                                                        | --
-- |________________________________________________________________________| --
-- |                                                                        | --
-- |                                                                        | --
-- | Credits:                                                               | --
-- | Idea based on AHFavorites from Narwick.                                | --
-- | Cycle search idea based on AHsearchNext from Evan.                     | --
-- |                                                                        | --
-- |________________________________________________________________________| --
--                                                                            --

-- ---------------------------------------------------------------------------------------------------------------------
AHsearch = CreateFrame("Frame")     -- container
AHsearch_Data = {}                  -- SavedVariable Data table
AHsearch_Options = {}               -- SavedVariable Options table

local L = AHsearch_Locales          -- localization table

local _G = _G
local table_insert = _G.table.insert
local table_sort = _G.table.sort
local math_floor = _G.math.floor
local GetTime = _G.GetTime
local unpack = _G.unpack

local maxBoxInputLength = 50        -- maximum characters for box entries
local maxItemLevel = 85             -- maximum required ItemLevel (maximum level a character can reach)
local cycleChk                      -- nil/false = NO key binding allowed | true = key binding allowed 
local cycleCat                      -- cycle search category
local cycleSQ                       -- cycle search search query

local FrameTime = GetTime()         -- for VisibilityCheck()

local minCat = 10                   -- minimum category size
local maxCat = 50                   -- maximum category size (>50 = scrollframe)
local GUI_CatFrame_data = {}        -- category data
local GUI_CatFrame_Count            -- category data count

local minQuery = 10                 -- minimum search query size
local maxQuery = 50                 -- maximum search query size (>50 = scrollframe)
local GUI_SQFrame_data = {}         -- search query data
local GUI_SQFrame_Count = {}        -- search query data count

local GUI_Tab = 1                   -- Configuration Tabs
local GUI_curCat                    -- current category
local GUI_curSQ                     -- current search query

local FilterTable = {}              -- AH Filter table
local Filter1Table = {}             -- Lvl1 AH Filter
local Filter2Table = {}             -- Lvl2 AH Filter
local Filter3Table = {}             -- Lvl3 AH Filter
local curFilter1                    -- selected Lvl1 Filter
local curFilter2                    -- selected Lvl2 Filter
local curFilter3                    -- selected Lvl3 Filter

local configMode = false            -- config mode check value
local confirmIsShown = false        -- confirm frame check value
local hideMain = false              -- config mode main frame hide/show

local default_curCat        = 25    -- default option: height rows
local default_curCatWidth   = 175   -- default option: width px
local default_curQuery      = 25    -- default option: height rows
local default_curQueryWidth = 175   -- default option: width px
local default_cycleNP       = false -- default option: cycle search Next/Previous
local default_cycleRND      = false -- default option: cycle search Random
local default_scaleMain     = 100   -- default option: Main Frame (cat/sq) Scale

-- color
local normalBorderColor = {0.4, 0.4, 0.4, 1}
local configBorderColor = {0.8, 0.2, 0.2, 1}

-- size START
local sizeGeneral = {}
sizeGeneral.Border     = 1
sizeGeneral.Space      = 2
sizeGeneral.Offset     = 5
sizeGeneral.BarHeight  = 14
sizeGeneral.SliderSize = 14--17

-- - category
local function CatSizeUpdate(newsize)
	sizeGeneral.CatButtonSFWidth     = newsize-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
	sizeGeneral.CatButtonNoSFWidth   = newsize-sizeGeneral.Offset-sizeGeneral.Offset
	sizeGeneral.CatTextSFWidth       = newsize-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border - sizeGeneral.BarHeight-sizeGeneral.Space
	sizeGeneral.CatTextNoSFWidth     = newsize-sizeGeneral.Offset-sizeGeneral.Offset - sizeGeneral.BarHeight-sizeGeneral.Space
end
CatSizeUpdate(default_curCatWidth)

-- - search queries
local function QuerySizeUpdate(newsize)
	sizeGeneral.QueryButtonSFWidth   = newsize-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
	sizeGeneral.QueryButtonNoSFWidth = newsize-sizeGeneral.Offset-sizeGeneral.Offset
	sizeGeneral.QueryTextSFWidth     = newsize-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
	sizeGeneral.QueryTextNoSFWidth   = newsize-sizeGeneral.Offset-sizeGeneral.Offset
end
QuerySizeUpdate(default_curQueryWidth)

-- - config
sizeGeneral.ConfigTextWidth   = 200
sizeGeneral.ConfigSliderWidth = 150
sizeGeneral.ConfigCatWidth    = 175

sizeGeneral.ConfigCatButtonSFWidth        = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigCatButtonNoSFWidth      = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset-sizeGeneral.Offset
sizeGeneral.ConfigCatTextSFWidth          = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border - sizeGeneral.BarHeight-sizeGeneral.Space
sizeGeneral.ConfigCatTextNoSFWidth        = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset-sizeGeneral.Offset - sizeGeneral.BarHeight-sizeGeneral.Space

sizeGeneral.ConfigQueryButtonSFWidth      = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigQueryButtonNoSFWidth    = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset-sizeGeneral.Offset
sizeGeneral.ConfigQueryTextSFWidth        = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigQueryTextNoSFWidth      = sizeGeneral.ConfigCatWidth-sizeGeneral.Offset-sizeGeneral.Offset

sizeGeneral.ConfigPullDownOrderWidth      = 50
sizeGeneral.ConfigPullDownOrderSFWidth    = sizeGeneral.ConfigPullDownOrderWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigPullDownOrderNoSFWidth  = sizeGeneral.ConfigPullDownOrderWidth-sizeGeneral.Offset-sizeGeneral.Offset

sizeGeneral.ConfigPullDownColorWidth      = 110
sizeGeneral.ConfigPullDownColorSFWidth    = sizeGeneral.ConfigPullDownColorWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigPullDownColorNoSFWidth  = sizeGeneral.ConfigPullDownColorWidth-sizeGeneral.Offset-sizeGeneral.Offset

sizeGeneral.ConfigPullDownIconWidth       = 50
sizeGeneral.ConfigPullDownIconSFWidth     = sizeGeneral.ConfigPullDownIconWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigPullDownIconNoSFWidth   = sizeGeneral.ConfigPullDownIconWidth-sizeGeneral.Offset-sizeGeneral.Offset

sizeGeneral.ConfigPullDownFilterWidth     = 100
sizeGeneral.ConfigPullDownFilterSFWidth   = sizeGeneral.ConfigPullDownFilterWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigPullDownFilterNoSFWidth = sizeGeneral.ConfigPullDownFilterWidth-sizeGeneral.Offset-sizeGeneral.Offset

sizeGeneral.ConfigPullDownSortWidth       = 100
sizeGeneral.ConfigPullDownSortSFWidth     = sizeGeneral.ConfigPullDownSortWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border
sizeGeneral.ConfigPullDownSortNoSFWidth   = sizeGeneral.ConfigPullDownSortWidth-sizeGeneral.Offset-sizeGeneral.Offset
-- size END

-- magical 180 degree texture cut center rotation
local function rt(H,E,M,P)
	return E,P,E,M,H,P,H,M
end

local Textures = {
	AHsBmain        = {coords     =    { 37/128,  56/128, 19/64, 38/64},
	                   coordsPS   =    { 57/128,  76/128, 19/64, 38/64},
	                   coordsHL   =    { 77/128,  96/128, 19/64, 38/64}},
	AHsBprev        = {coords     =    { 37/128,  59/128, 39/64, 52/64},
	                   coordsPS   =    { 60/128,  82/128, 39/64, 52/64},
	                   coordsHL   =    { 83/128, 105/128, 39/64, 52/64}},
	AHsBnext        = {coords     = {rt( 37/128,  59/128, 39/64, 52/64)},
	                   coordsPS   = {rt( 60/128,  82/128, 39/64, 52/64)},
	                   coordsHL   = {rt( 83/128, 105/128, 39/64, 52/64)}},
	AHsBrandom      = {coords     =    { 97/128, 110/128, 19/64, 32/64},
	                   coordsHL   =    {111/128, 124/128, 19/64, 32/64}},

	HighLightDef    = {coords     =    { 19/128,  36/128, 19/64, 36/64}},
	ScrollFrameUp   = {coords     =    { 73/128,  90/128,  1/64, 18/64},
	                   coordsHL   =    { 91/128, 108/128,  1/64, 18/64},
	                   coordsDIS  =    {109/128, 126/128,  1/64, 18/64}},
	ScrollFrameDown = {coords     = {rt( 73/128,  90/128,  1/64, 18/64)},  -- 180 degree rota
	                   coordsHL   = {rt( 91/128, 108/128,  1/64, 18/64)},  -- 180 degree rota
	                   coordsDIS  = {rt(109/128, 126/128,  1/64, 18/64)}}, -- 180 degree rota
	ScrollFrameKnob = {coords     =    {  1/128,  18/128, 19/64, 36/64}},
	SearchBoxIcons  = {coordsLoff =    {  1/128,   6/128,  1/64, 18/64},
	                   coordsMoff =    {  9/128,  10/128,  1/64, 18/64},
	                   coordsRoff =    { 13/128,  18/128,  1/64, 18/64},
	                   coordsLon  =    { 19/128,  24/128,  1/64, 18/64},
	                   coordsMon  =    { 27/128,  28/128,  1/64, 18/64},
	                   coordsRon  =    { 31/128,  36/128,  1/64, 18/64},
	                   width      =    17,
	                   height     =    17},
	SliderKnob      = {coords     =    { 19/128,  36/128, 37/64, 54/64},
	                   coordsHL   =    { 91/128, 108/128,  1/64, 18/64}},
	SliderBG        = {coordsL    =    {  1/128,   6/128, 42/64, 48/64},
	                   coordsM    =    {  9/128,  10/128, 42/64, 48/64},
	                   coordsR    =    { 13/128,  18/128, 42/64, 48/64}},
	Expand          = {coords     =    { 37/128,  54/128,  1/64, 18/64}},
	Collapse        = {coords     = {rt( 37/128,  54/128,  1/64, 18/64)}}, -- 180 degree rota
	SelectArrow     = {coordsR    =    {119/128, 125/128, 39/64, 51/64},
	                   coordsL    = {rt(118/128, 127/128, 37/64, 52/64)},  -- 180 degree rota
	                   width      =    6,
	                   height     =    12},
	SoryOrderIcon   = {coordsDesc =    {109/128, 116/128, 41/64, 49/64},
	                   coordsAsc  = {rt(109/128, 116/128, 41/64, 49/64)}}, -- 180 degree rota
}

local itemRarity = {
  [1] = {color = "|cffffff9a", name = L["All"]},           -- all       (AH: -1)
  [2] = {color = "|cff9d9d9d", name = ITEM_QUALITY0_DESC}, -- poor      (AH: 0)
  [3] = {color = "|cffffffff", name = ITEM_QUALITY1_DESC}, -- common    (AH: 1)
  [4] = {color = "|cff1eff00", name = ITEM_QUALITY2_DESC}, -- uncommon  (AH: 2)
  [5] = {color = "|cff0070dd", name = ITEM_QUALITY3_DESC}, -- rare      (AH: 3)
  [6] = {color = "|cffa335ee", name = ITEM_QUALITY4_DESC}  -- epic      (AH: 4)
--[7] = {color = "|cffff8000"                              -- legendary (not in AH index)
--[8] = {color = "|cffe6cc80"                              -- artifact  (not in AH index)
}

local SortByTable = {
	[1] = "quality",  -- -> L["quality"]
	[2] = "level",    -- -> L["level"]
	[3] = "duration", -- -> L["duration"]
	[4] = "seller",   -- -> L["seller"]
	[5] = "bid"       -- -> L["bid"]
}

local categoryColors = {
	[1] = "|cffffffff",
	[2] = "|cffbbbbbb",
	[3] = "|cff777777",
	[4] = "|cfff32f00",
	[5] = "|cffff8c00",
	[6] = "|cffffd700",
	[7] = "|cffdeb887",
	[8] = "|cffa0522d",
	[9] = "|cffe9967a",
	[10] = "|cff4169e1",
	[11] = "|cff008cb4",
	[12] = "|cff7edef4",
	[13] = "|cffa020f0",
	[14] = "|cffee82ee",
	[15] = "|cff95004a",
	[16] = "|cff00ff00"
}

local categoryIcons = { -- fudge factor ftw
	[1]  = {texture = "",                                                           coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[2]  = {texture = "Interface\\GroupFrame\\UI-Group-LeaderIcon",                 coords = {0.00000001, 0.93750001, 0.00000001, 0.99999999}}, --{0, 15/16, 0, 1}},
	[3]  = {texture = "Interface\\Minimap\\Tracking\\Auctioneer",                   coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[4]  = {texture = "Interface\\Minimap\\PartyRaidBlips",                         coords = {0.76953125, 0.85937501, 0.53125001, 0.71093751}}, --{197/256, 220/256, 68/128, 91/128}},
	[5]  = {texture = "Interface\\FriendsFrame\\InformationIcon",                   coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[6]  = {texture = "Interface\\FriendsFrame\\ClearBroadcastIcon",                coords = {0.00000001, 0.93750001, 0.06250001, 0.99999999}}, --{0, 15/16, 1/16, 1}},
	[7]  = {texture = "Interface\\FriendsFrame\\StatusIcon-DnD",                    coords = {0.00000001, 0.93750001, 0.00000001, 0.93750001}}, --{0, 15/16, 0, 15/16}},
	[8]  = {texture = "Interface\\GossipFrame\\AvailableQuestIcon",                 coords = {0.06250001, 0.99999999, 0.00000001, 0.99999999}}, --{1/16, 1, 0, 1}},
	[9]  = {texture = "Interface\\HelpFrame\\HotIssueIcon",                         coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[10] = {texture = "Interface\\LFGFrame\\LFGRole",                               coords = {0.75000001, 0.99999999, 0.00000001, 0.99999999}}, --{48/64, 64/64, 0, 1}},
	[11] = {texture = "Interface\\PaperDollInfoFrame\\UI-EquipmentManager-Toggle",  coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[12] = {texture = "Interface\\GossipFrame\\TrainerGossipIcon",                  coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[13] = {texture = "Interface\\Minimap\\Tracking\\Reagents",                     coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[14] = {texture = "Interface\\Minimap\\Tracking\\Food",                         coords = {0.00000001, 0.99999999, 0.00000001, 0.99999999}}, --{0, 1, 0, 1}},
	[15] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.00000001, 0.12500001, 0.00000001, 0.50000001}}, --{  0/256,  32/256,  0/64, 32/64}},
	[16] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.12500001, 0.25000001, 0.00000001, 0.50000001}}, --{ 32/256,  64/256,  0/64, 32/64}},
	[17] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.25000001, 0.37500001, 0.00000001, 0.50000001}}, --{ 64/256,  96/256,  0/64, 32/64}},
	[18] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.37500001, 0.50000001, 0.00000001, 0.50000001}}, --{ 96/256, 128/256,  0/64, 32/64}},
	[19] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.50000001, 0.62500001, 0.00000001, 0.50000001}}, --{128/256, 160/256,  0/64, 32/64}},
	[20] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.62500001, 0.75000001, 0.00000001, 0.50000001}}, --{160/256, 192/256,  0/64, 32/64}},
	[21] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.75000001, 0.87500001, 0.00000001, 0.50000001}}, --{192/256, 224/256,  0/64, 32/64}},
	[22] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.87500001, 0.99999999, 0.00000001, 0.50000001}}, --{224/256, 256/256,  0/64, 32/64}},
	[23] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.00000001, 0.12500001, 0.50000001, 0.99999999}}, --{  0/256,  32/256, 32/64, 64/64}},
	[24] = {texture = "Interface\\Addons\\AHsearch\\AHsearch-textures-tradeskills", coords = {0.12500001, 0.25000001, 0.50000001, 0.99999999}}, --{ 32/256,  64/256, 32/64, 64/64}},
}
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function DBupdate()
	if AHsearch_Data then
		for k in pairs(AHsearch_Data) do

			if AHsearch_Data[k].icon then
				local newIcon = 1
				for i = 1, #categoryIcons do
					if AHsearch_Data[k].icon == categoryIcons[i].texture then
						newIcon = i
					end
				end
				AHsearch_Data[k].icon = newIcon
				--print("DBupdate - icon update:", k, newIcon) -- TEST
			end

			if AHsearch_Data[k].queries then
				local x = 0
				for k2 in pairs(AHsearch_Data[k].queries) do
					x = x + 1
				end
				if x == 0 then
					AHsearch_Data[k].queries = nil
					--print("DBupdate - delete empty queries:", k) -- TEST
				end
			end

			if AHsearch_Data[k].queries then
				for k2, v2 in pairs(AHsearch_Data[k].queries) do
					v2.SortBy = "quality"
					--v2.SortOrder = 1 -- nil is blizz AH default
					--print("DBupdate - added default SortBy (SortOrder=nil):", v2.SortBy) -- TEST
					if v2.minLevel then
						if v2.minLevel == "" then
							v2.minLevel = nil
						else
							local minNum = tonumber(v2.minLevel)
							if type(minNum) == "number" then
								if minNum < 1 or minNum > 99 then
									v2.minLevel = nil
								else
									v2.minLevel = minNum
								end
							else
								v2.minLevel = nil
							end
						end
						--print("DBupdate - minLevel to number:", type(v2.minLevel), v2.minLevel) -- TEST
					end
					if v2.maxLevel then
						if v2.maxLevel == "" then
							v2.maxLevel = nil
						else
							local maxNum = tonumber(v2.maxLevel)
							if type(maxNum) == "number" then
								if maxNum < 1 or maxNum > 99 then
									v2.maxLevel = nil
								else
									v2.maxLevel = maxNum
								end
							else
								v2.maxLevel = nil
							end
						end
						--print("DBupdate - maxLevel to number:", type(v2.maxLevel), v2.maxLevel) -- TEST
					end
					if v2.isUsable then
						if v2.isUsable == 0 then
							v2.isUsable = nil
						end
						--print("DBupdate - isUsable:", type(v2.isUsable), v2.isUsable) -- TEST
					end
					if v2.showOnPlayer then
						if v2.showOnPlayer == 0 then
							v2.showOnPlayer = nil
						end
						--print("DBupdate - showOnPlayer:", type(v2.showOnPlayer), v2.showOnPlayer) -- TEST
					end
				end
			end

		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- Template BorderTRBL START ----------------------------------------
local function TemplateBorderTRBL(frame, color)
	local texture = frame:CreateTexture(nil, "BORDER")
	texture:SetPoint("TOPLEFT", 1, -1)
	texture:SetPoint("BOTTOMRIGHT", -1, 1)
	texture:SetTexture(0, 0, 0, 1)
	local textureBackground = frame:CreateTexture(frame:GetName().."Border", "BACKGROUND")
	textureBackground:SetPoint("TOPLEFT", 0, 0)
	textureBackground:SetPoint("BOTTOMRIGHT", 0, 0)
	if color then
		if color == 1 then
			textureBackground:SetTexture(0.8, 0.2, 0.2, 1)
		end
	else
		textureBackground:SetTexture(0.4, 0.4, 0.4, 1)
	end
end
-- Template BorderTRBL END ----------------------------------------

-- Template TextButton START ----------------------------------------
local function DisableTextButton(name)
	_G[name.."Border"]:SetTexture(0.4, 0.4, 0.4, 1)
	_G[name]:Disable()
end
local function EnableTextButton(name, action)
	local buttoncolor
	if action == 1 then
		bordercolor = {0.73, 0.26, 0.21, 1}
	elseif action == 2 then
		bordercolor = {0.43, 0.32, 0.68, 1}
	elseif action == 3 then
		bordercolor = {0.24, 0.46, 0.21, 1}
	else
		bordercolor = {1, 1, 1, 1}
	end
	_G[name.."Border"]:SetTexture(bordercolor[1], bordercolor[2], bordercolor[3], bordercolor[4])
	_G[name]:Enable()
end

local function TemplateTextButton(button, text, action)
	local buttonName = button:GetName()

	local buttoncolor
	if action == 1 then
		button:SetNormalFontObject("GameFontNormal")
		button:SetDisabledFontObject("GameFontDisable") 
		buttoncolor = {0.38, 0, 0, 1}
		bordercolor = {0.73, 0.26, 0.21, 1}
	elseif action == 2 then
		button:SetNormalFontObject("GameFontWhiteSmall")
		button:SetDisabledFontObject("GameFontDisableSmall")
		buttoncolor = {0, 0, 0.5, 1}
		bordercolor = {0.43, 0.32, 0.68, 1}
	elseif action == 3 then
		button:SetNormalFontObject("GameFontWhiteSmall")
		button:SetDisabledFontObject("GameFontDisableSmall")
		buttoncolor = {0, 0.2, 0, 1}
		bordercolor = {0.24, 0.46, 0.21, 1}
	else
		button:SetNormalFontObject("GameFontNormal")
		button:SetDisabledFontObject("GameFontDisable")
		buttoncolor = {0, 0, 0, 1}
		bordercolor = {1, 1, 1, 1}
	end

	local textureBack = button:CreateTexture(nil, "BORDER")
	textureBack:SetPoint("TOPLEFT", 1, -1)
	textureBack:SetPoint("BOTTOMRIGHT", -1, 1)
	textureBack:SetTexture(0, 0, 0, 1)

	local textureBorder = button:CreateTexture(buttonName.."Border", "BACKGROUND")
	textureBorder:SetPoint("TOPLEFT", 0, 0)
	textureBorder:SetPoint("BOTTOMRIGHT", 0, 0)
	textureBorder:SetTexture(bordercolor[1], bordercolor[2], bordercolor[3], bordercolor[4])--0.4, 0.4, 0.4, 1)

	local textureNormal = button:CreateTexture(nil, "ARTWORK")
	textureNormal:SetPoint("TOPLEFT", 2, -2)
	textureNormal:SetPoint("BOTTOMRIGHT", -2, 2)
	textureNormal:SetTexture(buttoncolor[1], buttoncolor[2], buttoncolor[3], buttoncolor[4])
	button:SetNormalTexture(textureNormal)

	local textureDisabled = button:CreateTexture(nil, "OVERLAY")
	textureDisabled:SetPoint("TOPLEFT", 3, -3)
	textureDisabled:SetPoint("BOTTOMRIGHT", -3, 3)
	textureDisabled:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetDisabledTexture(textureDisabled)

	local textureHighlight = button:CreateTexture(nil, "OVERLAY")
	textureHighlight:SetPoint("TOPLEFT", 3, -3)
	textureHighlight:SetPoint("BOTTOMRIGHT", -3, 3)
	textureHighlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(textureHighlight)

	button:SetPushedTextOffset(1, -1)	
	button:SetText(text)
end
-- Template TextButton END ----------------------------------------

-- Template EditBox START ----------------------------------------
local function DisableEditBox(editboxName)
	_G[editboxName.."DisableTexture"]:Show()
	_G[editboxName]:EnableMouse(false)
	_G[editboxName]:ClearFocus()
	_G[editboxName.."Left"]:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsLoff))
	_G[editboxName.."Right"]:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsRoff))
	_G[editboxName.."Middle"]:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsMoff))
end
local function EnableEditBox(editboxName)
	_G[editboxName.."DisableTexture"]:Hide()
	_G[editboxName]:EnableMouse(true)
	_G[editboxName.."Left"]:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsLon))
	_G[editboxName.."Right"]:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsRon))
	_G[editboxName.."Middle"]:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsMon))
end

local function TemplateEditBox(editbox)
	local editboxName = editbox:GetName()
	editbox:SetHeight(Textures.SearchBoxIcons.height)
	editbox:SetAutoFocus(false)
	editbox:SetFontObject(GameFontWhiteSmall)
	editbox:SetTextInsets(2, 0, 0, 0) 

	_G[editboxName.."Left"]:SetWidth(5)
	_G[editboxName.."Left"]:SetHeight(Textures.SearchBoxIcons.height)
	_G[editboxName.."Left"]:SetPoint("LEFT", 0, 0)
	_G[editboxName.."Left"]:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	_G[editboxName.."Left"]:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsLoff))

	_G[editboxName.."Right"]:SetWidth(5)
	_G[editboxName.."Right"]:SetHeight(Textures.SearchBoxIcons.height)
	_G[editboxName.."Right"]:SetPoint("RIGHT", 0, 0)
	_G[editboxName.."Right"]:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	_G[editboxName.."Right"]:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsRoff))

	_G[editboxName.."Middle"]:SetWidth(1)
	_G[editboxName.."Middle"]:SetHeight(Textures.SearchBoxIcons.height)
	_G[editboxName.."Middle"]:SetPoint("LEFT", _G[editboxName.."Left"], "RIGHT", 0, 0)
	_G[editboxName.."Middle"]:SetPoint("RIGHT", _G[editboxName.."Right"], "LEFT", 0, 0)
	_G[editboxName.."Middle"]:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	_G[editboxName.."Middle"]:SetTexCoord(unpack(Textures.SearchBoxIcons.coordsMoff))

	local editboxDisableTexture = editbox:CreateTexture(editboxName.."DisableTexture", "ARTWORK")
	editboxDisableTexture:SetPoint("TOPLEFT", 4, -4)
	editboxDisableTexture:SetPoint("BOTTOMRIGHT", -4, 4)
	editboxDisableTexture:SetTexture(0.6, 0.6, 0.6, 0.2)
	editboxDisableTexture:Hide()

	editbox:SetScript("OnShow",         function(self) self:SetAutoFocus(false) end)
	editbox:SetScript("OnEnterPressed", function(self) self:ClearFocus() end)
end
-- Template EditBox END ----------------------------------------

-- Template TabButton START ----------------------------------------
local function SetTabButton(name, showIt)
	if showIt then
		_G[name.."Bottom"]:SetTexture(0, 0, 0, 1)
		_G[name.."Border"]:SetTexture(unpack(configBorderColor))
	else
		_G[name.."Bottom"]:SetTexture(unpack(configBorderColor))
		_G[name.."Border"]:SetTexture(0.4, 0.4, 0.4, 0.4)
	end
end
local function DisableTabButton(name)
	_G[name.."Text"]:SetTextColor(0.2, 0.2, 0.2, 1)
	_G[name]:Disable()
end
local function EnableTabButton(name)
	_G[name.."Text"]:SetTextColor(1, 1, 1, 1)
	_G[name]:Enable()
end

local function TemplateTabButton(button, text, detail, buttonwidth)
	local buttonName = button:GetName()
	local texture = button:CreateTexture(nil, "BORDER")
	texture:SetPoint("TOPLEFT", 1, -1)
	texture:SetPoint("BOTTOMRIGHT", -1, 1)
	texture:SetTexture(0, 0, 0, 1)

	local textureBorder = button:CreateTexture(buttonName.."Border", "BACKGROUND")
	textureBorder:SetPoint("TOPLEFT", 0, 0)
	textureBorder:SetPoint("BOTTOMRIGHT", -1, 1)
	textureBorder:SetPoint("TOPRIGHT", 0, 0)
	textureBorder:SetPoint("BOTTOMLEFT" ,1, 1)
	textureBorder:SetTexture(0.4, 0.4, 0.4, 1)

	local textureBottom = button:CreateTexture(buttonName.."Bottom", "ARTWORK")
	textureBottom:SetPoint("TOPLEFT", button, "BOTTOMLEFT" ,1, 2)
	textureBottom:SetPoint("BOTTOMLEFT", 1, 1)
	textureBottom:SetPoint("TOPRIGHT", button, "BOTTOMRIGHT" ,-1, 2)
	textureBottom:SetPoint("BOTTOMRIGHT" ,-1, 1)
	textureBottom:SetTexture(0.4, 0.4, 0.4, 1)

	local textureHighlight = button:CreateTexture(nil, "ARTWORK")
	textureHighlight:SetPoint("TOPLEFT", 3, -3)
	textureHighlight:SetPoint("BOTTOMRIGHT", -3, 3)
	textureHighlight:SetTexture(1, 1, 1, 0.2)
	button:SetHighlightTexture(textureHighlight)

	local buttontext = button:CreateFontString(buttonName.."Text", "OVERLAY", "GameFontNormalSmall")
	buttontext:SetText(text)
	buttontext:SetWidth(buttonwidth)
	buttontext:SetHeight(12)
	buttontext:SetPoint("CENTER", button, "CENTER", 0, 0)
	buttontext:SetJustifyH("CENTER")

	if detail == 1 then
		buttontext:SetTextColor(1, 1, 1, 1)
	elseif detail == 2 then
		buttontext:SetTextColor(0, 0.75, 0, 1)
	elseif detail == 3 then
		buttontext:SetTextColor(1, 0, 0, 1)
	end
end
-- Template TabButton END ----------------------------------------

-- Template ScrollFrame START ----------------------------------------
local function TemplateButtonTexture(button, pos)
	local textureNormal = button:CreateTexture(nil, "ARTWORK")
	textureNormal:SetWidth(sizeGeneral.SliderSize)
	textureNormal:SetHeight(sizeGeneral.SliderSize)
	textureNormal:SetPoint("LEFT", 0, 0)
	textureNormal:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	textureNormal:SetTexCoord(unpack(Textures[pos].coords))
	button:SetNormalTexture(textureNormal)

	local texturePush = button:CreateTexture(nil, "ARTWORK")
	texturePush:SetWidth(sizeGeneral.SliderSize)
	texturePush:SetHeight(sizeGeneral.SliderSize)
	texturePush:SetPoint("TOPLEFT", 1, -1)
	texturePush:SetPoint("BOTTOMRIGHT", -1, 1)
	texturePush:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	texturePush:SetTexCoord(unpack(Textures[pos].coords))
	button:SetPushedTexture(texturePush)

	local textureHighlight = button:CreateTexture(nil, "ARTWORK")
	textureHighlight:SetWidth(sizeGeneral.SliderSize)
	textureHighlight:SetHeight(sizeGeneral.SliderSize)
	textureHighlight:SetPoint("LEFT", 0, 0)
	textureHighlight:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	textureHighlight:SetTexCoord(unpack(Textures[pos].coordsHL))
	button:SetHighlightTexture(textureHighlight)

	local textureDisable = button:CreateTexture(nil, "ARTWORK")
	textureDisable:SetWidth(sizeGeneral.SliderSize)
	textureDisable:SetHeight(sizeGeneral.SliderSize)
	textureDisable:SetPoint("LEFT", 0, 0)
	textureDisable:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	textureDisable:SetTexCoord(unpack(Textures[pos].coordsDIS))
	button:SetDisabledTexture(textureDisable)

	--local shaderSupported = textureDisable:SetDesaturated(true)
	--if not shaderSupported then
	--	textureDisable:SetVertexColor(0.5, 0.5, 0.5)
	--end
end

local function TemplateScrollFrame(frame, child)
	local name = frame:GetName()

	frame.offset = 0
	frame:EnableMouseWheel(true)

	local scrollBar = CreateFrame("Slider", name.."ScrollBar", frame)
	scrollBar:SetWidth(sizeGeneral.SliderSize)
	scrollBar:SetHeight(0)
	scrollBar:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -(sizeGeneral.Space+sizeGeneral.Border), -(sizeGeneral.Offset+sizeGeneral.SliderSize))
	scrollBar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -(sizeGeneral.Space+sizeGeneral.Border), sizeGeneral.Offset+sizeGeneral.SliderSize)
	scrollBar:SetMinMaxValues(0, 0)
	scrollBar:SetValue(0)
	scrollBar:SetOrientation("VERTICAL")
	scrollBar:EnableMouseWheel(true)
	scrollBar:SetScript("OnValueChanged", function(self, value)
		frame:SetVerticalScroll(value)
	end)
	scrollBar:SetScript("OnMouseWheel", function(self, delta)
		ScrollFrameTemplate_OnMouseWheel(frame, delta)
	end)

	local scrollBG = scrollBar:CreateTexture(name.."ScrollBarBackgroundTexture", "BACKGROUND")
	scrollBG:SetAllPoints(scrollBar)
	scrollBG:SetTexture(1, 1, 1, 0.05)

	local upButton = CreateFrame("Button",  name.."ScrollBarScrollUpButton", scrollBar)
	upButton:SetWidth(sizeGeneral.SliderSize)
	upButton:SetHeight(sizeGeneral.SliderSize)
	upButton:SetPoint("BOTTOM", scrollBar, "TOP", 0, 0)
	upButton:SetScript("OnClick", function()
		scrollBar:SetValue(scrollBar:GetValue() - (scrollBar:GetHeight() / 2))
		PlaySound("UChatScrollButton")
	end)
	TemplateButtonTexture(upButton, "ScrollFrameUp")

	local downButton = CreateFrame("Button",  name.."ScrollBarScrollDownButton", scrollBar)
	downButton:SetWidth(sizeGeneral.SliderSize)
	downButton:SetHeight(sizeGeneral.SliderSize)
	downButton:SetPoint("TOP", scrollBar, "BOTTOM", 0, 0)
	downButton:SetScript("OnClick", function()
		scrollBar:SetValue(scrollBar:GetValue() + (scrollBar:GetHeight() / 2))
		PlaySound("UChatScrollButton")
	end)
	TemplateButtonTexture(downButton, "ScrollFrameDown")

	local thumb = scrollBar:CreateTexture(name.."ScrollBarThumbTexture", "BORDER")
	thumb:SetWidth(sizeGeneral.SliderSize)
	thumb:SetHeight(sizeGeneral.SliderSize)
	thumb:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	thumb:SetTexCoord(unpack(Textures.ScrollFrameKnob.coords))
	scrollBar:SetThumbTexture(thumb)

	frame:SetScript("OnScrollRangeChanged", function(self, xrange, yrange)
		-- ScrollFrame_OnScrollRangeChanged(self, xrange, yrange)
		local scrollbar = _G[self:GetName().."ScrollBar"]
		if not yrange then
			yrange = self:GetVerticalScrollRange()
		end
		local value = scrollbar:GetValue()
		if value > yrange then
			value = yrange
		end
		scrollbar:SetMinMaxValues(0, yrange)
		scrollbar:SetValue(value)
		if floor(yrange) == 0 then
			_G[scrollbar:GetName().."ScrollDownButton"]:Disable()
			_G[scrollbar:GetName().."ScrollUpButton"]:Disable()
			_G[scrollbar:GetName().."ScrollDownButton"]:Show()
			_G[scrollbar:GetName().."ScrollUpButton"]:Show()
			--_G[scrollbar:GetName().."ThumbTexture"]:Hide()
		else
			_G[scrollbar:GetName().."ScrollDownButton"]:Show()
			_G[scrollbar:GetName().."ScrollUpButton"]:Show()
			_G[self:GetName().."ScrollBar"]:Show()
			--_G[scrollbar:GetName().."ThumbTexture"]:Show()
			-- The 0.005 is to account for precision errors
			if yrange - value > 0.005 then
				_G[scrollbar:GetName().."ScrollDownButton"]:Enable()
			else
				_G[scrollbar:GetName().."ScrollDownButton"]:Disable()
			end
		end
	end)
	frame:SetScript("OnMouseWheel", function(self, delta)
		ScrollFrameTemplate_OnMouseWheel(self, delta)
	end)
	frame:SetScript("OnVerticalScroll", function(self, off)
		scrollBar:SetValue(off)
	end)

	if child then
		local childFrame = CreateFrame("Frame",  name.."ScrollChildFrame", frame)
		childFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
		childFrame:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 0, 0)
		--childFrame:SetHeight(0)
		--childFrame:SetWidth(0)
		frame:SetScrollChild(childFrame)
	end
end
-- Template ScrollFrame END ----------------------------------------

-- Template PullDownMenu START ----------------------------------------
local function DisablePullDownMenu(name)
	_G[name.."_Border"]:SetTexture(0.4, 0.4, 0.4, 1)
	_G[name]:Disable()
end
local function EnablePullDownMenu(name)
	_G[name.."_Border"]:SetTexture(0.8, 0.2, 0.2, 1)
	_G[name]:Enable()
end

local function TemplatePullDownMenu(button, buttonText, pulldownWidth, contentTable, contentTexture, func)
	local buttonName = button:GetName()
	local maxNum = #contentTable
	if maxNum < 10 then
		maxNum = 10
	end
	--print("TemplatePullDownMenu", maxNum) -- TEST
	local textureBack = button:CreateTexture(nil, "BORDER")
	textureBack:SetPoint("TOPLEFT", 1, -1)
	textureBack:SetPoint("BOTTOMRIGHT", -1, 1)
	textureBack:SetTexture(0, 0, 0, 1)

	local textureBorder = button:CreateTexture(buttonName.."_Border", "BACKGROUND")
	textureBorder:SetPoint("TOPLEFT", 0, 0)
	textureBorder:SetPoint("BOTTOMRIGHT", 0, 0)
	textureBorder:SetTexture(0.4, 0.4, 0.4, 1)

	local textureNormal = button:CreateTexture(nil, "ARTWORK")
	textureNormal:SetPoint("TOPLEFT", 2, -2)
	textureNormal:SetPoint("BOTTOMRIGHT", -2, 2)
	textureNormal:SetTexture(0, 0, 0, 1)

	local buttontexture = button:CreateTexture(nil, "ARTWORK")
	buttontexture:SetHeight(sizeGeneral.BarHeight)
	buttontexture:SetWidth(sizeGeneral.BarHeight)
	buttontexture:SetPoint("RIGHT", button, "RIGHT", -(sizeGeneral.Space+sizeGeneral.Border), 0)
	buttontexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	buttontexture:SetTexCoord(unpack(Textures.Expand.coords))
	button:SetNormalTexture(buttontexture)

	local textureDisabled = button:CreateTexture(nil, "OVERLAY")
	textureDisabled:SetPoint("TOPLEFT", 3, -3)
	textureDisabled:SetPoint("BOTTOMRIGHT", -3, 3)
	textureDisabled:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetDisabledTexture(textureDisabled)

	local textureHighlight = button:CreateTexture(nil, "OVERLAY")
	textureHighlight:SetPoint("TOPLEFT", 3, -3)
	textureHighlight:SetPoint("BOTTOMRIGHT", -3, 3)
	textureHighlight:SetTexture(0.6, 0.6, 0.6, 0.2)
	button:SetHighlightTexture(textureHighlight)

	if contentTexture then
		if contentTexture == 1 then
			local texture = button:CreateTexture(buttonName.."_PullDownMenuTexture", "ARTWORK")
			texture:SetWidth(sizeGeneral.BarHeight)
			texture:SetHeight(sizeGeneral.BarHeight)
			texture:SetPoint("LEFT", sizeGeneral.Offset+sizeGeneral.Space, 0)
		elseif contentTexture == 2 then
			local texture = button:CreateTexture(buttonName.."_PullDownMenuTexture", "ARTWORK")
			texture:SetWidth(sizeGeneral.BarHeight)
			texture:SetHeight(sizeGeneral.BarHeight)
			texture:SetPoint("LEFT", sizeGeneral.Offset+sizeGeneral.Space, 0)
			local text = button:CreateFontString(buttonName.."_PullDownMenuText", "ARTWORK", "GameFontWhiteSmall")
			text:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.BarHeight - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
			text:SetHeight(sizeGeneral.BarHeight)
			text:SetPoint("LEFT", sizeGeneral.Offset+sizeGeneral.Space+sizeGeneral.BarHeight+sizeGeneral.Space, 0)
			text:SetJustifyH("LEFT")
			text:SetText(buttonText)
		end
	else
		local text = button:CreateFontString(buttonName.."_PullDownMenuText", "ARTWORK", "GameFontWhiteSmall")
		text:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
		text:SetHeight(sizeGeneral.BarHeight)
		text:SetPoint("LEFT", sizeGeneral.Offset+sizeGeneral.Space, 0)
		text:SetJustifyH("LEFT")
		text:SetText(buttonText)
	end

	local scrollFrameContainer = CreateFrame("Frame", buttonName.."_PullDownMenu", button)
	TemplateBorderTRBL(scrollFrameContainer, 1)
	scrollFrameContainer:EnableMouse(true)
	scrollFrameContainer:SetToplevel(true)
	scrollFrameContainer:SetWidth(pulldownWidth)
	scrollFrameContainer:SetHeight(sizeGeneral.Offset+(10*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	scrollFrameContainer:SetPoint("TOPLEFT", button, "BOTTOMLEFT", 0, 1)
	scrollFrameContainer:Hide()

	local scrollFrame = CreateFrame("ScrollFrame", buttonName.."_PullDownMenuScrollFrame", scrollFrameContainer)
	TemplateScrollFrame(scrollFrame)
	scrollFrame:SetPoint("TOPLEFT", 0, 0)
	scrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	scrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll(buttonName.."_PullDownMenu", value) end)

	local function OnLeave()
		if not button:IsMouseOver() and not scrollFrameContainer:IsMouseOver() then
			scrollFrameContainer:Hide()
			buttontexture:SetTexCoord(unpack(Textures.Expand.coords))
		end
	end

	for i = 1, maxNum do
		local pdbutton = CreateFrame("Button", buttonName.."_PullDownMenuScrollFrame_Button"..i, scrollFrameContainer)
		pdbutton:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
		pdbutton:SetHeight(sizeGeneral.BarHeight)
		pdbutton:SetFrameLevel( scrollFrame:GetFrameLevel() + 5 )
		pdbutton:Hide()
		if i == 1 then
			pdbutton:SetPoint("TOPLEFT", buttonName.."_PullDownMenuScrollFrame", "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			pdbutton:SetPoint("TOPLEFT", buttonName.."_PullDownMenuScrollFrame_Button"..(i-1), "BOTTOMLEFT", 0, 0)
		end

		if contentTexture then
			if contentTexture == 1 then
				local texture = pdbutton:CreateTexture(buttonName.."_PullDownMenuScrollFrame_Button"..i.."Texture", "ARTWORK")
				texture:SetWidth(sizeGeneral.BarHeight)
				texture:SetHeight(sizeGeneral.BarHeight)
				texture:SetPoint("LEFT", sizeGeneral.Space, 0)
			elseif contentTexture == 2 then
				local texture = pdbutton:CreateTexture(buttonName.."_PullDownMenuScrollFrame_Button"..i.."Texture", "ARTWORK")
				texture:SetWidth(sizeGeneral.BarHeight)
				texture:SetHeight(sizeGeneral.BarHeight)
				texture:SetPoint("LEFT", sizeGeneral.Space, 0)
				local text = pdbutton:CreateFontString(buttonName.."_PullDownMenuScrollFrame_Button"..i.."Text", "ARTWORK", "GameFontWhiteSmall")
				text:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
				text:SetHeight(sizeGeneral.BarHeight)
				text:SetPoint("LEFT", sizeGeneral.Space+sizeGeneral.BarHeight+sizeGeneral.Space, 0)
				text:SetJustifyH("LEFT")
			end
		else
			local text = pdbutton:CreateFontString(buttonName.."_PullDownMenuScrollFrame_Button"..i.."Text", "ARTWORK", "GameFontWhiteSmall")
			text:SetWidth(pulldownWidth-sizeGeneral.Offset - sizeGeneral.Space-sizeGeneral.SliderSize-sizeGeneral.Space-sizeGeneral.Border)
			text:SetHeight(sizeGeneral.BarHeight)
			text:SetPoint("LEFT", sizeGeneral.Space, 0)
			text:SetJustifyH("LEFT")
		end

		local selected = pdbutton:CreateTexture(buttonName.."_PullDownMenuScrollFrame_Button"..i.."Selected", "ARTWORK")
		selected:SetPoint("TOPLEFT", pdbutton, "TOPLEFT", 0, 0)
		selected:SetPoint("BOTTOMRIGHT", pdbutton, "BOTTOMRIGHT", 0, 0)
		selected:SetTexture(1, 1, 1, 0.1)
		selected:Hide()

		pdbutton:SetScript("OnLeave", OnLeave)
		pdbutton:SetScript("OnClick", function()
			if contentTexture then
				if contentTexture == 1 then
					button.value = pdbutton.value
					_G[buttonName.."_PullDownMenuTexture"]:SetTexture( categoryIcons[button.value].texture )
					_G[buttonName.."_PullDownMenuTexture"]:SetTexCoord(unpack(categoryIcons[button.value].coords))
					scrollFrameContainer:Hide()
					buttontexture:SetTexCoord(unpack(Textures.Expand.coords))
				elseif contentTexture == 2 then
					button.value = pdbutton.value
					_G[buttonName.."_PullDownMenuTexture"]:SetTexture( categoryIcons[ GUI_CatFrame_data[button.value].Texture ].texture )
					_G[buttonName.."_PullDownMenuTexture"]:SetTexCoord(unpack(categoryIcons[ GUI_CatFrame_data[button.value].Texture ].coords))
					_G[buttonName.."_PullDownMenuText"]:SetText( GUI_CatFrame_data[ button.value ].CatColor..GUI_CatFrame_data[ button.value ].Cat.."|r" )
					scrollFrameContainer:Hide()
					buttontexture:SetTexCoord(unpack(Textures.Expand.coords))
				end
			else
				button.value = pdbutton.value
				_G[buttonName.."_PullDownMenuText"]:SetText( _G[buttonName.."_PullDownMenuScrollFrame_Button"..i.."Text"]:GetText() )
				scrollFrameContainer:Hide()
				buttontexture:SetTexCoord(unpack(Textures.Expand.coords))
			end
			if func then
				func(button.value)
			end
		end)

		local highlight = pdbutton:CreateTexture(nil, "ARTWORK")
		highlight:SetPoint("TOPLEFT", pdbutton, "TOPLEFT", 0, 0)
		highlight:SetPoint("BOTTOMRIGHT", pdbutton, "BOTTOMRIGHT", 0, 0)
		highlight:SetTexture(1, 1, 1, 0.2)
		pdbutton:SetHighlightTexture(highlight)
	end
	AHsearch:FillList(buttonName.."_PullDownMenu")

	_G[buttonName.."_PullDownMenu"]:SetScript("OnLeave", OnLeave)
	_G[buttonName.."_PullDownMenu"]:SetScript("OnHide", function(self) self:Hide() end) -- for esc close
	_G[buttonName.."_PullDownMenuScrollFrame"]:SetScript("OnLeave", OnLeave)
	_G[buttonName.."_PullDownMenuScrollFrameScrollBar"]:SetScript("OnLeave", OnLeave)
	_G[buttonName.."_PullDownMenuScrollFrameScrollBarScrollUpButton"]:SetScript("OnLeave", OnLeave)
	_G[buttonName.."_PullDownMenuScrollFrameScrollBarScrollDownButton"]:SetScript("OnLeave", OnLeave)

	button:SetScript("OnLeave", OnLeave)
	button:SetScript("OnClick", function()
		if scrollFrameContainer:IsShown() then
			scrollFrameContainer:Hide()
			buttontexture:SetTexCoord(unpack(Textures.Expand.coords))
		else
			scrollFrameContainer:Show()
			buttontexture:SetTexCoord(unpack(Textures.Collapse.coords))
		end
	end)
end
-- Template PullDownMenu END ----------------------------------------

-- Template CheckButton START ----------------------------------------
local function DisableCheckButton(name, texture)
	if texture then
		_G[name.."Texture"]:SetAlpha(0.2)
	else
		_G[name.."Text"]:SetTextColor(0.5, 0.5, 0.5)
	end
	_G[name.."Border"]:SetTexture(0.4, 0.4, 0.4, 1)
	_G[name]:Disable()
end
local function EnableCheckButton(name, texture)
	if texture then
		_G[name.."Texture"]:SetAlpha(1)
	else
		_G[name.."Text"]:SetTextColor(1, 1, 1)
	end
	_G[name.."Border"]:SetTexture(0.8, 0.2, 0.2, 1)
	_G[name]:Enable()
end

local function TemplateCheckButton(button, size, space, text, texture)
	local buttonName = button:GetName()
	local textureBorder = button:CreateTexture(buttonName.."Border", "BACKGROUND")
	textureBorder:SetWidth( size )
	textureBorder:SetHeight( size )
	textureBorder:SetPoint("LEFT", 0, 0)
	textureBorder:SetTexture(0.4, 0.4, 0.4, 1)

	local textureBackground = button:CreateTexture(nil, "BORDER")
	textureBackground:SetPoint("TOPLEFT", textureBorder, "TOPLEFT", 1, -1)
	textureBackground:SetPoint("BOTTOMRIGHT", textureBorder, "BOTTOMRIGHT", -1, 1)
	textureBackground:SetTexture(0, 0, 0, 1)

	local textureNormal = button:CreateTexture(nil, "ARTWORK")
	textureNormal:SetPoint("TOPLEFT", textureBorder, "TOPLEFT", 1, -1)
	textureNormal:SetPoint("BOTTOMRIGHT", textureBorder, "BOTTOMRIGHT", -1, 1)
	textureNormal:SetTexture(0, 0, 0, 1)
	button:SetNormalTexture(textureNormal)

	local texturePush = button:CreateTexture(nil, "ARTWORK")
	texturePush:SetPoint("TOPLEFT", textureBorder, "TOPLEFT", 4, -4)
	texturePush:SetPoint("BOTTOMRIGHT", textureBorder, "BOTTOMRIGHT", -4, 4)
	texturePush:SetTexture(0.4, 0.4, 0.4, 0.5)
	button:SetPushedTexture(texturePush)

	local textureDisabled = button:CreateTexture(nil, "ARTWORK")
	textureDisabled:SetPoint("TOPLEFT", textureBorder, "TOPLEFT", 3, -3)
	textureDisabled:SetPoint("BOTTOMRIGHT", textureBorder, "BOTTOMRIGHT", -3, 3)
	textureDisabled:SetTexture(0.4, 0.4, 0.4, 0.5)
	button:SetDisabledTexture(textureDisabled)

	local textureChecked = button:CreateTexture(nil, "ARTWORK")
	textureChecked:SetWidth( size )
	textureChecked:SetHeight( size )
	textureChecked:SetPoint("LEFT", 0, 0)
	textureChecked:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
	button:SetCheckedTexture(textureChecked)

	if texture then
		if texture == 1 then
			local textureIcon = button:CreateTexture(buttonName.."Texture", "OVERLAY")
			textureIcon:SetWidth(7)
			textureIcon:SetHeight(8)
			textureIcon:SetPoint("LEFT", textureNormal, "RIGHT", space, 0)
			textureIcon:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
			textureIcon:SetTexCoord(unpack(Textures.SoryOrderIcon.coordsAsc))

			button:SetWidth(size + space + 7 + space)
			button:SetHeight(size)
		elseif texture == 2 then
			local textureIcon = button:CreateTexture(buttonName.."Texture", "OVERLAY")
			textureIcon:SetWidth(7)
			textureIcon:SetHeight(8)
			textureIcon:SetPoint("LEFT", textureNormal, "RIGHT", space, 0)
			textureIcon:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
			textureIcon:SetTexCoord(unpack(Textures.SoryOrderIcon.coordsDesc))

			button:SetWidth(size + space + 7 + space)
			button:SetHeight(size)
		end
	else
		local buttontext = button:CreateFontString(buttonName.."Text", "OVERLAY", "GameFontWhiteSmall")
		buttontext:SetHeight( size )
		buttontext:SetPoint("LEFT", textureNormal, "RIGHT", space, 0)
		buttontext:SetJustifyH("LEFT")
		buttontext:SetText(text)

		button:SetWidth(size + space + buttontext:GetStringWidth() + space)
		button:SetHeight(size)
	end
	local textureHighlight = button:CreateTexture(nil, "OVERLAY")
	textureHighlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
	textureHighlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
	textureHighlight:SetTexture(1, 1, 1, 0.1)
	textureHighlight:Hide()

	button:SetScript("OnEnter", function() textureHighlight:Show() end)
	button:SetScript("OnLeave", function() textureHighlight:Hide() end)
end
-- Template CheckButton END ----------------------------------------

-- Template Slider START ----------------------------------------
local function TemplateSlider(slider, width, step, minVal, maxVal, curVal, func, pro)
	local sliderName = slider:GetName()

	slider:SetWidth(width)
	slider:SetHeight(sizeGeneral.SliderSize)
	slider:SetValueStep(step) 
	slider:SetMinMaxValues(minVal, maxVal)
	slider:SetValue(curVal)
	slider:SetOrientation("HORIZONTAL")

	local textMin = slider:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	textMin:SetPoint("TOP", slider, "BOTTOM", 0, -2)
	textMin:SetPoint("LEFT", slider, "LEFT", 0, 0)
	textMin:SetJustifyH("CENTER")
	if pro then
		textMin:SetText(minVal.."%")
	else
		textMin:SetText(minVal)
	end
	local textMax = slider:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	textMax:SetPoint("TOP", slider, "BOTTOM", 0, -2)
	textMax:SetPoint("RIGHT", slider, "RIGHT", 0, 0)
	textMax:SetJustifyH("CENTER")
	if pro then
		textMax:SetText(maxVal.."%")
	else
		textMax:SetText(maxVal)
	end
	local textCur = slider:CreateFontString(sliderName.."CurText", "ARTWORK", "GameFontWhiteSmall")
	textCur:SetPoint("TOP", slider, "BOTTOM", 0, -2)
	textCur:SetPoint("CENTER", slider, "CENTER", 0, 0)
	textCur:SetJustifyH("CENTER")
	if pro then
		textCur:SetText(curVal.."%")
	else
		textCur:SetText(curVal)
	end

	local sliderBGL = slider:CreateTexture(nil, "BACKGROUND")
	sliderBGL:SetWidth(5)
	sliderBGL:SetHeight(6)
	sliderBGL:SetPoint("LEFT", slider, "LEFT", 0, 0)
	sliderBGL:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	sliderBGL:SetTexCoord(unpack(Textures.SliderBG.coordsL))
	local sliderBGM = slider:CreateTexture(nil, "BACKGROUND")
	sliderBGM:SetWidth(width-5-5)
	sliderBGM:SetHeight(6)
	sliderBGM:SetPoint("LEFT", sliderBGL, "RIGHT", 0, 0)
	sliderBGM:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	sliderBGM:SetTexCoord(unpack(Textures.SliderBG.coordsM))
	local sliderBGR = slider:CreateTexture(nil, "BACKGROUND")
	sliderBGR:SetWidth(5)
	sliderBGR:SetHeight(6)
	sliderBGR:SetPoint("LEFT", sliderBGM, "RIGHT", 0, 0)
	sliderBGR:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	sliderBGR:SetTexCoord(unpack(Textures.SliderBG.coordsR))

	local thumb = slider:CreateTexture(nil, "BORDER")
	thumb:SetWidth(sizeGeneral.SliderSize)
	thumb:SetHeight(sizeGeneral.SliderSize)
	thumb:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	thumb:SetTexCoord(unpack(Textures.SliderKnob.coords))
	slider:SetThumbTexture(thumb)

	slider:SetScript("OnValueChanged", function(self, value)
		if pro then
			textCur:SetText(value.."%")
		else
			textCur:SetText(value)
		end
		if func then
			func(self, value)
		end
	end)
end
-- Template Slider END ----------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function Print(...)
	print("|cffffff7fAHsearch:|r", ...)
end

local function NOOP() end

local function SortedPairs(t, lvl)
	local ntab = {}
	for k,v in pairs(t) do
		table_insert(ntab,k)
	end

	if lvl then
		table_sort(ntab,
			function(a,b)
				if t[a].order == t[b].order then
					if t[a].qualityIndex == t[b].qualityIndex then
						if a < b then
							return true
						end
					end
					if t[a].qualityIndex > t[b].qualityIndex then
						return true
					end	
				end
				if t[a].order < t[b].order then
					return true
				end
			end
		)
	else
		table_sort(ntab,
			function(a,b)
				if t[a].order == t[b].order then
					if a < b then
						return true
					end
				end
				if t[a].order < t[b].order then
					return true
				end
			end
		)
	end

	local i = 0
	local function _f(_s,_v)
		i = i + 1
		local k = ntab[i]
		if (k) then
			return k,t[k]
		end
	end
	return _f,nil,nil
end

local function SearchQueryGlobalCheck()
	for cat in pairs(AHsearch_Data) do
		if AHsearch_Data[cat].queries then
			for sq in pairs(AHsearch_Data[cat].queries) do
				return true
			end
		end
	end
	return false
end

local function SearchQueryCheck(cat)
	if AHsearch_Data[cat].queries then
		for sq in pairs(AHsearch_Data[cat].queries) do
			return false
		end
	end
	return true
end

local function VisibilityCheck(newduration)
	if configMode then
		AHsearch.FrameOnUpdate:SetScript("OnUpdate", nil)
		return
	end

	if (not AHsearch_QueryFrame:IsMouseOver() or not AHsearch_QueryFrame:IsShown()) and not AHsearch_CatFrame:IsMouseOver() then
		FrameTime = GetTime()

		local duration = 0.75
		if newduration and type(newduration) == "number" then
			duration = newduration
		end

		AHsearch.FrameOnUpdate:SetScript("OnUpdate", function()
			if configMode then
				AHsearch.FrameOnUpdate:SetScript("OnUpdate", nil)
				return
			end
			if GetTime() - FrameTime > duration then
				if (not AHsearch_QueryFrame:IsMouseOver() or not AHsearch_QueryFrame:IsShown()) and not AHsearch_CatFrame:IsMouseOver() then
					AHsearch_CatFrame:Hide()
					AHsearch.FrameOnUpdate:SetScript("OnUpdate", nil)
				else
					AHsearch.FrameOnUpdate:SetScript("OnUpdate", nil)
				end
			end
		end)

	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- Categories settingcheck START
local function ConfigurationFrame_Categories_SetNewCat(text)
	if AHsearch_ConfigurationFrame_Categories.editType == "new" then
		if text ~= "" then
			AHsearch:ConfigurationFrame_Categories_CreateNewCat()
		end
	end
end

local function ConfigurationFrame_Categories_SetCheck()
	if AHsearch_ConfigurationFrame_Categories.editType ~= "selected" then return end

	if AHsearch_ConfigurationFrame_Categories_CurrentCatText.value and AHsearch_ConfigurationFrame_Categories_CurrentCatText.value ~= "" and
	   (
	   AHsearch_ConfigurationFrame_Categories.originalCat ~= AHsearch_ConfigurationFrame_Categories_CurrentCatText.value or
	   AHsearch_ConfigurationFrame_Categories.originalOrder ~= AHsearch_ConfigurationFrame_Categories_Order.value or
	   AHsearch_ConfigurationFrame_Categories.originalColor ~= AHsearch_ConfigurationFrame_Categories_Color.value or
	   AHsearch_ConfigurationFrame_Categories.originalIcon ~= AHsearch_ConfigurationFrame_Categories_Icon.value
	   )
	then
		AHsearch_ConfigurationFrame_Categories_UpdateCat:SetText(L["Update"])
		EnableTextButton("AHsearch_ConfigurationFrame_Categories_UpdateCat", 1)
	else
		AHsearch_ConfigurationFrame_Categories_UpdateCat:SetText(L["Update"])
		DisableTextButton("AHsearch_ConfigurationFrame_Categories_UpdateCat")
	end
end

local function ConfigurationFrame_Categories_SetName(text)
	local color = AHsearch_ConfigurationFrame_Categories_Color.value
	if not color then
		color = "|cffffffff"
		AHsearch_ConfigurationFrame_Categories_Color.value = color
	end
	AHsearch_ConfigurationFrame_Categories_CurrentCatText.value = text
	AHsearch_ConfigurationFrame_Categories_CurrentCatText:SetText(color..text.."|r")

	if AHsearch_ConfigurationFrame_Categories.editType == "new" then return end
	ConfigurationFrame_Categories_SetCheck()
end

local function ConfigurationFrame_Categories_SetOrder()
	ConfigurationFrame_Categories_SetCheck()
end

local function ConfigurationFrame_Categories_SetColor(val)
	AHsearch_ConfigurationFrame_Categories_CurrentCatText:SetText(val..AHsearch_ConfigurationFrame_Categories_CurrentCatText.value)
	ConfigurationFrame_Categories_SetCheck()
end

local function ConfigurationFrame_Categories_SetIcon(val)
	AHsearch_ConfigurationFrame_Categories_CurrentCatTexture:SetTexture(categoryIcons[val].texture)
	AHsearch_ConfigurationFrame_Categories_CurrentCatTexture:SetTexCoord(unpack(categoryIcons[val].coords))
	ConfigurationFrame_Categories_SetCheck()
end
-- Categories settingcheck END

-- SearchQueries settingcheck START
local function ConfigurationFrame_SearchQueries_SetCheck()
	if AHsearch_ConfigurationFrame_SearchQueries.editType ~= "selected" then return end
	if AHsearch_ConfigurationFrame_SearchQueries.currentSQtitle ~= "" and
	  (
	  AHsearch_ConfigurationFrame_SearchQueries.originalSQtitle ~= AHsearch_ConfigurationFrame_SearchQueries.currentSQtitle or
	  AHsearch_ConfigurationFrame_SearchQueries.originalNameQuerySQ ~= AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value or
	  AHsearch_ConfigurationFrame_SearchQueries.originalOrder ~= AHsearch_ConfigurationFrame_SearchQueries_Order.value or
	  AHsearch_ConfigurationFrame_SearchQueries.originalRarity ~= AHsearch_ConfigurationFrame_SearchQueries_Rarity.value or

	  (
	   (
	    AHsearch_ConfigurationFrame_SearchQueries.originalFilter1 ~= AHsearch_ConfigurationFrame_SearchQueries_Filter1.value and
	    AHsearch_ConfigurationFrame_SearchQueries.originalFilter1 and AHsearch_ConfigurationFrame_SearchQueries.originalFilter1 > 0
	   )
	  or
	   (
	    AHsearch_ConfigurationFrame_SearchQueries.originalFilter1 ~= AHsearch_ConfigurationFrame_SearchQueries_Filter1.value and
	    AHsearch_ConfigurationFrame_SearchQueries_Filter1.value and AHsearch_ConfigurationFrame_SearchQueries_Filter1.value > 0
	   )
	  ) or

	  (
	   (
	    AHsearch_ConfigurationFrame_SearchQueries.originalFilter2 ~= AHsearch_ConfigurationFrame_SearchQueries_Filter2.value and
	    AHsearch_ConfigurationFrame_SearchQueries.originalFilter2 and AHsearch_ConfigurationFrame_SearchQueries.originalFilter2 > 0
	   )
	  or
	   (
	    AHsearch_ConfigurationFrame_SearchQueries.originalFilter2 ~= AHsearch_ConfigurationFrame_SearchQueries_Filter2.value and
	    AHsearch_ConfigurationFrame_SearchQueries_Filter2.value and AHsearch_ConfigurationFrame_SearchQueries_Filter2.value > 0
	   )
	  ) or

	  (
	   (
	    AHsearch_ConfigurationFrame_SearchQueries.originalFilter3 ~= AHsearch_ConfigurationFrame_SearchQueries_Filter3.value and
	    AHsearch_ConfigurationFrame_SearchQueries.originalFilter3 and AHsearch_ConfigurationFrame_SearchQueries.originalFilter3 > 0
	   )
	  or
	   (
	    AHsearch_ConfigurationFrame_SearchQueries.originalFilter3 ~= AHsearch_ConfigurationFrame_SearchQueries_Filter3.value and
	    AHsearch_ConfigurationFrame_SearchQueries_Filter3.value and AHsearch_ConfigurationFrame_SearchQueries_Filter3.value > 0
	   )
	  ) or

	  AHsearch_ConfigurationFrame_SearchQueries.originalLevelMin ~= AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value or
	  AHsearch_ConfigurationFrame_SearchQueries.originalLevelMax ~= AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value or

	  AHsearch_ConfigurationFrame_SearchQueries.originalSortBy ~= AHsearch_ConfigurationFrame_SearchQueries_SortBy.value or
	  AHsearch_ConfigurationFrame_SearchQueries.originalSortOrderAsc ~= AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value or

	  AHsearch_ConfigurationFrame_SearchQueries.originalUsable ~= AHsearch_ConfigurationFrame_SearchQueries_Usable.value or
	  AHsearch_ConfigurationFrame_SearchQueries.originalDisplayOnChar ~= AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value
	  )
	then
		if AHsearch_ConfigurationFrame_SearchQueries_Move.value then
			if AHsearch_ConfigurationFrame_SearchQueries_Move.value == AHsearch_ConfigurationFrame_SearchQueries_SelectCat.cat then
				AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Update"])
				EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ", 1)
			else
				AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Update"].." & "..L["Move"])
				EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ", 1)
			end
		else
			AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Update"])
			EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ", 1)
		end
	else
		if AHsearch_ConfigurationFrame_SearchQueries_Move.value then
			if AHsearch_ConfigurationFrame_SearchQueries_Move.value == AHsearch_ConfigurationFrame_SearchQueries_SelectCat.cat then
				AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Update"])
				DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ")
			else
				if AHsearch_ConfigurationFrame_SearchQueries.currentSQtitle ~= "" then
					AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Move"])
					EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ", 1)
				else
					AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Move"])
					DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ")
				end
			end
		else
			AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Update"])
			DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ")
		end
	end
end

local function ConfigurationFrame_SearchQueries_SetTitleSQ(text)
	AHsearch_ConfigurationFrame_SearchQueries.currentSQtitle = text
	ConfigurationFrame_SearchQueries_SetCheck()
end

local function ConfigurationFrame_SearchQueries_SetFilter1(val)
	curFilter1 = val
	curFilter2 = nil
	AHsearch_ConfigurationFrame_SearchQueries_Filter2.value = nil
	curFilter3 = nil
	AHsearch_ConfigurationFrame_SearchQueries_Filter3.value = nil
	AHsearch:UpdateFilter()
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Filter1_PullDownMenu")
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Filter2_PullDownMenu")
	ConfigurationFrame_SearchQueries_SetCheck()
end

local function ConfigurationFrame_SearchQueries_SetFilter2(val)
	curFilter2 = val
	curFilter3 = nil
	AHsearch_ConfigurationFrame_SearchQueries_Filter3.value = nil
	AHsearch:UpdateFilter()
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Filter1_PullDownMenu")
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Filter2_PullDownMenu")
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Filter3_PullDownMenu")
	ConfigurationFrame_SearchQueries_SetCheck()
end

local function ConfigurationFrame_SearchQueries_SetFilter3()
	ConfigurationFrame_SearchQueries_SetCheck()
end

local function ConfigurationFrame_SearchQueries_SetMove(val)
	AHsearch_ConfigurationFrame_SearchQueries_Move.value = GUI_CatFrame_data[ val ].Cat
	if AHsearch_ConfigurationFrame_SearchQueries_Move.value == AHsearch_ConfigurationFrame_SearchQueries_SelectCat.cat then
		AHsearch_ConfigurationFrame_SearchQueries_MoveAlertIcon:Hide()
	else
		AHsearch_ConfigurationFrame_SearchQueries_MoveAlertIcon:Show()
	end
	ConfigurationFrame_SearchQueries_SetCheck()
end
-- SearchQueries settingcheck END
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:InitOptions()
	-- VERSION CHECK START --------------------
	-- new user START
	if AHsearch_Options.version == nil then
		AHsearch_Options.version = 4
	end
	-- new user END

	-- very very old user START
	if AHsearch_Options.version <= 3 then
		if AHsearch_Options then table.wipe(AHsearch_Options) end
		if AHsearch_Data then table.wipe(AHsearch_Data) end
		if AHsearchDB then table.wipe(AHsearchDB) end
		Print("Sorry, your AHsearch version is <= 3 (26 Sep 2008) and is no longer supported.")
		Print("All data and options has been deleted!")
		AHsearch_Options.version = 4
	end
	-- very very old user END

	-- toc30000+ user START
	if AHsearch_Options.version == 3.1 then
		AHsearch_Options.chatOutput = nil
		AHsearch_Options.WotLKfix = nil
		AHsearch_Options.noWotLKmessage = nil
		AHsearch_Options.ttDisplay = nil
		AHsearch_Options.ttFiltersDisplay = nil
		if AHsearch_Options.cycleButtonsNextPrev then AHsearch_Options.cycleNP  = AHsearch_Options.cycleButtonsNextPrev end
		if AHsearch_Options.cycleButtonsRandom   then AHsearch_Options.cycleRND = AHsearch_Options.cycleButtonsRandom   end
		AHsearch_Options.cycleButtonsNextPrev = nil
		AHsearch_Options.cycleButtonsRandom = nil
		DBupdate()
		AHsearch_Options.version = 4
	end
	-- toc30000+ user END
	-- VERSION CHECK END --------------------
	
	if AHsearch_Options.curCat        == nil then AHsearch_Options.curCat        = default_curCat        end -- = 25 rows
	if AHsearch_Options.curCatWidth   == nil then AHsearch_Options.curCatWidth   = default_curCatWidth   end -- = 175 px
	if AHsearch_Options.curQuery      == nil then AHsearch_Options.curQuery      = default_curQuery      end -- = 25 rows
	if AHsearch_Options.curQueryWidth == nil then AHsearch_Options.curQueryWidth = default_curQueryWidth end -- = 175 px
	if AHsearch_Options.cycleNP       == nil then AHsearch_Options.cycleNP       = default_cycleNP       end -- = false
	if AHsearch_Options.cycleRND      == nil then AHsearch_Options.cycleRND      = default_cycleRND      end -- = false
	if AHsearch_Options.scaleMain     == nil then AHsearch_Options.scaleMain     = default_scaleMain     end -- = 100%
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:AHsearchFrame_Create()
	AHsearch.FrameOnUpdate = CreateFrame("Button", nil, UIParent)

	local frame = CreateFrame("Frame", "AHsearchFrame", AuctionFrameBrowse)
	frame:SetFrameLevel( AuctionFrameBrowse:GetFrameLevel() + 20 )
	frame:SetWidth(20)
	frame:SetHeight(20)
	frame:SetToplevel(true)
	frame:ClearAllPoints()
	frame:SetPoint("RIGHT", "BrowseName", "RIGHT", 0, 0)

	-- AHsearch Button START
	local button = CreateFrame("Button", "AHsearchButton", AHsearchFrame)
	button:SetFrameLevel( AHsearchFrame:GetFrameLevel() + 10 )
	button:SetWidth(20)
	button:SetHeight(20)
	button:ClearAllPoints()
	button:SetPoint("TOPLEFT", "AHsearchFrame", 0, 0)
	button:RegisterForClicks("AnyUp")
	button:SetScript("OnClick", function(self, button)
		if button == "LeftButton" then
			if configMode then return end
			AHsearch:CatFrame_Show()
			if AHsearch_CatFrame:IsShown() then
				if GUI_CatFrame_Count == 0 then
					VisibilityCheck(4)
				else
					VisibilityCheck(2)
				end
			else
				AHsearch.FrameOnUpdate:SetScript("OnUpdate", nil)
			end
		elseif button == "RightButton" then
			if configMode then
				AHsearch:ConfigDisable()
			else
				AHsearch:ConfigEnable()
			end
		end
	end)
	button:EnableMouseWheel(true)
	button:SetScript("OnMouseWheel", NOOP)
	
	local normalTexture = button:CreateTexture(nil, "BACKGROUND")
	normalTexture:SetAllPoints(button)
	normalTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	normalTexture:SetTexCoord(unpack(Textures.AHsBmain.coords))
	button:SetNormalTexture(normalTexture)

	local highlightTexture = button:CreateTexture(nil, "BACKGROUND")
	highlightTexture:SetAllPoints(button)
	highlightTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	highlightTexture:SetTexCoord(unpack(Textures.AHsBmain.coordsHL))
	button:SetHighlightTexture(highlightTexture)

	local pushedTexture = button:CreateTexture(nil, "BACKGROUND")
	pushedTexture:SetAllPoints(button)
	pushedTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	pushedTexture:SetTexCoord(unpack(Textures.AHsBmain.coordsPS))
	button:SetPushedTexture(pushedTexture)
	-- AHsearch Button END

	-- AHsearch Previous Button START
	local button = CreateFrame("Button", "AHsearchPrevButton", AHsearchFrame)
	button:SetWidth(20.30769) -- 22/13*12 original: 22
	button:SetHeight(12) -- original: 13
	button:ClearAllPoints()
	button:SetPoint("TOPRIGHT", "AHsearchFrame", "BOTTOMLEFT", 1, 0)
	button:SetScript("OnClick", function()
		if configMode then return end
		AHsearch:CycleSearch(-1)
	end)
	button:EnableMouseWheel(true)
	button:SetScript("OnMouseWheel", NOOP)

	local normalTexture = button:CreateTexture(nil, "BACKGROUND")
	normalTexture:SetAllPoints(button)
	normalTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	normalTexture:SetTexCoord(unpack(Textures.AHsBprev.coords))
	button:SetNormalTexture(normalTexture)

	local highlightTexture = button:CreateTexture(nil, "BACKGROUND")
	highlightTexture:SetAllPoints(button)
	highlightTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	highlightTexture:SetTexCoord(unpack(Textures.AHsBprev.coordsHL))
	button:SetHighlightTexture(highlightTexture)

	local disableTexture = button:CreateTexture(nil, "BACKGROUND")
	disableTexture:SetAllPoints(button)
	disableTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	disableTexture:SetTexCoord(unpack(Textures.AHsBprev.coords))
	disableTexture:SetAlpha(0.5)
	local shaderSupported = disableTexture:SetDesaturated(true)
	if not shaderSupported then
		disableTexture:SetVertexColor(0.5, 0.5, 0.5)
	end
	button:SetDisabledTexture(disableTexture)

	local pushedTexture = button:CreateTexture(nil, "BACKGROUND")
	pushedTexture:SetAllPoints(button)
	pushedTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	pushedTexture:SetTexCoord(unpack(Textures.AHsBprev.coordsPS))
	button:SetPushedTexture(pushedTexture)
	-- AHsearch Previous Button END

	-- AHsearch Next Button START
	local button = CreateFrame("Button", "AHsearchNextButton", AHsearchFrame)
	button:SetWidth(20.30769) -- 22/13*12 original: 22
	button:SetHeight(12) -- original: 13
	button:ClearAllPoints()
	button:SetPoint("TOPLEFT", "AHsearchFrame", "BOTTOMRIGHT", -1, 0)
	button:SetScript("OnClick", function()
		if configMode then return end
		AHsearch:CycleSearch(1)
	end)
	button:EnableMouseWheel(true)
	button:SetScript("OnMouseWheel", NOOP)

	local normalTexture = button:CreateTexture(nil, "BACKGROUND")
	normalTexture:SetAllPoints(button)
	normalTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	normalTexture:SetTexCoord(unpack(Textures.AHsBnext.coords))
	button:SetNormalTexture(normalTexture)

	local highlightTexture = button:CreateTexture(nil, "BACKGROUND")
	highlightTexture:SetAllPoints(button)
	highlightTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	highlightTexture:SetTexCoord(unpack(Textures.AHsBnext.coordsHL))
	button:SetHighlightTexture(highlightTexture)

	local disableTexture = button:CreateTexture(nil, "BACKGROUND")
	disableTexture:SetAllPoints(button)
	disableTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	disableTexture:SetTexCoord(unpack(Textures.AHsBnext.coords))
	disableTexture:SetAlpha(0.5)
	local shaderSupported = disableTexture:SetDesaturated(true)
	if not shaderSupported then
		disableTexture:SetVertexColor(0.5, 0.5, 0.5)
	end
	button:SetDisabledTexture(disableTexture)

	local pushedTexture = button:CreateTexture(nil, "BACKGROUND")
	pushedTexture:SetAllPoints(button)
	pushedTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	pushedTexture:SetTexCoord(unpack(Textures.AHsBnext.coordsPS))
	button:SetPushedTexture(pushedTexture)
	-- AHsearch Next Button END

	-- AHsearch Random Button START
	local button = CreateFrame("Button", "AHsearchRandomButton", AHsearchFrame)
	button:SetWidth(12)
	button:SetHeight(12)
	button:ClearAllPoints()
	button:SetPoint("TOP", "AHsearchFrame", "BOTTOM", 0, 0)
	button:SetScript("OnClick", function()
		if configMode then return end
		AHsearch:RandomSearch()
	end)
	button:EnableMouseWheel(true)
	button:SetScript("OnMouseWheel", NOOP)

	local normalTexture = button:CreateTexture(nil, "BACKGROUND")
	normalTexture:SetAllPoints(button)
	normalTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	normalTexture:SetTexCoord(unpack(Textures.AHsBrandom.coords))
	button:SetNormalTexture(normalTexture)

	local highlightTexture = button:CreateTexture(nil, "BACKGROUND")
	highlightTexture:SetAllPoints(button)
	highlightTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	highlightTexture:SetTexCoord(unpack(Textures.AHsBrandom.coordsHL))
	button:SetHighlightTexture(highlightTexture)

	local disableTexture = button:CreateTexture(nil, "BACKGROUND")
	disableTexture:SetAllPoints(button)
	disableTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	disableTexture:SetTexCoord(unpack(Textures.AHsBrandom.coords))
	disableTexture:SetAlpha(0.5)
	local shaderSupported = disableTexture:SetDesaturated(true)
	if not shaderSupported then
		disableTexture:SetVertexColor(0.5, 0.5, 0.5)
	end
	button:SetDisabledTexture(disableTexture)

	local pushedTexture = button:CreateTexture(nil, "BACKGROUND")
	pushedTexture:SetAllPoints(button)
	pushedTexture:SetPoint("TOPLEFT", 1, -1)
	pushedTexture:SetPoint("BOTTOMRIGHT", -1, 1)
	pushedTexture:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
	pushedTexture:SetTexCoord(unpack(Textures.AHsBrandom.coords))
	button:SetPushedTexture(pushedTexture)
	-- AHsearch Random Button END
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CycleButtons_Toggle()
	if AHsearch_Options.cycleNP then
		AHsearchPrevButton:Show()
		AHsearchNextButton:Show()
	else
		AHsearchPrevButton:Hide()
		AHsearchNextButton:Hide()
	end
	if AHsearch_Options.cycleRND then
		AHsearchRandomButton:Show()
		AHsearchPrevButton:SetPoint("TOPRIGHT", "AHsearchFrame", "BOTTOMLEFT", 1, 0)
		AHsearchNextButton:SetPoint("TOPLEFT", "AHsearchFrame", "BOTTOMRIGHT", -1, 0)
	else
		AHsearchRandomButton:Hide()
		AHsearchPrevButton:SetPoint("TOPRIGHT", "AHsearchFrame", "BOTTOMLEFT", 5, 0)
		AHsearchNextButton:SetPoint("TOPLEFT", "AHsearchFrame", "BOTTOMRIGHT", -5, 0)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:BuildStandardTable()
	table.wipe(GUI_CatFrame_data)
	table.wipe(GUI_SQFrame_data)
	local x = 1
	for cat in SortedPairs(AHsearch_Data) do
		GUI_CatFrame_data[x] = {}
		GUI_CatFrame_data[x].Texture = AHsearch_Data[cat].icon
		GUI_CatFrame_data[x].CatColor = AHsearch_Data[cat].color
		GUI_CatFrame_data[x].Order = AHsearch_Data[cat].order
		GUI_CatFrame_data[x].Cat = cat
		x = x + 1

		local y = 1
		if not GUI_SQFrame_data[cat] then GUI_SQFrame_data[cat] = {} end
		GUI_SQFrame_Count[cat] = 0
		if AHsearch_Data[cat].queries then
			for sq in SortedPairs(AHsearch_Data[cat].queries, 1) do
				GUI_SQFrame_data[cat][y] = {}
				GUI_SQFrame_data[cat][y].QueryColor = itemRarity[ AHsearch_Data[cat].queries[sq].qualityIndex+2 ].color
				GUI_SQFrame_data[cat][y].Order = AHsearch_Data[cat].queries[sq].order
				GUI_SQFrame_data[cat][y].Query = sq
				GUI_SQFrame_data[cat][y].Cat = cat
				y = y + 1
			end
			GUI_SQFrame_Count[cat] = #GUI_SQFrame_data[cat]
		end

	end
	GUI_CatFrame_Count = #GUI_CatFrame_data
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:BuildFilterTable()
	local x1 = 1
	local itemClasses = { GetAuctionItemClasses() }
	for itemClassIndex, itemClass in pairs(itemClasses) do
		--print(itemClassIndex, itemClass) -- TEST
		FilterTable[x1] = {}
		FilterTable[x1].itemClassIndex = itemClassIndex
		FilterTable[x1].itemClass = itemClass

		local x2 = 1
		local subClasses = { GetAuctionItemSubClasses(itemClassIndex) }
		for subClassIndex, subClass in pairs(subClasses) do
			--print("-", subClassIndex, subClass) -- TEST
			if not FilterTable[x1].subClasses then
				FilterTable[x1].subClasses = {}
			end
			FilterTable[x1].subClasses[x2] = {}
			FilterTable[x1].subClasses[x2].subClassIndex = subClassIndex
			FilterTable[x1].subClasses[x2].subClass = subClass

			local x3 = 1
			local typ
			local subClassInvTypes = { GetAuctionInvTypes(itemClassIndex,subClassIndex) }
			for subClassInvTypIndex, subClassInvTyp in pairs(subClassInvTypes) do
				if type(subClassInvTyp) == "string" then
					typ = subClassInvTyp
				elseif type(subClassInvTyp) == "number" then
					--print("--", subClassInvTypIndex/2, _G[typ]) -- TEST
					if not FilterTable[x1].subClasses[x2].subClassInvTypes then
						FilterTable[x1].subClasses[x2].subClassInvTypes = {}
					end
					FilterTable[x1].subClasses[x2].subClassInvTypes[x3] = {}
					FilterTable[x1].subClasses[x2].subClassInvTypes[x3].subClassInvTypIndex = subClassInvTypIndex/2
					FilterTable[x1].subClasses[x2].subClassInvTypes[x3].subClassInvTyp = _G[typ] or "localization error"
					x3 = x3 + 1
				end
			end
			x2 = x2 + 1			

		end
		x1 = x1 + 1
	end

--[[
	-- TEST
	print("----")
	for i = 1, #FilterTable do
		print(i, "#", FilterTable[i].itemClassIndex, FilterTable[i].itemClass)
		if FilterTable[i].subClasses then
			for j = 1, #FilterTable[i].subClasses do
				print(i, j, "#", "-", FilterTable[i].subClasses[j].subClassIndex, FilterTable[i].subClasses[j].subClass)
				if FilterTable[i].subClasses[j].subClassInvTypes then
					for k = 1, #FilterTable[i].subClasses[j].subClassInvTypes do
						print(i, j, k, "#", "--", FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTypIndex, FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTyp)
					end
				end
			end
		end
	end
	-- TEST
--]]
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:OnVerticalScroll(frameName, value)
	_G[frameName.."ScrollFrameScrollBar"]:SetValue(value)
	_G[frameName.."ScrollFrame"].offset = math_floor((value / sizeGeneral.BarHeight) + 0.5)

	if configMode then
		if frameName == "AHsearch_CatFrame" then
			local catMaxConfig = (GUI_CatFrame_Count - 10) * sizeGeneral.BarHeight
			local catMaxNormal = (GUI_CatFrame_Count - AHsearch_Options.curCat) * sizeGeneral.BarHeight
			local dif = catMaxConfig / catMaxNormal
			local nvalue = math_floor((value*dif) + 0.5)
			--print("#", GUI_CatFrame_Count, catMaxConfig, catMaxNormal, dif, nvalue, type(nvalue)) -- TEST
			local nvalueOffset = math_floor((nvalue / sizeGeneral.BarHeight) + 0.5)
			AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrameScrollBar:SetValue(nvalue)
			AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame.offset = nvalueOffset
			AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrameScrollBar:SetValue(nvalue)
			AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame.offset = nvalueOffset
		elseif frameName == "AHsearch_ConfigurationFrame_Categories_SelectCat" then
			local catMaxConfig = (GUI_CatFrame_Count - 10) * sizeGeneral.BarHeight
			local catMaxNormal = (GUI_CatFrame_Count - AHsearch_Options.curCat) * sizeGeneral.BarHeight
			local dif = catMaxNormal / catMaxConfig
			local nvalue = math_floor((value*dif) + 0.5)
			--print("-", GUI_CatFrame_Count, catMaxConfig, catMaxNormal, dif, nvalue, type(nvalue)) -- TEST
			if nvalue < 0 then
				nvalue = 0
			end
			AHsearch_CatFrameScrollFrameScrollBar:SetValue(nvalue)
			AHsearch_CatFrameScrollFrame.offset = math_floor((nvalue / sizeGeneral.BarHeight) + 0.5)
		elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_SelectCat" then
			local catMaxConfig = (GUI_CatFrame_Count - 10) * sizeGeneral.BarHeight
			local catMaxNormal = (GUI_CatFrame_Count - AHsearch_Options.curCat) * sizeGeneral.BarHeight
			local dif = catMaxNormal / catMaxConfig
			local nvalue = math_floor((value*dif) + 0.5)
			--print("*", GUI_CatFrame_Count, catMaxConfig, catMaxNormal, dif, nvalue, type(nvalue)) -- TEST
			if nvalue < 0 then
				nvalue = 0
			end
			AHsearch_CatFrameScrollFrameScrollBar:SetValue(nvalue)
			AHsearch_CatFrameScrollFrame.offset = math_floor((nvalue / sizeGeneral.BarHeight) + 0.5)
		elseif frameName == "AHsearch_QueryFrame" then
			local sqMaxConfig = (GUI_SQFrame_Count[GUI_curCat] - 10) * sizeGeneral.BarHeight
			local sqMaxNormal = (GUI_SQFrame_Count[GUI_curCat] - AHsearch_Options.curQuery) * sizeGeneral.BarHeight
			local dif = sqMaxConfig / sqMaxNormal
			local nvalue = math_floor((value*dif) + 0.5)
			--print("+", GUI_SQFrame_Count[GUI_curCat], sqMaxConfig, sqMaxNormal, dif, nvalue, type(nvalue)) -- TEST
			AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrameScrollBar:SetValue(nvalue)
			AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame.offset = math_floor((nvalue / sizeGeneral.BarHeight) + 0.5)
		elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_SelectSQ" then
			local sqMaxConfig = (GUI_SQFrame_Count[GUI_curCat] - 10) * sizeGeneral.BarHeight
			local sqMaxNormal = (GUI_SQFrame_Count[GUI_curCat] - AHsearch_Options.curQuery) * sizeGeneral.BarHeight
			local dif = sqMaxNormal / sqMaxConfig
			local nvalue = math_floor((value*dif) + 0.5)
			--print("-", GUI_SQFrame_Count[GUI_curCat], sqMaxConfig, sqMaxNormal, dif, nvalue, type(nvalue)) -- TEST
			if nvalue < 0 then
				nvalue = 0
			end
			AHsearch_QueryFrameScrollFrameScrollBar:SetValue(nvalue)
			AHsearch_QueryFrameScrollFrame.offset = math_floor((nvalue / sizeGeneral.BarHeight) + 0.5)
		end
	end

	AHsearch:FillList(frameName)
end

function AHsearch:FillList(frameName)
	-- frameName dependent START ----------
	local maxNum = 0
	local countNum = 0
	local numToDisplay = 0
	if frameName == "AHsearch_CatFrame" then
		maxNum = maxCat
		countNum = GUI_CatFrame_Count
		numToDisplay = AHsearch_Options.curCat
	elseif frameName == "AHsearch_QueryFrame" then
		maxNum = maxQuery
		countNum = GUI_SQFrame_Count[GUI_curCat]
		numToDisplay = AHsearch_Options.curQuery
	elseif frameName == "AHsearch_ConfigurationFrame_Categories_SelectCat" then
		maxNum = 10
		countNum = GUI_CatFrame_Count
		numToDisplay = maxNum
	elseif frameName == "AHsearch_ConfigurationFrame_Categories_Color_PullDownMenu" then
		maxNum = 10
		countNum = #categoryColors
		numToDisplay = maxNum
	elseif frameName == "AHsearch_ConfigurationFrame_Categories_Icon_PullDownMenu" then
		maxNum = 10
		countNum = #categoryIcons
		numToDisplay = maxNum
	elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_SelectCat" then
		maxNum = 10
		countNum = GUI_CatFrame_Count
		numToDisplay = maxNum
	elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_SelectSQ" then
		maxNum = 10
		countNum = GUI_SQFrame_Count[GUI_curCat]
		numToDisplay = maxNum
	elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenu" then
		maxNum = 10
		countNum = GUI_CatFrame_Count
		numToDisplay = maxNum
	elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_Rarity_PullDownMenu" then
		maxNum = 10
		countNum = #itemRarity
		numToDisplay = maxNum
	elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_Filter1_PullDownMenu" then
		maxNum = 10
		countNum = #Filter1Table
		numToDisplay = maxNum
	elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_Filter2_PullDownMenu" then
		maxNum = 10
		countNum = #Filter2Table
		numToDisplay = maxNum
	elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_Filter3_PullDownMenu" then
		maxNum = 10
		countNum = #Filter3Table
		numToDisplay = maxNum
	elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_SortBy_PullDownMenu" then
		maxNum = 10
		countNum = #SortByTable
		numToDisplay = maxNum
	end
	-- frameName dependent END ----------

	local scrollFrameName = frameName.."ScrollFrame"
	local buttonName = frameName.."ScrollFrame_Button"
	local scrollBar = _G[scrollFrameName.."ScrollBar"]
	local scrollUpButton = _G[scrollFrameName.."ScrollBarScrollUpButton"]
	local scrollDownButton = _G[scrollFrameName.."ScrollBarScrollDownButton"]
	local scrollFrameHeight = 0
	--local numToDisplay = maxNum
	local valueStep = sizeGeneral.BarHeight
	local scrollFrameIsShown

	if countNum < numToDisplay then
		numToDisplay = countNum
	end

	if countNum > 0 then
		scrollFrameHeight = (countNum - numToDisplay) * valueStep
		if scrollFrameHeight < 0 then
			scrollFrameHeight = 0
		end
	end
	scrollBar:SetMinMaxValues(0, scrollFrameHeight)
	scrollBar:SetValueStep(valueStep)

	if countNum <= numToDisplay then
		scrollFrameIsShown = false
		_G[scrollFrameName.."ScrollBar"]:Hide()
	else
		scrollFrameIsShown = true
		_G[scrollFrameName.."ScrollBar"]:Show()
	end

	if scrollBar:GetValue() == 0 then
		scrollUpButton:Disable()
	else
		scrollUpButton:Enable()
	end
	if ((scrollBar:GetValue() - scrollFrameHeight) == 0) then
		scrollDownButton:Disable()
	else
		scrollDownButton:Enable()
	end

	for i=1, maxNum do
		_G[buttonName..i]:Hide()
	end
	if countNum == 0 then return end

	local j = 1
	local sliderPos = _G[scrollFrameName].offset
	for i=sliderPos+1, sliderPos+numToDisplay do
		if sliderPos < countNum then
			-- frameName dependent START ----------
			if frameName == "AHsearch_CatFrame" then
				_G[buttonName..j.."Texture"]:SetTexture(categoryIcons[GUI_CatFrame_data[i].Texture].texture)
				_G[buttonName..j.."Texture"]:SetTexCoord(unpack(categoryIcons[GUI_CatFrame_data[i].Texture].coords))
				_G[buttonName..j.."Text"]:SetText(GUI_CatFrame_data[i].CatColor..GUI_CatFrame_data[i].Cat.."|r")
				_G[buttonName..j].cat = GUI_CatFrame_data[i].Cat
				if configMode and GUI_curCat == GUI_CatFrame_data[i].Cat then
					_G[buttonName..j.."Selected"]:Show()
					_G[buttonName..j.."SelectedArrow"]:Show()
				else
					_G[buttonName..j.."Selected"]:Hide()
					_G[buttonName..j.."SelectedArrow"]:Hide()
				end
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.CatButtonSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.CatTextSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.CatButtonNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.CatTextNoSFWidth)
				end
			elseif frameName == "AHsearch_QueryFrame" then
				_G[buttonName..j.."Text"]:SetText(GUI_SQFrame_data[GUI_curCat][i].QueryColor..GUI_SQFrame_data[GUI_curCat][i].Query.."|r")
				_G[buttonName..j].sq = GUI_SQFrame_data[GUI_curCat][i].Query
				_G[buttonName..j].cat = GUI_SQFrame_data[GUI_curCat][i].Cat
				if configMode and GUI_curSQ == GUI_SQFrame_data[GUI_curCat][i].Query then
					_G[buttonName..j.."Selected"]:Show()
					_G[buttonName..j.."SelectedArrow"]:Show()
				else
					_G[buttonName..j.."Selected"]:Hide()
					_G[buttonName..j.."SelectedArrow"]:Hide()
				end
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.QueryButtonSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.QueryTextSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.QueryButtonNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.QueryTextNoSFWidth)
				end

			elseif frameName == "AHsearch_ConfigurationFrame_Categories_SelectCat" then
				_G[buttonName..j.."Texture"]:SetTexture(categoryIcons[GUI_CatFrame_data[i].Texture].texture)
				_G[buttonName..j.."Texture"]:SetTexCoord(unpack(categoryIcons[GUI_CatFrame_data[i].Texture].coords))
				_G[buttonName..j.."Text"]:SetText(GUI_CatFrame_data[i].CatColor..GUI_CatFrame_data[i].Cat.."|r")
				_G[buttonName..j.."Order"]:SetText(GUI_CatFrame_data[i].Order)
				_G[buttonName..j].cat = GUI_CatFrame_data[i].Cat
				if GUI_curCat == GUI_CatFrame_data[i].Cat then
					_G[buttonName..j.."Selected"]:Show()
					_G[buttonName..j.."SelectedArrow"]:Show()
				else
					_G[buttonName..j.."Selected"]:Hide()
					_G[buttonName..j.."SelectedArrow"]:Hide()
				end
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigCatButtonSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigCatTextSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigCatButtonNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigCatTextNoSFWidth)
				end
			elseif frameName == "AHsearch_ConfigurationFrame_Categories_Color_PullDownMenu" then
				_G[buttonName..j.."Text"]:SetText(categoryColors[i].."### "..i)
				_G[buttonName..j].value = categoryColors[i]
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownColorSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownColorSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownColorNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownColorNoSFWidth)
				end
			elseif frameName == "AHsearch_ConfigurationFrame_Categories_Icon_PullDownMenu" then
				_G[buttonName..j.."Texture"]:SetTexture(categoryIcons[i].texture)
				_G[buttonName..j.."Texture"]:SetTexCoord(unpack(categoryIcons[i].coords))
				_G[buttonName..j].value = i
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownIconSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownIconNoSFWidth)
				end

			elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_SelectCat" then
				_G[buttonName..j.."Texture"]:SetTexture(categoryIcons[GUI_CatFrame_data[i].Texture].texture)
				_G[buttonName..j.."Texture"]:SetTexCoord(unpack(categoryIcons[GUI_CatFrame_data[i].Texture].coords))
				_G[buttonName..j.."Text"]:SetText(GUI_CatFrame_data[i].CatColor..GUI_CatFrame_data[i].Cat.."|r")
				_G[buttonName..j].cat = GUI_CatFrame_data[i].Cat
				if GUI_curCat == GUI_CatFrame_data[i].Cat then
					_G[buttonName..j.."Selected"]:Show()
					_G[buttonName..j.."SelectedArrow"]:Show()
				else
					_G[buttonName..j.."Selected"]:Hide()
					_G[buttonName..j.."SelectedArrow"]:Hide()
				end
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigCatButtonSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigCatTextSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigCatButtonNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigCatTextNoSFWidth)
				end
			elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_SelectSQ" then
				_G[buttonName..j.."Text"]:SetText(GUI_SQFrame_data[GUI_curCat][i].QueryColor..GUI_SQFrame_data[GUI_curCat][i].Query.."|r")
				_G[buttonName..j.."Order"]:SetText(GUI_SQFrame_data[GUI_curCat][i].Order)
				_G[buttonName..j].sq = GUI_SQFrame_data[GUI_curCat][i].Query
				_G[buttonName..j].cat = GUI_SQFrame_data[GUI_curCat][i].Cat
				if configMode and GUI_curSQ == GUI_SQFrame_data[GUI_curCat][i].Query then
					_G[buttonName..j.."Selected"]:Show()
					_G[buttonName..j.."SelectedArrow"]:Show()
				else
					_G[buttonName..j.."Selected"]:Hide()
					_G[buttonName..j.."SelectedArrow"]:Hide()
				end
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigQueryButtonSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigQueryTextSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigQueryButtonNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigQueryTextNoSFWidth)
				end
			elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenu" then
				_G[buttonName..j.."Texture"]:SetTexture(categoryIcons[GUI_CatFrame_data[i].Texture].texture)
				_G[buttonName..j.."Texture"]:SetTexCoord(unpack(categoryIcons[GUI_CatFrame_data[i].Texture].coords))
				_G[buttonName..j.."Text"]:SetText(GUI_CatFrame_data[i].CatColor..GUI_CatFrame_data[i].Cat.."|r")
				_G[buttonName..j].value = i
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigCatButtonSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigCatTextSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigCatButtonNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigCatTextNoSFWidth)
				end
			elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_Rarity_PullDownMenu" then
				_G[buttonName..j.."Text"]:SetText(itemRarity[i].color..itemRarity[i].name.."|r")
				_G[buttonName..j].value = i - 2
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownColorSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownColorSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownColorNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownColorNoSFWidth)
				end
			elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_Filter1_PullDownMenu" then
				_G[buttonName..j.."Text"]:SetText(Filter1Table[i].name)
				_G[buttonName..j].value = Filter1Table[i].index
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
				end
			elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_Filter2_PullDownMenu" then
				_G[buttonName..j.."Text"]:SetText(Filter2Table[i].name)
				_G[buttonName..j].value = Filter2Table[i].index
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
				end
			elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_Filter3_PullDownMenu" then
				_G[buttonName..j.."Text"]:SetText(Filter3Table[i].name)
				_G[buttonName..j].value = Filter3Table[i].index
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownFilterSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownFilterNoSFWidth)
				end
			elseif frameName == "AHsearch_ConfigurationFrame_SearchQueries_SortBy_PullDownMenu" then
				_G[buttonName..j.."Text"]:SetText(L[ SortByTable[i] ])
				_G[buttonName..j].value = SortByTable[i]
				if scrollFrameIsShown then
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownSortSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownSortSFWidth)
				else
					_G[buttonName..j]:SetWidth(sizeGeneral.ConfigPullDownSortNoSFWidth)
					_G[buttonName..j.."Text"]:SetWidth(sizeGeneral.ConfigPullDownSortNoSFWidth)
				end
			end
			-- frameName dependent END ----------
			_G[buttonName..j]:Show()
			j = j + 1
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:InitFrames()
	CatSizeUpdate(AHsearch_Options.curCatWidth)
	QuerySizeUpdate(AHsearch_Options.curQueryWidth)
	AHsearch:SetupMainScale(AHsearch_Options.scaleMain)
	AHsearch:ConfigFramePos()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CatFrame_Show(force)
	AHsearch:FillList("AHsearch_CatFrame")

	local height = GUI_CatFrame_Count
	if GUI_CatFrame_Count > AHsearch_Options.curCat then
		height = AHsearch_Options.curCat
	end

	if GUI_CatFrame_Count == 0 then
		AHsearch_CatFrame_NoCatText:Show()
		AHsearch_CatFrame:SetHeight(150)
		AHsearch_ConfigurationFrame_Categories_SelectCatTitle:SetTextColor(0.5, 0.5, 0.5)
		AHsearch_ConfigurationFrame_Categories_SelectCatBorder:SetTexture(0.4, 0.4, 0.4, 1)
	else
		AHsearch_CatFrame_NoCatText:Hide()
		AHsearch_CatFrame:SetHeight(sizeGeneral.Offset+(height*sizeGeneral.BarHeight)+sizeGeneral.Offset)
		AHsearch_ConfigurationFrame_Categories_SelectCatTitle:SetTextColor(1, 0.82, 0)
		AHsearch_ConfigurationFrame_Categories_SelectCatBorder:SetTexture(0.8, 0.2, 0.2, 1)
	end

	AHsearch_CatFrame:SetPoint("TOPLEFT", AHsearchFrame, "BOTTOMLEFT", -(20/(AHsearch_Options.scaleMain/100)), -(12/(AHsearch_Options.scaleMain/100)))

	if configMode and hideMain then return end

	if not force then
		if AHsearch_CatFrame:IsShown() then
			AHsearch_CatFrame:Hide()
		else
			AHsearch_CatFrame:Show()
		end
	else
		AHsearch_CatFrame:Show()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:QueryFrame_Show(cat)
	if not cat or SearchQueryCheck(cat) then
		if not configMode then
			for i = 1, maxCat do
				_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:Hide()
			end
		end
		AHsearch_QueryFrame:Hide()

		for i = 1, maxQuery do
			_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i]:Hide()
		end
		AHsearch_ConfigurationFrame_SearchQueries_SelectSQ:Hide()
		AHsearch_ConfigurationFrame_SearchQueries_SelectSQTitle:SetTextColor(0.5, 0.5, 0.5)

		return
	end

	GUI_curCat = cat
	AHsearch:FillList("AHsearch_QueryFrame")

	AHsearch_ConfigurationFrame_SearchQueries_SelectSQ:Show()
	AHsearch_ConfigurationFrame_SearchQueries_SelectSQTitle:SetTextColor(1, 0.82, 0)
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectSQ")

	local height = GUI_SQFrame_Count[GUI_curCat]
	if GUI_SQFrame_Count[GUI_curCat] > AHsearch_Options.curQuery then
		height = AHsearch_Options.curQuery
	end
	AHsearch_QueryFrame:SetHeight(sizeGeneral.Offset+(height*sizeGeneral.BarHeight)+sizeGeneral.Offset)

	-- sq posi calc START VODOO
	local dataNum = 1
	for i = 1, GUI_CatFrame_Count do
		if GUI_CatFrame_data[i].Cat == cat then
			dataNum = i
		end
	end
	local scrollval = AHsearch_CatFrameScrollFrameScrollBar:GetValue()
	AHsearch_QueryFrame:SetPoint("TOP", _G["AHsearch_CatFrameScrollFrame_Button"..dataNum], "TOP", 0, sizeGeneral.Offset+(scrollval) )
	-- sq posi calc END

	AHsearch_QueryFrame:SetPoint("LEFT", AHsearch_CatFrame, "RIGHT", -sizeGeneral.Border, 0)
	if configMode and hideMain then return end
	AHsearch_QueryFrame:Show()
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CatFrame_ButtonEventUpdate(cat, RightClickConfig)
	if configMode then
		if confirmIsShown then return end

		for i = 1, maxCat do
			_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:Hide()
			_G["AHsearch_CatFrameScrollFrame_Button"..i.."SelectedArrow"]:Hide()
			if _G["AHsearch_CatFrameScrollFrame_Button"..i].cat == cat then
				_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:Show()
				_G["AHsearch_CatFrameScrollFrame_Button"..i.."SelectedArrow"]:Show()
			end
			_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Selected"]:Hide()
			_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Hide()
			if _G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i].cat == cat then
				_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Selected"]:Show()
				_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Show()
			end
			_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Selected"]:Hide()
			_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Hide()
			if _G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i].cat == cat then
				_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Selected"]:Show()
				_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Show()
			end
		end

		for i = 1, maxQuery do
			_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Selected"]:Hide()
			_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."SelectedArrow"]:Hide()
		end

		AHsearch:QueryFrame_Show(cat)
		AHsearch:ConfigurationFrame_Categories_SelectCat(cat)
		AHsearch:ConfigurationFrame_SearchQueries_SelectCat(cat)

		if RightClickConfig then
			GUI_Tab = 1
			AHsearch:ConfigurationFrame_ShowContent()
		end
		if hideMain then
			AHsearch_CatFrame:Hide()
		end
	else
		for i = 1, maxCat do
			_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:Hide()
			if _G["AHsearch_CatFrameScrollFrame_Button"..i].cat == cat then
				_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:Show()
			end
		end
		AHsearch:QueryFrame_Show(cat)
		VisibilityCheck()
	end
end

function AHsearch:QueryFrame_ButtonEventUpdate(cat, sq, RightClickConfig, TabChange)
	if configMode then
		if confirmIsShown then return end
		GUI_curSQ = sq
		for i = 1, maxQuery do
			_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Selected"]:Hide()
			_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."SelectedArrow"]:Hide()
			if _G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i].sq == sq then
				_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Selected"]:Show()
				_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."SelectedArrow"]:Show()
			end
			_G["AHsearch_QueryFrameScrollFrame_Button"..i.."Selected"]:Hide()
			_G["AHsearch_QueryFrameScrollFrame_Button"..i.."SelectedArrow"]:Hide()
			if _G["AHsearch_QueryFrameScrollFrame_Button"..i].sq == sq then
				_G["AHsearch_QueryFrameScrollFrame_Button"..i.."Selected"]:Show()
				_G["AHsearch_QueryFrameScrollFrame_Button"..i.."SelectedArrow"]:Show()
			end
		end
		if RightClickConfig then
			AHsearch:QueryFrame_Show(cat)
			AHsearch:ConfigurationFrame_Categories_SelectCat(cat)
			AHsearch:ConfigurationFrame_SearchQueries_SelectCat(cat)

			for i = 1, maxCat do
				_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:Hide()
				_G["AHsearch_CatFrameScrollFrame_Button"..i.."SelectedArrow"]:Hide()
				if _G["AHsearch_CatFrameScrollFrame_Button"..i].cat == cat then
					_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:Show()
					_G["AHsearch_CatFrameScrollFrame_Button"..i.."SelectedArrow"]:Show()
				end
				_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Selected"]:Hide()
				_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Hide()
				if _G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i].cat == cat then
					_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Selected"]:Show()
					_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Show()
				end
				_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Selected"]:Hide()
				_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Hide()
				if _G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i].cat == cat then
					_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Selected"]:Show()
					_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Show()
				end
			end

			for i = 1, maxQuery do
				if _G["AHsearch_QueryFrameScrollFrame_Button"..i].sq == sq then
					_G["AHsearch_QueryFrameScrollFrame_Button"..i.."Selected"]:Show()
					_G["AHsearch_QueryFrameScrollFrame_Button"..i.."SelectedArrow"]:Show()
				end
				if _G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i].sq == sq then
					_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Selected"]:Show()
					_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."SelectedArrow"]:Show()
				end
			end
		end
		if TabChange then
			GUI_Tab = 2
			AHsearch:ConfigurationFrame_ShowContent()
		end
		AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, sq)
		if hideMain then
			GUI_curSQ = sq
			AHsearch_CatFrame:Hide()
		end
	else
		VisibilityCheck()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CatFrame_CreateFrame()
	if AHsearch_CatFrame then return end

	local frame = CreateFrame("Frame", "AHsearch_CatFrame", AHsearchFrame)
	TemplateBorderTRBL(frame)
	frame:SetWidth(AHsearch_Options.curCatWidth)
	frame:Hide()
	frame:EnableMouse(true)
	frame:SetToplevel(true)
	frame:SetScript("OnLeave", VisibilityCheck)
	frame:SetScript("OnEnter", VisibilityCheck)

	local text = frame:CreateFontString("AHsearch_CatFrame_NoCatText", "ARTWORK", "GameFontNormalSmall")
	text:SetWidth(125)
	text:SetHeight(150)
	text:SetPoint("CENTER", 0, 0)
	text:SetJustifyH("CENTER")
	text:SetText(L["No category available!"].."\n\n"..L["Right-click AHsearch Button for configuration!"])

	local scrollFrame = CreateFrame("ScrollFrame", "AHsearch_CatFrameScrollFrame", frame)
	TemplateScrollFrame(scrollFrame)
	scrollFrame:SetPoint("TOPLEFT", 0, 0)
	scrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	scrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll("AHsearch_CatFrame", value) end)
	AHsearch_CatFrameScrollFrameScrollBar:SetScript("OnEnter", VisibilityCheck)
	AHsearch_CatFrameScrollFrameScrollBar:SetScript("OnLeave", VisibilityCheck)
	AHsearch_CatFrameScrollFrameScrollBarScrollUpButton:SetScript("OnEnter", VisibilityCheck)
	AHsearch_CatFrameScrollFrameScrollBarScrollUpButton:SetScript("OnLeave", VisibilityCheck)
	AHsearch_CatFrameScrollFrameScrollBarScrollDownButton:SetScript("OnEnter", VisibilityCheck)
	AHsearch_CatFrameScrollFrameScrollBarScrollDownButton:SetScript("OnLeave", VisibilityCheck)

	for i = 1, maxCat do
		local button = CreateFrame("Button", "AHsearch_CatFrameScrollFrame_Button"..i, scrollFrame)
		button:SetWidth(sizeGeneral.CatButtonSFWidth)
		button:SetHeight(sizeGeneral.BarHeight)
		button:SetFrameLevel( scrollFrame:GetFrameLevel() + 5 )
		button:Hide()
		if i == 1 then
			button:SetPoint("TOPLEFT", "AHsearch_CatFrameScrollFrame", "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			button:SetPoint("TOPLEFT", "AHsearch_CatFrameScrollFrame_Button"..(i-1), "BOTTOMLEFT", 0, 0)
		end

		local texture = button:CreateTexture("AHsearch_CatFrameScrollFrame_Button"..i.."Texture", "ARTWORK")
		texture:SetWidth(sizeGeneral.BarHeight)
		texture:SetHeight(sizeGeneral.BarHeight)
		texture:SetPoint("LEFT", 0, 0)

		local text = button:CreateFontString("AHsearch_CatFrameScrollFrame_Button"..i.."Text", "ARTWORK", "GameFontNormalSmall")
		text:SetWidth(sizeGeneral.CatTextSFWidth)
		text:SetHeight(sizeGeneral.BarHeight)
		text:SetPoint("LEFT", sizeGeneral.BarHeight+sizeGeneral.Space, 0)
		text:SetJustifyH("LEFT")

		local selected = button:CreateTexture("AHsearch_CatFrameScrollFrame_Button"..i.."Selected", "ARTWORK")
		selected:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		selected:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		selected:SetTexture(1, 1, 1, 0.1)
		selected:Hide()
		local selectedArrow = button:CreateTexture("AHsearch_CatFrameScrollFrame_Button"..i.."SelectedArrow", "ARTWORK")
		selectedArrow:SetWidth(Textures.SelectArrow.width)
		selectedArrow:SetHeight(Textures.SelectArrow.height)
		selectedArrow:SetPoint("LEFT", -sizeGeneral.Offset, 0)
		selectedArrow:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
		selectedArrow:SetTexCoord(unpack(Textures.SelectArrow.coordsR))
		selectedArrow:Hide()

		button:RegisterForClicks("AnyUp")
		button:SetScript("OnClick", function(self, button)
			if button == "RightButton" then
				if not configMode then
					AHsearch:ConfigEnable()
				end
				AHsearch:CatFrame_ButtonEventUpdate(self.cat, 1)
			end
		end)
		button:SetScript("OnEnter", function(self) AHsearch:CatFrame_ButtonEventUpdate(self.cat) end)
		button:SetScript("OnLeave", VisibilityCheck)

		local highlight = button:CreateTexture(nil, "ARTWORK")
		highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		highlight:SetTexture(1, 1, 1, 0.2)
		button:SetHighlightTexture(highlight)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:QueryFrame_CreateFrame()
	if AHsearch_QueryFrame then return end

	local frame = CreateFrame("Frame", "AHsearch_QueryFrame", AHsearch_CatFrame)
	TemplateBorderTRBL(frame)
	frame:SetWidth(AHsearch_Options.curQueryWidth)
	frame:SetClampedToScreen(true)
	frame:Hide()
	frame:EnableMouse(true)
	frame:SetToplevel(true)
	frame:SetScript("OnLeave", VisibilityCheck)
	frame:SetScript("OnEnter", VisibilityCheck)

	local scrollFrame = CreateFrame("ScrollFrame", "AHsearch_QueryFrameScrollFrame", frame)
	TemplateScrollFrame(scrollFrame)
	scrollFrame:SetPoint("TOPLEFT", 0, 0)
	scrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	scrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll("AHsearch_QueryFrame", value) end)
	AHsearch_QueryFrameScrollFrameScrollBar:SetScript("OnEnter", VisibilityCheck)
	AHsearch_QueryFrameScrollFrameScrollBar:SetScript("OnLeave", VisibilityCheck)
	AHsearch_QueryFrameScrollFrameScrollBarScrollUpButton:SetScript("OnEnter", VisibilityCheck)
	AHsearch_QueryFrameScrollFrameScrollBarScrollUpButton:SetScript("OnLeave", VisibilityCheck)
	AHsearch_QueryFrameScrollFrameScrollBarScrollDownButton:SetScript("OnEnter", VisibilityCheck)
	AHsearch_QueryFrameScrollFrameScrollBarScrollDownButton:SetScript("OnLeave", VisibilityCheck)

	for i = 1, maxQuery do
		local button = CreateFrame("Button", "AHsearch_QueryFrameScrollFrame_Button"..i, frame)
		button:SetWidth(sizeGeneral.QueryButtonSFWidth)
		button:SetHeight(sizeGeneral.BarHeight)
		button:SetFrameLevel( scrollFrame:GetFrameLevel() + 5 )
		button:Hide()
		if i == 1 then
			button:SetPoint("TOPLEFT", "AHsearch_QueryFrameScrollFrame", "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			button:SetPoint("TOPLEFT", "AHsearch_QueryFrameScrollFrame_Button"..(i-1), "BOTTOMLEFT", 0, 0)
		end

		local text = button:CreateFontString("AHsearch_QueryFrameScrollFrame_Button"..i.."Text", "ARTWORK", "GameFontNormalSmall")
		text:SetWidth(sizeGeneral.QueryTextSFWidth)
		text:SetHeight(sizeGeneral.BarHeight)
		text:SetPoint("LEFT", 0, 0)
		text:SetJustifyH("LEFT")

		local selected = button:CreateTexture("AHsearch_QueryFrameScrollFrame_Button"..i.."Selected", "ARTWORK")
		selected:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		selected:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		selected:SetTexture(1, 1, 1, 0.1)
		selected:Hide()
		local selectedArrow = button:CreateTexture("AHsearch_QueryFrameScrollFrame_Button"..i.."SelectedArrow", "ARTWORK")
		selectedArrow:SetWidth(Textures.SelectArrow.width)
		selectedArrow:SetHeight(Textures.SelectArrow.height)
		selectedArrow:SetPoint("LEFT", -sizeGeneral.Offset, 0)
		selectedArrow:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
		selectedArrow:SetTexCoord(unpack(Textures.SelectArrow.coordsR))
		selectedArrow:Hide()

		button:RegisterForClicks("AnyUp")
		button:SetScript("OnClick", function(self, button)
			if button == "LeftButton" then
				if configMode then return end
				AHsearch:Search(self.cat, self.sq)
			elseif button == "RightButton" then
				if configMode then
					AHsearch:QueryFrame_ButtonEventUpdate(self.cat, self.sq, nil, 1)
				else
					AHsearch:ConfigEnable()
					AHsearch:QueryFrame_ButtonEventUpdate(self.cat, self.sq, 1, 1)
				end
			end
		end)
		button:SetScript("OnEnter", function(self) AHsearch:QueryFrame_ButtonEventUpdate(self.cat, self.sq) end)
		button:SetScript("OnLeave", VisibilityCheck)

		local highlight = button:CreateTexture(nil, "ARTWORK")
		highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		highlight:SetTexture(1, 1, 1, 0.2)
		button:SetHighlightTexture(highlight)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:ConfigurationFrame_CreateFrame()
	if AHsearch_ConfigurationFrame then return end

	local mainWidth = 425
	local mainHeight = 637--610
	local topheight = 87--60

	local configframe = CreateFrame("Frame", "AHsearch_ConfigurationFrame", UIParent)
	TemplateBorderTRBL(configframe)
	configframe:SetWidth(mainWidth)
	configframe:SetHeight(mainHeight)
	configframe:SetMovable(true)
	configframe:SetToplevel(true)
	configframe:SetClampedToScreen(true)
	configframe:SetClampRectInsets((mainWidth-50)/2, -((mainWidth-50)/2), -(mainHeight-topheight), mainHeight-topheight)
	configframe:EnableMouse(true)
	configframe:EnableMouseWheel(true)
	configframe:SetScript("OnMouseWheel", NOOP)
	configframe:Hide()

	-- tab START ----------------------------------------
	local selection = CreateFrame("Frame", "AHsearch_ConfigurationFrame_Selection", configframe)
	TemplateBorderTRBL(selection)
	selection:SetWidth(mainWidth)
	selection:SetHeight(topheight)
	selection:SetPoint("TOPLEFT", configframe, "TOPLEFT", 0, 0)
	selection:EnableMouse(true)
	selection:SetToplevel(true)
	selection:SetScript("OnMouseDown", function()
		AHsearch_ConfigurationFrame:StartMoving() AHsearch_ConfigurationFrame:StopMovingOrSizing()
	end)

	local closeConfig = CreateFrame("Button", "AHsearch_ConfigurationFrame_CloseConfig", selection)
	TemplateTextButton(closeConfig, L["Close Configuration"], 1)
	closeConfig:SetWidth(300)
	closeConfig:SetHeight(24)
	closeConfig:SetPoint("TOP", selection, "TOP", 0, -10)
	closeConfig:SetScript("OnClick", function() AHsearch:ConfigDisable() end)

	local hideCatSQ = CreateFrame("Button", "AHsearch_ConfigurationFrame_HideCatSQ", selection)
	TemplateTextButton(hideCatSQ, L["HIDE Categories / Search Queries"], 3)
	hideCatSQ:SetWidth(220)
	hideCatSQ:SetHeight(18)
	hideCatSQ:SetPoint("TOP", closeConfig, "BOTTOM", 0, -5)
	hideCatSQ:SetScript("OnClick", function()
		if AHsearch_CatFrame:IsShown() then
			AHsearch_CatFrame:Hide()
			hideCatSQ:SetText(L["SHOW Categories / Search Queries"])
			hideMain = true
		else
			AHsearch_CatFrame:Show()
			hideCatSQ:SetText(L["HIDE Categories / Search Queries"])
			hideMain = false
		end
	end)

	local tabwidth = (mainWidth-(4*10))/3

	local tabCategories = CreateFrame("Button", "AHsearch_ConfigurationFrame_Categories", selection)
	TemplateTabButton(tabCategories, L["Categories"], 1, tabwidth)
	tabCategories:SetWidth(tabwidth)
	tabCategories:SetHeight(20)
	tabCategories:SetPoint("BOTTOMLEFT", selection, "BOTTOMLEFT", 10, -1)
	tabCategories:SetScript("OnMouseDown", function()
		AHsearch_ConfigurationFrame:StartMoving() AHsearch_ConfigurationFrame:StopMovingOrSizing()
	end)
	tabCategories:SetScript("OnClick", function()
		if GUI_Tab == 1 then return end
		GUI_Tab = 1
		AHsearch:ConfigurationFrame_ShowContent()
	end)

	local tabSearchQueries = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries", selection)
	TemplateTabButton(tabSearchQueries, L["Search Queries"], 1, tabwidth)
	tabSearchQueries:SetWidth(tabwidth)
	tabSearchQueries:SetHeight(20)
	tabSearchQueries:SetPoint("BOTTOMLEFT", selection, "BOTTOMLEFT", 10+tabwidth+10, -1)
	tabSearchQueries:SetScript("OnMouseDown", function()
		AHsearch_ConfigurationFrame:StartMoving() AHsearch_ConfigurationFrame:StopMovingOrSizing()
	end)
	tabSearchQueries:SetScript("OnClick", function()
		if GUI_Tab == 2 then return end
		GUI_Tab = 2
		AHsearch:ConfigurationFrame_ShowContent()
	end)

	local tabOptions = CreateFrame("Button", "AHsearch_ConfigurationFrame_Options", selection)
	TemplateTabButton(tabOptions, L["Options"], 1, tabwidth)
	tabOptions:SetWidth(tabwidth)
	tabOptions:SetHeight(20)
	tabOptions:SetPoint("BOTTOMLEFT", selection, "BOTTOMLEFT", 10+tabwidth+10+tabwidth+10, -1)
	tabOptions:SetScript("OnMouseDown", function()
		AHsearch_ConfigurationFrame:StartMoving() AHsearch_ConfigurationFrame:StopMovingOrSizing()
	end)
	tabOptions:SetScript("OnClick", function()
		if GUI_Tab == 3 then return end
		GUI_Tab = 3
		AHsearch:ConfigurationFrame_ShowContent()
	end)
	-- tab END ----------------------------------------

	-- Categories START ----------------------------------------
	local contentCategories = CreateFrame("Frame", "AHsearch_ConfigurationFrame_Categories_Content", configframe)
	TemplateBorderTRBL(contentCategories)
	contentCategories:SetWidth(mainWidth)
	contentCategories:SetHeight(mainHeight-topheight+sizeGeneral.Border)
	contentCategories:SetPoint("TOPLEFT", selection, "BOTTOMLEFT", 0, 1)
	contentCategories:EnableMouse(true)
	contentCategories:Hide()

	-- - Categories create new cat START
	local textCreate = contentCategories:CreateFontString("AHsearch_ConfigurationFrame_Categories_NewCatTitle", "ARTWORK", "GameFontNormalSmall")
	textCreate:SetHeight(20)
	textCreate:SetPoint("TOPLEFT", contentCategories, "TOPLEFT", 10, -10)
	textCreate:SetJustifyH("LEFT")
	textCreate:SetText(L["Create New"]..":")

	local editbox = CreateFrame("EditBox", "AHsearch_ConfigurationFrame_Categories_NewCat", contentCategories, "InputBoxTemplate")
	TemplateEditBox(editbox)
	editbox:SetWidth(sizeGeneral.ConfigTextWidth)
	editbox:SetPoint("LEFT", textCreate, "RIGHT", 10, 0)
	editbox:SetMaxLetters(maxBoxInputLength)
	editbox:SetScript("OnTextChanged",  function(self)
		local text = self:GetText()
		if text == "" then

		else
			AHsearch_ConfigurationFrame_Categories.editType = "new"
		end
		ConfigurationFrame_Categories_SetNewCat(text)
	end)
	EnableEditBox("AHsearch_ConfigurationFrame_Categories_NewCat")

	local textCreateNote = contentCategories:CreateFontString("AHsearch_ConfigurationFrame_Categories_NewCatNote", "ARTWORK", "GameFontNormalSmall")
	textCreateNote:SetHeight(20)
	textCreateNote:SetPoint("LEFT", AHsearch_ConfigurationFrame_Categories_NewCat, "RIGHT", 5, 0)
	textCreateNote:SetJustifyH("LEFT")
	textCreateNote:SetText(L["*NEW"])
	textCreateNote:SetTextColor(0.5, 0.5, 0.5) -- NOTETEXT
	-- - Categories create new cat END

	-- - Categories select cat START
	local textSelectCat = contentCategories:CreateFontString("AHsearch_ConfigurationFrame_Categories_SelectCatTitle", "ARTWORK", "GameFontNormalSmall")
	textSelectCat:SetHeight(20)
	textSelectCat:SetPoint("TOP", "AHsearch_ConfigurationFrame_Categories_NewCat", "BOTTOM", 0, -10)
	textSelectCat:SetPoint("LEFT", textCreate, "LEFT", 0, 0)
	textSelectCat:SetJustifyH("LEFT")
	textSelectCat:SetText(L["Select"]..":")

	local scrollFrameContainer = CreateFrame("Frame", "AHsearch_ConfigurationFrame_Categories_SelectCat", contentCategories)
	TemplateBorderTRBL(scrollFrameContainer, 1)
	scrollFrameContainer:SetWidth(sizeGeneral.ConfigCatWidth)
	scrollFrameContainer:SetHeight(sizeGeneral.Offset+(10*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	scrollFrameContainer:SetPoint("TOP", "AHsearch_ConfigurationFrame_Categories_NewCat", "BOTTOM", 0, -10)
	scrollFrameContainer:SetPoint("LEFT", textSelectCat, "RIGHT", 10, 0)

	local scrollFrame = CreateFrame("ScrollFrame", "AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame", scrollFrameContainer)
	TemplateScrollFrame(scrollFrame)
	scrollFrame:SetPoint("TOPLEFT", 0, 0)
	scrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	scrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll("AHsearch_ConfigurationFrame_Categories_SelectCat", value) end)

	for i = 1, maxCat do
		local button = CreateFrame("Button", "AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i, scrollFrameContainer)
		button:SetWidth(sizeGeneral.ConfigCatButtonSFWidth)
		button:SetHeight(sizeGeneral.BarHeight)
		button:Hide()
		if i == 1 then
			button:SetPoint("TOPLEFT", "AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame", "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			button:SetPoint("TOPLEFT", "AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..(i-1), "BOTTOMLEFT", 0, 0)
		end

		local texture = button:CreateTexture("AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Texture", "ARTWORK")
		texture:SetWidth(sizeGeneral.BarHeight)
		texture:SetHeight(sizeGeneral.BarHeight)
		texture:SetPoint("LEFT", 0, 0)

		local text = button:CreateFontString("AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Text", "ARTWORK", "GameFontNormalSmall")
		text:SetWidth(sizeGeneral.ConfigCatTextSFWidth)
		text:SetHeight(sizeGeneral.BarHeight)
		text:SetPoint("LEFT", sizeGeneral.BarHeight+sizeGeneral.Space, 0)
		text:SetJustifyH("LEFT")

		local selected = button:CreateTexture("AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Selected", "ARTWORK")
		selected:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		selected:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		selected:SetTexture(1, 1, 1, 0.1)
		selected:Hide()
		local selectedArrow = button:CreateTexture("AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."SelectedArrow", "ARTWORK")
		selectedArrow:SetWidth(Textures.SelectArrow.width)
		selectedArrow:SetHeight(Textures.SelectArrow.height)
		selectedArrow:SetPoint("LEFT", -sizeGeneral.Offset, 0)
		selectedArrow:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
		selectedArrow:SetTexCoord(unpack(Textures.SelectArrow.coordsR))
		selectedArrow:Hide()

		local order = button:CreateFontString("AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Order", "ARTWORK", "GameFontWhiteSmall")
		order:SetWidth(30)
		order:SetHeight(sizeGeneral.BarHeight)
		order:SetPoint("RIGHT", button, "LEFT", -(sizeGeneral.Offset+sizeGeneral.Space), 0)
		order:SetJustifyH("RIGHT")
		order:SetTextColor(0, 0.75, 0) -- ORDER_COLOR

		button:SetScript("OnClick", function(self) AHsearch:CatFrame_ButtonEventUpdate(self.cat) end)

		local highlight = button:CreateTexture(nil, "ARTWORK")
		highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		highlight:SetTexture(1, 1, 1, 0.2)
		button:SetHighlightTexture(highlight)
	end
	AHsearch:FillList("AHsearch_ConfigurationFrame_Categories_SelectCat")
	-- - Categories select cat END

	-- - Categories current cat START
	local textCurrentCat = contentCategories:CreateFontString("AHsearch_ConfigurationFrame_Categories_CurrentCatTitle", "ARTWORK", "GameFontNormalSmall")
	textCurrentCat:SetHeight(20)
	textCurrentCat:SetPoint("TOP", "AHsearch_ConfigurationFrame_Categories_SelectCat", "BOTTOM", 0, -10)
	textCurrentCat:SetPoint("LEFT", textSelectCat, "LEFT", 0, 0)
	textCurrentCat:SetJustifyH("LEFT")
	textCurrentCat:SetText(L["Current"]..":")
	textCurrentCat:SetTextColor(0.5, 0.5, 0.5)

	local selectedCategoryTexture = contentCategories:CreateTexture("AHsearch_ConfigurationFrame_Categories_CurrentCatTexture", "ARTWORK")
	selectedCategoryTexture:SetWidth(sizeGeneral.BarHeight)
	selectedCategoryTexture:SetHeight(sizeGeneral.BarHeight)
	selectedCategoryTexture:SetPoint("LEFT", textCurrentCat, "LEFT", 20, 0)

	local selectedCategoryText = contentCategories:CreateFontString("AHsearch_ConfigurationFrame_Categories_CurrentCatText", "ARTWORK", "GameFontNormalSmall")
	selectedCategoryText:SetHeight(20)
	selectedCategoryText:SetPoint("LEFT", selectedCategoryTexture, "RIGHT", 2, 0)
	selectedCategoryText:SetJustifyH("LEFT")
	selectedCategoryText:SetText("")
	-- - Categories current cat END

	-- - Categories name START
	local textName = contentCategories:CreateFontString("AHsearch_ConfigurationFrame_Categories_NameCatTitle", "ARTWORK", "GameFontNormalSmall")
	textName:SetHeight(20)
	textName:SetPoint("TOPLEFT", textCurrentCat, "BOTTOMLEFT", 0, -10)
	textName:SetJustifyH("LEFT")
	textName:SetText(L["Title"]..":") -- formerly (L["Name"]..":")
	textName:SetTextColor(0.5, 0.5, 0.5)

	local editbox = CreateFrame("EditBox", "AHsearch_ConfigurationFrame_Categories_NameCat", contentCategories, "InputBoxTemplate")
	TemplateEditBox(editbox)
	editbox:SetWidth(sizeGeneral.ConfigTextWidth)
	editbox:SetPoint("LEFT", textName, "RIGHT", 10, 0)
	editbox:SetMaxLetters(maxBoxInputLength)
	editbox:SetScript("OnTextChanged",  function(self)
		local text = self:GetText()
		ConfigurationFrame_Categories_SetName(text)
	end)
	DisableEditBox("AHsearch_ConfigurationFrame_Categories_NameCat")

	local textNameNote = contentCategories:CreateFontString("AHsearch_ConfigurationFrame_Categories_NameCatNote", "ARTWORK", "GameFontNormalSmall")
	textNameNote:SetHeight(20)
	textNameNote:SetPoint("LEFT", AHsearch_ConfigurationFrame_Categories_NameCat, "RIGHT", 5, 0)
	textNameNote:SetJustifyH("LEFT")
	textNameNote:SetText(L["*Required"])
	textNameNote:SetTextColor(0.5, 0.5, 0.5) -- NOTETEXT
	-- - Categories name END

	-- - Categories order START
	local textOrder = contentCategories:CreateFontString("AHsearch_ConfigurationFrame_Categories_OrderTitle", "ARTWORK", "GameFontNormalSmall")
	textOrder:SetHeight(20)
	textOrder:SetPoint("TOPLEFT", textName, "BOTTOMLEFT", 0, -10)
	textOrder:SetJustifyH("LEFT")
	textOrder:SetText(L["Position"]..":")
	textOrder:SetTextColor(0.5, 0.5, 0.5)

	local orderEditbox = CreateFrame("EditBox", "AHsearch_ConfigurationFrame_Categories_Order", contentCategories, "InputBoxTemplate")
	TemplateEditBox(orderEditbox)
	orderEditbox:SetWidth(40)
	orderEditbox:SetPoint("LEFT", textOrder, "RIGHT", 10, 0)
	orderEditbox:SetNumeric(true)
	orderEditbox:SetMaxLetters(2)
	orderEditbox:SetTextColor(0, 0.75, 0) -- ORDER_COLOR
	orderEditbox:SetScript("OnTextChanged",  function(self)
		local number = AHsearch_ConfigurationFrame_Categories_Order:GetNumber()
		if number == 0 then
			self:SetNumber(1)
			number = 1
		end
		AHsearch_ConfigurationFrame_Categories_Order.value = number
		ConfigurationFrame_Categories_SetOrder(number)
	end)
	DisableEditBox("AHsearch_ConfigurationFrame_Categories_Order")
	-- - Categories order END

	-- - Categories color START
	local textColor = contentCategories:CreateFontString("AHsearch_ConfigurationFrame_Categories_ColorTitle", "ARTWORK", "GameFontNormalSmall")
	textColor:SetHeight(20)
	textColor:SetPoint("TOPLEFT", textOrder, "BOTTOMLEFT", 0, -10)
	textColor:SetJustifyH("LEFT")
	textColor:SetText(L["Color"]..":")
	textColor:SetTextColor(0.5, 0.5, 0.5)

	local colorPullDown = CreateFrame("Button", "AHsearch_ConfigurationFrame_Categories_Color", contentCategories)
	TemplatePullDownMenu(colorPullDown, "", sizeGeneral.ConfigPullDownColorWidth, categoryColors, false, ConfigurationFrame_Categories_SetColor)
	colorPullDown:SetWidth(sizeGeneral.ConfigPullDownColorWidth)
	colorPullDown:SetHeight(18)
	colorPullDown:SetPoint("LEFT", textColor, "RIGHT", 10, 0)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_Categories_Color")
	-- - Categories color END

	-- - Categories icon START
	local textIcon = contentCategories:CreateFontString("AHsearch_ConfigurationFrame_Categories_IconTitle", "ARTWORK", "GameFontNormalSmall")
	textIcon:SetHeight(20)
	textIcon:SetPoint("TOPLEFT", textColor, "BOTTOMLEFT", 0, -10)
	textIcon:SetJustifyH("LEFT")
	textIcon:SetText(L["Icon"]..":")
	textIcon:SetTextColor(0.5, 0.5, 0.5)

	local iconPullDown = CreateFrame("Button", "AHsearch_ConfigurationFrame_Categories_Icon", contentCategories)
	TemplatePullDownMenu(iconPullDown, "", sizeGeneral.ConfigPullDownIconWidth, categoryIcons, 1, ConfigurationFrame_Categories_SetIcon)
	iconPullDown:SetWidth(sizeGeneral.ConfigPullDownIconWidth)
	iconPullDown:SetHeight(18)
	iconPullDown:SetPoint("LEFT", textIcon, "RIGHT", 10, 0)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_Categories_Icon")
	-- - Categories icon END

	-- - Categories update START
	local updateCategory = CreateFrame("Button", "AHsearch_ConfigurationFrame_Categories_UpdateCat", contentCategories)
	TemplateTextButton(updateCategory, L["Update"], 1)
	updateCategory:SetWidth(200)
	updateCategory:SetHeight(24)
	updateCategory:SetPoint("TOP", "AHsearch_ConfigurationFrame_Categories_Icon", "BOTTOM", 0, -20)
	updateCategory:SetPoint("LEFT", contentCategories, "LEFT", (mainWidth-200-10-200)/2, 0)
	updateCategory:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_UpdateCat(false) end)
	DisableTextButton("AHsearch_ConfigurationFrame_Categories_UpdateCat")
	-- - Categories update END

	-- - Categories delete START
	local deleteCategory = CreateFrame("Button", "AHsearch_ConfigurationFrame_Categories_DeleteCat", contentCategories)
	TemplateTextButton(deleteCategory, L["Delete"], 1)
	deleteCategory:SetWidth(200)
	deleteCategory:SetHeight(24)
	deleteCategory:SetPoint("LEFT", updateCategory, "RIGHT", 10, 0)
	deleteCategory:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_DeleteCat(false) end)
	DisableTextButton("AHsearch_ConfigurationFrame_Categories_DeleteCat")
	-- - Categories delete END

	-- - Categories clearall START
	local clearallCategory = CreateFrame("Button", "AHsearch_ConfigurationFrame_Categories_ClearAllCat", contentCategories)
	TemplateTextButton(clearallCategory, L["Clear All"], 2)
	clearallCategory:SetWidth(120)
	clearallCategory:SetHeight(18)
	clearallCategory:SetPoint("TOP", "AHsearch_ConfigurationFrame_Categories_DeleteCat", "BOTTOM", 0, -10)
	clearallCategory:SetPoint("LEFT", contentCategories, "LEFT", (mainWidth-120)/2, 0)
	clearallCategory:SetScript("OnClick", function() AHsearch:ConfigurationFrame_ClearAll() end)
	DisableTextButton("AHsearch_ConfigurationFrame_Categories_ClearAllCat")
	-- - Categories clearall END

	-- - Categories reposition START
	local nw = 10
	local cw = 0
	cw = textCreate:GetStringWidth()     if cw > nw then nw = cw end
	cw = textCurrentCat:GetStringWidth() if cw > nw then nw = cw end
	cw = textSelectCat:GetStringWidth()  if cw > nw then nw = cw end
	cw = textName:GetStringWidth()       if cw > nw then nw = cw end
	cw = textOrder:GetStringWidth()      if cw > nw then nw = cw end
	cw = textColor:GetStringWidth()      if cw > nw then nw = cw end
	cw = textIcon:GetStringWidth()       if cw > nw then nw = cw end
	nw = floor(nw+10)
	if nw < 70 then nw = 70 end

	AHsearch_ConfigurationFrame_Categories_NewCat:SetPoint("LEFT", textCreate, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_Categories_SelectCat:SetPoint("LEFT", textSelectCat, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_Categories_CurrentCatTexture:SetPoint("LEFT", textCurrentCat, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_Categories_NameCat:SetPoint("LEFT", textName, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_Categories_Order:SetPoint("LEFT", textOrder, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_Categories_Color:SetPoint("LEFT", textColor, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_Categories_Icon:SetPoint("LEFT", textIcon, "LEFT", nw, 0)
	-- - Categories reposition END
	-- Categories END ----------------------------------------

	-- SearchQueries START ----------------------------------------
	local contentSearchQueries = CreateFrame("Frame", "AHsearch_ConfigurationFrame_SearchQueries_Content", configframe)
	TemplateBorderTRBL(contentSearchQueries)
	contentSearchQueries:SetWidth(mainWidth)
	contentSearchQueries:SetHeight(mainHeight-topheight+sizeGeneral.Border)
	contentSearchQueries:SetPoint("TOPLEFT", selection, "BOTTOMLEFT", 0, 1)
	contentSearchQueries:EnableMouse(true)
	contentSearchQueries:Hide()

	-- - SearchQueries create new query START
	local textCreate = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_NewSQTitle", "ARTWORK", "GameFontNormalSmall")
	textCreate:SetHeight(20)
	textCreate:SetPoint("TOPLEFT", contentSearchQueries, "TOPLEFT", 10, -10)
	textCreate:SetJustifyH("LEFT")
	textCreate:SetText(L["Create New"]..":")
	textCreate:SetTextColor(0.5, 0.5, 0.5)

	local editbox = CreateFrame("EditBox", "AHsearch_ConfigurationFrame_SearchQueries_NewSQ", contentSearchQueries, "InputBoxTemplate")
	TemplateEditBox(editbox)
	editbox:SetWidth(sizeGeneral.ConfigTextWidth)
	editbox:SetPoint("LEFT", textCreate, "RIGHT", 10, 0)
	editbox:SetMaxLetters(maxBoxInputLength)
	editbox:SetScript("OnTextChanged",  function(self)
		local text = self:GetText()
		if text ~= "" then
			AHsearch:ConfigurationFrame_SearchQueries_CreateNewSQ()
			AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Create New"])
			EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ", 1)
		else
			if AHsearch_ConfigurationFrame_SearchQueries.editType == "new" then
				AHsearch:ConfigurationFrame_SearchQueries_CreateNewSQ()
				AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Create New"])
				DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ")
			end
		end
	end)
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_NewSQ")

	local textCreateNote = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_NewSQNote", "ARTWORK", "GameFontNormalSmall")
	textCreateNote:SetHeight(20)
	textCreateNote:SetPoint("LEFT", AHsearch_ConfigurationFrame_SearchQueries_NewSQ, "RIGHT", 5, 0)
	textCreateNote:SetJustifyH("LEFT")
	textCreateNote:SetText(L["*NEW"])
	textCreateNote:SetTextColor(0.5, 0.5, 0.5) -- NOTETEXT
	-- - SearchQueries create new query END

	-- - SearchQueries selectCat START
	local textSelectCat = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_SelectCatTitle", "ARTWORK", "GameFontNormalSmall")
	textSelectCat:SetHeight(20)
	textSelectCat:SetPoint("LEFT", contentSearchQueries, "LEFT", 10, 0)
	textSelectCat:SetPoint("TOP", "AHsearch_ConfigurationFrame_SearchQueries_NewSQ", "BOTTOM", 0, 0)
	textSelectCat:SetJustifyH("LEFT")
	textSelectCat:SetText(L["Select Category"]..":")

	local scrollFrameContainer = CreateFrame("Frame", "AHsearch_ConfigurationFrame_SearchQueries_SelectCat", contentSearchQueries)
	TemplateBorderTRBL(scrollFrameContainer, 1)
	scrollFrameContainer:SetWidth(sizeGeneral.ConfigCatWidth)
	scrollFrameContainer:SetHeight(sizeGeneral.Offset+(10*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	scrollFrameContainer:SetPoint("TOP", textSelectCat, "BOTTOM", 0, 0)
	scrollFrameContainer:SetPoint("LEFT", textSelectCat, "LEFT", 0, 0)

	local scrollFrame = CreateFrame("ScrollFrame", "AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame", scrollFrameContainer)
	TemplateScrollFrame(scrollFrame)
	scrollFrame:SetPoint("TOPLEFT", 0, 0)
	scrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	scrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll("AHsearch_ConfigurationFrame_SearchQueries_SelectCat", value) end)

	for i = 1, maxCat do
		local button = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i, scrollFrameContainer)
		button:SetWidth(sizeGeneral.ConfigCatButtonSFWidth)
		button:SetHeight(sizeGeneral.BarHeight)
		button:Hide()
		if i == 1 then
			button:SetPoint("TOPLEFT", "AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame", "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			button:SetPoint("TOPLEFT", "AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..(i-1), "BOTTOMLEFT", 0, 0)
		end

		local texture = button:CreateTexture("AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Texture", "ARTWORK")
		texture:SetWidth(sizeGeneral.BarHeight)
		texture:SetHeight(sizeGeneral.BarHeight)
		texture:SetPoint("LEFT", 0, 0)

		local text = button:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Text", "ARTWORK", "GameFontNormalSmall")
		text:SetWidth(sizeGeneral.ConfigCatTextSFWidth)
		text:SetHeight(sizeGeneral.BarHeight)
		text:SetPoint("LEFT", sizeGeneral.BarHeight+sizeGeneral.Space, 0)
		text:SetJustifyH("LEFT")

		local selected = button:CreateTexture("AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Selected", "ARTWORK")
		selected:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		selected:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		selected:SetTexture(1, 1, 1, 0.1)
		selected:Hide()
		local selectedArrow = button:CreateTexture("AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."SelectedArrow", "ARTWORK")
		selectedArrow:SetWidth(Textures.SelectArrow.width)
		selectedArrow:SetHeight(Textures.SelectArrow.height)
		selectedArrow:SetPoint("LEFT", -sizeGeneral.Offset, 0)
		selectedArrow:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
		selectedArrow:SetTexCoord(unpack(Textures.SelectArrow.coordsR))
		selectedArrow:Hide()

		button:SetScript("OnClick", function(self) AHsearch:CatFrame_ButtonEventUpdate(self.cat) end)

		local highlight = button:CreateTexture(nil, "ARTWORK")
		highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		highlight:SetTexture(1, 1, 1, 0.2)
		button:SetHighlightTexture(highlight)
	end
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectCat")
	-- - SearchQueries selectCat END

	-- - SearchQueries selectSQ START
	local textSelectSQ = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_SelectSQTitle", "ARTWORK", "GameFontNormalSmall")
	textSelectSQ:SetHeight(20)
	textSelectSQ:SetPoint("LEFT", "AHsearch_ConfigurationFrame_SearchQueries_SelectCat", "RIGHT", 30, 0)
	textSelectSQ:SetPoint("TOP", textSelectCat, "TOP", 0, 0)
	textSelectSQ:SetJustifyH("LEFT")
	textSelectSQ:SetText(L["Select Search Query"]..":")
	textSelectSQ:SetTextColor(0.5, 0.5, 0.5)

	local scrollFrameContainer = CreateFrame("Frame", "AHsearch_ConfigurationFrame_SearchQueries_SelectSQ", contentSearchQueries)
	TemplateBorderTRBL(scrollFrameContainer, 1)
	scrollFrameContainer:SetWidth(sizeGeneral.ConfigCatWidth)
	scrollFrameContainer:SetHeight(sizeGeneral.Offset+(10*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	scrollFrameContainer:SetPoint("TOP", textSelectSQ, "BOTTOM", 0, 0)
	scrollFrameContainer:SetPoint("LEFT", textSelectSQ, "LEFT", 0, 0)
	scrollFrameContainer:Hide()

	local scrollFrame = CreateFrame("ScrollFrame", "AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame", scrollFrameContainer)
	TemplateScrollFrame(scrollFrame)
	scrollFrame:SetPoint("TOPLEFT", 0, 0)
	scrollFrame:SetPoint("BOTTOMRIGHT", 0, 0)
	scrollFrame:SetScript("OnVerticalScroll", function(self, value) AHsearch:OnVerticalScroll("AHsearch_ConfigurationFrame_SearchQueries_SelectSQ", value) end)

	for i = 1, maxQuery do
		local button = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i, scrollFrameContainer)
		button:SetWidth(sizeGeneral.ConfigQueryButtonSFWidth)
		button:SetHeight(sizeGeneral.BarHeight)
		button:Hide()
		if i == 1 then
			button:SetPoint("TOPLEFT", "AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame", "TOPLEFT", sizeGeneral.Offset, -sizeGeneral.Offset)
		else
			button:SetPoint("TOPLEFT", "AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..(i-1), "BOTTOMLEFT", 0, 0)
		end

		local text = button:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Text", "ARTWORK", "GameFontNormalSmall")
		text:SetWidth(sizeGeneral.ConfigQueryTextSFWidth)
		text:SetHeight(sizeGeneral.BarHeight)
		text:SetPoint("LEFT", 0, 0)
		text:SetJustifyH("LEFT")

		local selected = button:CreateTexture("AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Selected", "ARTWORK")
		selected:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		selected:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		selected:SetTexture(1, 1, 1, 0.1)
		selected:Hide()
		local selectedArrow = button:CreateTexture("AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."SelectedArrow", "ARTWORK")
		selectedArrow:SetWidth(Textures.SelectArrow.width)
		selectedArrow:SetHeight(Textures.SelectArrow.height)
		selectedArrow:SetPoint("LEFT", -sizeGeneral.Offset, 0)
		selectedArrow:SetTexture("Interface\\Addons\\AHsearch\\AHsearch-textures-icons")
		selectedArrow:SetTexCoord(unpack(Textures.SelectArrow.coordsR))
		selectedArrow:Hide()

		local order = button:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Order", "ARTWORK", "GameFontWhiteSmall")
		order:SetWidth(30)
		order:SetHeight(sizeGeneral.BarHeight)
		order:SetPoint("RIGHT", button, "LEFT", -(sizeGeneral.Offset+sizeGeneral.Space), 0)
		order:SetJustifyH("RIGHT")
		order:SetTextColor(0, 0.75, 0) -- ORDER_COLOR

		button:SetScript("OnClick", function(self) AHsearch:QueryFrame_ButtonEventUpdate(self.cat, self.sq) end)

		local highlight = button:CreateTexture(nil, "ARTWORK")
		highlight:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
		highlight:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0)
		highlight:SetTexture(1, 1, 1, 0.2)
		button:SetHighlightTexture(highlight)
	end
	-- - SearchQueries selectSQ END

	-- - SearchQueries move START
	local movePullDown = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_Move", contentSearchQueries)
	TemplatePullDownMenu(movePullDown, "", sizeGeneral.ConfigCatWidth, GUI_CatFrame_data, 2, ConfigurationFrame_SearchQueries_SetMove)
	movePullDown:SetWidth(sizeGeneral.ConfigCatWidth)
	movePullDown:SetHeight(18)
	movePullDown:SetPoint("TOP", "AHsearch_ConfigurationFrame_SearchQueries_SelectSQ", "BOTTOM", 0, -10)
	movePullDown:SetPoint("RIGHT", "AHsearch_ConfigurationFrame_SearchQueries_SelectSQ", "RIGHT", 0, 0)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Move")

	local textMove = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_MoveTitle", "ARTWORK", "GameFontNormalSmall")
	textMove:SetHeight(20)
	textMove:SetPoint("RIGHT", movePullDown, "LEFT", -10, 0)
	textMove:SetJustifyH("RIGHT")
	textMove:SetText(L["Move to"]..":")
	textMove:SetTextColor(0.5, 0.5, 0.5)

	local moveAlertIcon = movePullDown:CreateTexture("AHsearch_ConfigurationFrame_SearchQueries_MoveAlertIcon", "ARTWORK")
	moveAlertIcon:SetHeight(24)
	moveAlertIcon:SetWidth(24)
	moveAlertIcon:SetPoint("LEFT", movePullDown, "RIGHT", 1, 2)
	moveAlertIcon:SetTexture("Interface\\DialogFrame\\DialogAlertIcon")
	moveAlertIcon:SetTexCoord(11/64, 52/64, 16/64, 48/64)
	moveAlertIcon:Hide()
	-- - SearchQueries move END

	-- - SearchQueries title START
	local textTitle = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_TitleSQTitle", "ARTWORK", "GameFontNormalSmall")
	textTitle:SetHeight(20)
	textTitle:SetPoint("TOP", "AHsearch_ConfigurationFrame_SearchQueries_Move", "BOTTOM", 0, -10)
	textTitle:SetPoint("LEFT", contentSearchQueries, "LEFT", 10, 0)
	textTitle:SetJustifyH("LEFT")
	textTitle:SetText(L["Title"]..":")
	textTitle:SetTextColor(0.5, 0.5, 0.5)

	local editbox = CreateFrame("EditBox", "AHsearch_ConfigurationFrame_SearchQueries_TitleSQ", contentSearchQueries, "InputBoxTemplate")
	TemplateEditBox(editbox)
	editbox:SetWidth(sizeGeneral.ConfigTextWidth)
	editbox:SetPoint("LEFT", textTitle, "RIGHT", 10, 0)
	editbox:SetMaxLetters(maxBoxInputLength)
	editbox:SetScript("OnTextChanged",  function(self)
		if AHsearch_ConfigurationFrame_SearchQueries.editType == "new" then
			return
		end
		local text = self:GetText()
		ConfigurationFrame_SearchQueries_SetTitleSQ(text)
	end)
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_TitleSQ")

	local textTitleNote = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_TitleSQNote", "ARTWORK", "GameFontNormalSmall")
	textTitleNote:SetHeight(20)
	textTitleNote:SetPoint("LEFT", AHsearch_ConfigurationFrame_SearchQueries_TitleSQ, "RIGHT", 5, 0)
	textTitleNote:SetJustifyH("LEFT")
	textTitleNote:SetText(L["*Required"])
	textTitleNote:SetTextColor(0.5, 0.5, 0.5) -- NOTETEXT
	-- - SearchQueries title END

	-- - SearchQueries query START
	local textNameQuery = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQTitle", "ARTWORK", "GameFontNormalSmall")
	textNameQuery:SetHeight(20)
	textNameQuery:SetPoint("TOP", "AHsearch_ConfigurationFrame_SearchQueries_TitleSQ", "BOTTOM", 0, -10)
	textNameQuery:SetPoint("LEFT", contentSearchQueries, "LEFT", 10, 0)
	textNameQuery:SetJustifyH("LEFT")
	textNameQuery:SetText(L["Name"]..":")
	textNameQuery:SetTextColor(0.5, 0.5, 0.5)

	local editbox = CreateFrame("EditBox", "AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ", contentSearchQueries, "InputBoxTemplate")
	TemplateEditBox(editbox)
	editbox:SetWidth(sizeGeneral.ConfigTextWidth)
	editbox:SetPoint("LEFT", textNameQuery, "RIGHT", 10, 0)
	editbox:SetMaxLetters(maxBoxInputLength)
	editbox:SetScript("OnTextChanged",  function(self)
		local text = self:GetText()
		AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value = text
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ")

	local textNameQueryNote = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQNote", "ARTWORK", "GameFontNormalSmall")
	textNameQueryNote:SetHeight(20)
	textNameQueryNote:SetPoint("LEFT", AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ, "RIGHT", 5, 0)
	textNameQueryNote:SetJustifyH("LEFT")
	textNameQueryNote:SetText(L["*Searchfield"])
	textNameQueryNote:SetTextColor(0.5, 0.5, 0.5) -- NOTETEXT
	-- - SearchQueries query END

	-- - SearchQueries order START
	local textOrder = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_OrderTitle", "ARTWORK", "GameFontNormalSmall")
	textOrder:SetHeight(20)
	textOrder:SetPoint("TOPLEFT", textNameQuery, "BOTTOMLEFT", 0, -10)
	textOrder:SetJustifyH("LEFT")
	textOrder:SetText(L["Position"]..":")
	textOrder:SetTextColor(0.5, 0.5, 0.5)

	local orderEditbox = CreateFrame("EditBox", "AHsearch_ConfigurationFrame_SearchQueries_Order", contentSearchQueries, "InputBoxTemplate")
	TemplateEditBox(orderEditbox)
	orderEditbox:SetWidth(40)
	orderEditbox:SetPoint("LEFT", textOrder, "RIGHT", 10, 0)
	orderEditbox:SetNumeric(true)
	orderEditbox:SetMaxLetters(2)
	orderEditbox:SetTextColor(0, 0.75, 0) -- ORDER_COLOR
	orderEditbox:SetScript("OnTextChanged",  function(self)
		local number = AHsearch_ConfigurationFrame_SearchQueries_Order:GetNumber()
		if number == 0 then
			self:SetNumber(1)
			number = 1
		end
		AHsearch_ConfigurationFrame_SearchQueries_Order.value = number
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_Order")
	-- - SearchQueries order END

	-- - SearchQueries rarity START
	local textRarity = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_RarityTitle", "ARTWORK", "GameFontNormalSmall")
	textRarity:SetHeight(20)
	textRarity:SetPoint("TOPLEFT", textOrder, "BOTTOMLEFT", 0, -10)
	textRarity:SetJustifyH("LEFT")
	textRarity:SetText(L["Rarity"]..":")
	textRarity:SetTextColor(0.5, 0.5, 0.5)

	local rarityPullDown = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_Rarity", contentSearchQueries)
	TemplatePullDownMenu(rarityPullDown, "", sizeGeneral.ConfigPullDownColorWidth, itemRarity, false, ConfigurationFrame_SearchQueries_SetCheck)
	rarityPullDown:SetWidth(sizeGeneral.ConfigPullDownColorWidth)
	rarityPullDown:SetHeight(18)
	rarityPullDown:SetPoint("LEFT", textRarity, "RIGHT", 10, 0)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Rarity")
	-- - SearchQueries rarity END

	-- - SearchQueries filter START
	local textFilter = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_FilterTitle", "ARTWORK", "GameFontNormalSmall")
	textFilter:SetHeight(20)
	textFilter:SetPoint("TOPLEFT", textRarity, "BOTTOMLEFT", 0, -10)
	textFilter:SetJustifyH("LEFT")
	textFilter:SetText(L["Filter"]..":")
	textFilter:SetTextColor(0.5, 0.5, 0.5)

	local filter1PullDown = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_Filter1", contentSearchQueries)
	TemplatePullDownMenu(filter1PullDown, "", sizeGeneral.ConfigPullDownFilterWidth, Filter1Table, false, ConfigurationFrame_SearchQueries_SetFilter1)
	filter1PullDown:SetWidth(sizeGeneral.ConfigPullDownFilterWidth)
	filter1PullDown:SetHeight(18)
	filter1PullDown:SetPoint("LEFT", textFilter, "RIGHT", 10, 0)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter1")
	local filter2PullDown = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_Filter2", contentSearchQueries)
	TemplatePullDownMenu(filter2PullDown, "", sizeGeneral.ConfigPullDownFilterWidth, Filter2Table, false, ConfigurationFrame_SearchQueries_SetFilter2)
	filter2PullDown:SetWidth(sizeGeneral.ConfigPullDownFilterWidth)
	filter2PullDown:SetHeight(18)
	filter2PullDown:SetPoint("LEFT", filter1PullDown, "RIGHT", 2, 0)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter2")
	local filter3PullDown = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_Filter3", contentSearchQueries)
	TemplatePullDownMenu(filter3PullDown, "", sizeGeneral.ConfigPullDownFilterWidth, Filter3Table, false, ConfigurationFrame_SearchQueries_SetFilter3)
	filter3PullDown:SetWidth(sizeGeneral.ConfigPullDownFilterWidth)
	filter3PullDown:SetHeight(18)
	filter3PullDown:SetPoint("LEFT", filter2PullDown, "RIGHT", 2, 0)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter3")
	-- - SearchQueries filter END

	-- - SearchQueries level START
	local textLevel = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_LevelTitle", "ARTWORK", "GameFontNormalSmall")
	textLevel:SetHeight(20)
	textLevel:SetPoint("LEFT", contentSearchQueries, "LEFT", 10, 0)
	textLevel:SetPoint("TOP", "AHsearch_ConfigurationFrame_SearchQueries_Filter1", "BOTTOM", 0, -10)
	textLevel:SetJustifyH("LEFT")
	textLevel:SetText(L["Level"]..":")
	textLevel:SetTextColor(0.5, 0.5, 0.5)

	local textLevelMin = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_LevelMinTitle", "ARTWORK", "GameFontNormalSmall")
	textLevelMin:SetHeight(20)
	textLevelMin:SetPoint("LEFT", textLevel, "RIGHT", 10, 0)
	textLevelMin:SetJustifyH("LEFT")
	textLevelMin:SetText(L["min."]..":")
	textLevelMin:SetTextColor(0.5, 0.5, 0.5)
	local levelMinEditbox = CreateFrame("EditBox", "AHsearch_ConfigurationFrame_SearchQueries_LevelMin", contentSearchQueries, "InputBoxTemplate")
	TemplateEditBox(levelMinEditbox)
	levelMinEditbox:SetWidth(40)
	levelMinEditbox:SetPoint("LEFT", textLevelMin, "RIGHT", 10, 0)
	levelMinEditbox:SetNumeric(true)
	levelMinEditbox:SetMaxLetters(2)
	levelMinEditbox:SetScript("OnTextChanged",  function(self)
		local number = AHsearch_ConfigurationFrame_SearchQueries_LevelMin:GetNumber()
		if number == 0 then
			self:SetNumber("")
			number = nil
		end
		AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value = number
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_LevelMin")

	local textLevelMax = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_LevelMaxTitle", "ARTWORK", "GameFontNormalSmall")
	textLevelMax:SetHeight(20)
	textLevelMax:SetPoint("LEFT", AHsearch_ConfigurationFrame_SearchQueries_LevelMin, "RIGHT", 10, 0)
	textLevelMax:SetJustifyH("LEFT")
	textLevelMax:SetText(L["max."]..":")
	textLevelMax:SetTextColor(0.5, 0.5, 0.5)
	local levelMaxEditbox = CreateFrame("EditBox", "AHsearch_ConfigurationFrame_SearchQueries_LevelMax", contentSearchQueries, "InputBoxTemplate")
	TemplateEditBox(levelMaxEditbox)
	levelMaxEditbox:SetWidth(40)
	levelMaxEditbox:SetPoint("LEFT", textLevelMax, "RIGHT", 10, 0)
	levelMaxEditbox:SetNumeric(true)
	levelMaxEditbox:SetMaxLetters(2)
	levelMaxEditbox:SetScript("OnTextChanged",  function(self)
		local number = AHsearch_ConfigurationFrame_SearchQueries_LevelMax:GetNumber()
		if number == 0 then
			self:SetNumber("")
			number = nil
		end
		AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value = number
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_LevelMax")
	-- - SearchQueries level END

	-- - SearchQueries sort START
	local textSort = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_SortTitle", "ARTWORK", "GameFontNormalSmall")
	textSort:SetHeight(20)
	textSort:SetPoint("LEFT", contentSearchQueries, "LEFT", 10, 0)
	textSort:SetPoint("TOP", "AHsearch_ConfigurationFrame_SearchQueries_LevelMin", "BOTTOM", 0, -10)
	textSort:SetJustifyH("LEFT")
	textSort:SetText(L["Sort"]..":")
	textSort:SetTextColor(0.5, 0.5, 0.5)

	local sortByPullDown = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_SortBy", contentSearchQueries)
	TemplatePullDownMenu(sortByPullDown, "", sizeGeneral.ConfigPullDownSortWidth, SortByTable, false, ConfigurationFrame_SearchQueries_SetCheck)
	sortByPullDown:SetWidth(sizeGeneral.ConfigPullDownSortWidth)
	sortByPullDown:SetHeight(18)
	sortByPullDown:SetPoint("LEFT", textSort, "RIGHT", 10, 0)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_SortBy")

	local sortOrderAscCheckButton = CreateFrame("CheckButton", "AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc", contentSearchQueries)
	TemplateCheckButton(sortOrderAscCheckButton, 16, 4, nil, 1)
	sortOrderAscCheckButton:SetPoint("LEFT", sortByPullDown, "RIGHT", 10, 0)
	sortOrderAscCheckButton:SetChecked(false)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc", true)

	local sortOrderDescCheckButton = CreateFrame("CheckButton", "AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc", contentSearchQueries)
	TemplateCheckButton(sortOrderDescCheckButton, 16, 4, nil, 2)
	sortOrderDescCheckButton:SetPoint("LEFT", sortOrderAscCheckButton, "RIGHT", 10, 0)
	sortOrderDescCheckButton:SetChecked(false)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc", true)

	sortOrderAscCheckButton:SetScript("OnClick", function()
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value = true
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc:SetChecked(true)
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc.value = false
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc:SetChecked(false)
		ConfigurationFrame_SearchQueries_SetCheck()
	end)

	sortOrderDescCheckButton:SetScript("OnClick", function()
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc.value = true
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc:SetChecked(true)
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value = false
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc:SetChecked(false)
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	-- - SearchQueries sort END

	-- - SearchQueries misc START
	local textMisc = contentSearchQueries:CreateFontString("AHsearch_ConfigurationFrame_SearchQueries_MiscTitle", "ARTWORK", "GameFontNormalSmall")
	textMisc:SetHeight(20)
	textMisc:SetPoint("LEFT", contentSearchQueries, "LEFT", 10, 0)
	textMisc:SetPoint("TOP", "AHsearch_ConfigurationFrame_SearchQueries_SortBy", "BOTTOM", 0, -10)
	textMisc:SetJustifyH("LEFT")
	textMisc:SetText(L["Misc."]..":")
	textMisc:SetTextColor(0.5, 0.5, 0.5)

	local usableCheckButton = CreateFrame("CheckButton", "AHsearch_ConfigurationFrame_SearchQueries_Usable", contentSearchQueries)
	TemplateCheckButton(usableCheckButton, 16, 4, L["Usable Items"])
	usableCheckButton:SetPoint("LEFT", textMisc, "RIGHT", 10, 0)
	usableCheckButton:SetChecked(0)
	usableCheckButton:SetScript("OnClick", function(self)
		if self:GetChecked() then
			self.value = 1
			self:SetChecked(1)
		else
			self.value = 0
			self:SetChecked(0)
		end
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_Usable")

	local displayOnCharCheckButton = CreateFrame("CheckButton", "AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar", contentSearchQueries)
	TemplateCheckButton(displayOnCharCheckButton, 16, 4, L["Display on Character"])
	displayOnCharCheckButton:SetPoint("LEFT", usableCheckButton, "RIGHT", 10, 0)
	displayOnCharCheckButton:SetChecked(0)
	displayOnCharCheckButton:SetScript("OnClick", function(self)
		if self:GetChecked() then
			self.value = 1
			self:SetChecked(1)
		else
			self.value = 0
			self:SetChecked(0)
		end
		ConfigurationFrame_SearchQueries_SetCheck()
	end)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar")
	-- - SearchQueries misc END

	-- - SearchQueries save START
	local saveSQ = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_SaveSQ", contentSearchQueries)
	TemplateTextButton(saveSQ, L["Update"], 1)
	saveSQ:SetWidth(200)
	saveSQ:SetHeight(24)
	saveSQ:SetPoint("TOP", "AHsearch_ConfigurationFrame_SearchQueries_Usable", "BOTTOM", 0, -20)
	saveSQ:SetPoint("LEFT", contentSearchQueries, "LEFT", (mainWidth-200-10-200)/2, 0)
	saveSQ:SetScript("OnClick", function() AHsearch:ConfigurationFrame_SearchQueries_SaveSQ() end)
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ")
	-- - SearchQueries save END

	-- - SearchQueries delete START
	local deleteSQ = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_DeleteSQ", contentSearchQueries)
	TemplateTextButton(deleteSQ,  L["Delete"], 1)
	deleteSQ:SetWidth(200)
	deleteSQ:SetHeight(24)
	deleteSQ:SetPoint("LEFT", saveSQ, "RIGHT", 10, 0)
	deleteSQ:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_DeleteSQ() end)
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_DeleteSQ")
	-- - SearchQueries delete END

	-- - SearchQueries clearall START
	local clearallSearchQueries = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_ClearAllSQ", contentSearchQueries)
	TemplateTextButton(clearallSearchQueries, L["Clear All"], 2)
	clearallSearchQueries:SetWidth(120)
	clearallSearchQueries:SetHeight(18)
	clearallSearchQueries:SetPoint("TOP", "AHsearch_ConfigurationFrame_SearchQueries_DeleteSQ", "BOTTOM", 0, -10)
	clearallSearchQueries:SetPoint("LEFT", contentSearchQueries, "LEFT", (mainWidth-120-10-120)/2, 0)
	clearallSearchQueries:SetScript("OnClick", function() AHsearch:ConfigurationFrame_ClearAll() end)
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_ClearAllSQ")
	-- - SearchQueries clearall END

	-- testsearch
	local testsearch = CreateFrame("Button", "AHsearch_ConfigurationFrame_SearchQueries_TestSearch", contentSearchQueries)
	TemplateTextButton(testsearch, L["TEST SEARCH"], 3)
	testsearch:SetWidth(120)
	testsearch:SetHeight(18)
	testsearch:SetPoint("LEFT", clearallSearchQueries, "RIGHT", 10, 0)
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_TestSearch")
	testsearch:SetScript("OnClick", function() AHsearch:ConfigurationFrame_SearchQueries_TestSearch() end)
	testsearch:SetScript("OnEnter", function() if not hideMain then AHsearch_CatFrame:Hide() end end)
	testsearch:SetScript("OnLeave", function() if not hideMain then AHsearch_CatFrame:Show() end end)
	-- testsearch

	-- - SearchQueries reposition START
	local nw = 10
	local cw = 0
	cw = textCreate:GetStringWidth()    if cw > nw then nw = cw end
	cw = textTitle:GetStringWidth()     if cw > nw then nw = cw end
	cw = textNameQuery:GetStringWidth() if cw > nw then nw = cw end
	cw = textOrder:GetStringWidth()     if cw > nw then nw = cw end
	cw = textRarity:GetStringWidth()    if cw > nw then nw = cw end
	cw = textFilter:GetStringWidth()    if cw > nw then nw = cw end
	cw = textLevel:GetStringWidth()     if cw > nw then nw = cw end
	cw = textSort:GetStringWidth()      if cw > nw then nw = cw end
	cw = textMisc:GetStringWidth()      if cw > nw then nw = cw end
	nw = floor(nw+10)
	if nw < 70 then nw = 70 end

	AHsearch_ConfigurationFrame_SearchQueries_NewSQ:SetPoint("LEFT", textCreate, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_SearchQueries_TitleSQ:SetPoint("LEFT", textTitle, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ:SetPoint("LEFT", textNameQuery, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_SearchQueries_Order:SetPoint("LEFT", textOrder, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_SearchQueries_Rarity:SetPoint("LEFT", textRarity, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_SearchQueries_Filter1:SetPoint("LEFT", textFilter, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMinTitle:SetPoint("LEFT", textLevel, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_SearchQueries_SortBy:SetPoint("LEFT", textSort, "LEFT", nw, 0)
	AHsearch_ConfigurationFrame_SearchQueries_Usable:SetPoint("LEFT", textMisc, "LEFT", nw, 0)
	-- - SearchQueries reposition END
	-- SearchQueries END ----------------------------------------

	-- Options START ----------------------------------------
	local contentOptions = CreateFrame("Frame", "AHsearch_ConfigurationFrame_Options_Content", configframe)
	TemplateBorderTRBL(contentOptions)
	contentOptions:SetWidth(mainWidth)
	contentOptions:SetHeight(mainHeight-topheight+sizeGeneral.Border)
	contentOptions:SetPoint("TOPLEFT", selection, "BOTTOMLEFT", 0, 1)
	contentOptions:EnableMouse(true)
	contentOptions:Hide()

	-- - button settings START
	local textPrevNext = contentOptions:CreateFontString("AHsearch_ConfigurationFrame_Options_CyclePrevNextTitle", "ARTWORK", "GameFontNormalSmall")
	textPrevNext:SetHeight(20)
	textPrevNext:SetPoint("TOPLEFT", contentOptions, "TOPLEFT", 10, -10)
	textPrevNext:SetJustifyH("LEFT")
	textPrevNext:SetText(L["Button settings"]..":")

	local prevNextCheckButton = CreateFrame("CheckButton", "AHsearch_ConfigurationFrame_Options_CyclePrevNext", contentOptions)
	TemplateCheckButton(prevNextCheckButton, 16, 4, L["Show Button for Next and Previous Search"])
	prevNextCheckButton:SetPoint("TOP", textPrevNext, "BOTTOM", 0, -10)
	prevNextCheckButton:SetPoint("LEFT", textPrevNext, "LEFT", 20, 0)
	prevNextCheckButton:SetChecked(AHsearch_Options.cycleNP)
	prevNextCheckButton:SetScript("OnClick", function(self)
		AHsearch_Options.cycleNP = not AHsearch_Options.cycleNP
		self:SetChecked(AHsearch_Options.cycleNP)
		AHsearch:CycleButtons_Toggle()
	end)

	local randomCheckButton = CreateFrame("CheckButton", "AHsearch_ConfigurationFrame_Options_CycleRandom", contentOptions)
	TemplateCheckButton(randomCheckButton, 16, 4, L["Show Button for Random Search"])
	randomCheckButton:SetPoint("TOP", prevNextCheckButton, "BOTTOM", 0, -10)
	randomCheckButton:SetPoint("LEFT", textPrevNext, "LEFT", 20, 0)
	randomCheckButton:SetChecked(AHsearch_Options.cycleRND)
	randomCheckButton:SetScript("OnClick", function(self)
		AHsearch_Options.cycleRND = not AHsearch_Options.cycleRND
		self:SetChecked(AHsearch_Options.cycleRND)
		AHsearch:CycleButtons_Toggle()
	end)
	-- - button settings END

	-- category frame settings START
	local textCatSize = contentOptions:CreateFontString("AHsearch_ConfigurationFrame_Options_CatSizeTitle", "ARTWORK", "GameFontNormalSmall")
	textCatSize:SetHeight(20)
	textCatSize:SetPoint("TOP", randomCheckButton, "BOTTOM", 0, -20)
	textCatSize:SetPoint("LEFT", textPrevNext, "LEFT", 0, 0)
	textCatSize:SetJustifyH("LEFT")
	textCatSize:SetText(L["Category"]..":")

	local textCatWidth = contentOptions:CreateFontString("AHsearch_ConfigurationFrame_Options_CatWidthTitle", "ARTWORK", "GameFontNormalSmall")
	textCatWidth:SetHeight(20)
	textCatWidth:SetPoint("TOP", textCatSize, "BOTTOM", 0, -5)
	textCatWidth:SetPoint("LEFT", textCatSize, "LEFT", 10, 0)
	textCatWidth:SetJustifyH("LEFT")
	textCatWidth:SetText(L["Width"]..":")

	local catWidthSlider = CreateFrame("Slider", "AHsearch_ConfigurationFrame_Options_CatWidth", contentOptions)
	TemplateSlider(catWidthSlider, sizeGeneral.ConfigSliderWidth, 5, 125, 250, AHsearch_Options.curCatWidth, AHsearch.SetupCatWidth)
	catWidthSlider:SetPoint("TOP", textCatWidth, "BOTTOM", 0, -5)
	catWidthSlider:SetPoint("LEFT", textCatWidth, "LEFT", 0, 0)

	local textCatHeight = contentOptions:CreateFontString("AHsearch_ConfigurationFrame_Options_CatHeightTitle", "ARTWORK", "GameFontNormalSmall")
	textCatHeight:SetHeight(20)
	textCatHeight:SetPoint("TOP", textCatWidth, "TOP", 0, 0)
	textCatHeight:SetPoint("LEFT", catWidthSlider, "RIGHT", 10, 0)
	textCatHeight:SetJustifyH("LEFT")
	textCatHeight:SetText(L["Height (Rows)"]..":")

	local catHeightSlider = CreateFrame("Slider", "AHsearch_ConfigurationFrame_Options_CatHeight", contentOptions)
	TemplateSlider(catHeightSlider, sizeGeneral.ConfigSliderWidth, 1, minCat, maxCat, AHsearch_Options.curCat, AHsearch.SetupCatHeight)
	catHeightSlider:SetPoint("TOP", textCatHeight, "BOTTOM", 0, -5)
	catHeightSlider:SetPoint("LEFT", textCatHeight, "LEFT", 0, 0)
	-- category frame settings END

	-- search query frame settings START
	local textSQSize = contentOptions:CreateFontString("AHsearch_ConfigurationFrame_Options_SQSizeTitle", "ARTWORK", "GameFontNormalSmall")
	textSQSize:SetHeight(20)
	textSQSize:SetPoint("TOP", catHeightSlider, "BOTTOM", 0, -30)
	textSQSize:SetPoint("LEFT", textPrevNext, "LEFT", 0, 0)
	textSQSize:SetJustifyH("LEFT")
	textSQSize:SetText(L["Search Query"]..":")

	local textSQWidth = contentOptions:CreateFontString("AHsearch_ConfigurationFrame_Options_SQWidthTitle", "ARTWORK", "GameFontNormalSmall")
	textSQWidth:SetHeight(20)
	textSQWidth:SetPoint("TOP", textSQSize, "BOTTOM", 0, -5)
	textSQWidth:SetPoint("LEFT", textSQSize, "LEFT", 10, 0)
	textSQWidth:SetJustifyH("LEFT")
	textSQWidth:SetText(L["Width"]..":")

	local sqWidthSlider = CreateFrame("Slider", "AHsearch_ConfigurationFrame_Options_SQWidth", contentOptions)
	TemplateSlider(sqWidthSlider, sizeGeneral.ConfigSliderWidth, 5, 125, 250, AHsearch_Options.curQueryWidth, AHsearch.SetupSQWidth)
	sqWidthSlider:SetPoint("TOP", textSQWidth, "BOTTOM", 0, -5)
	sqWidthSlider:SetPoint("LEFT", textSQWidth, "LEFT", 0, 0)

	local textSQHeight = contentOptions:CreateFontString("AHsearch_ConfigurationFrame_Options_SQHeightTitle", "ARTWORK", "GameFontNormalSmall")
	textSQHeight:SetHeight(20)
	textSQHeight:SetPoint("TOP", textSQWidth, "TOP", 0, 0)
	textSQHeight:SetPoint("LEFT", sqWidthSlider, "RIGHT", 10, 0)
	textSQHeight:SetJustifyH("LEFT")
	textSQHeight:SetText(L["Height (Rows)"]..":")

	local sqHeightSlider = CreateFrame("Slider", "AHsearch_ConfigurationFrame_Options_SQHeight", contentOptions)
	TemplateSlider(sqHeightSlider, sizeGeneral.ConfigSliderWidth, 1, minQuery, maxQuery, AHsearch_Options.curQuery, AHsearch.SetupSQHeight)
	sqHeightSlider:SetPoint("TOP", textSQHeight, "BOTTOM", 0, -5)
	sqHeightSlider:SetPoint("LEFT", textSQHeight, "LEFT", 0, 0)
	-- search query frame settings END
	
	-- scale START
	local textScale = contentOptions:CreateFontString("AHsearch_ConfigurationFrame_Options_ScaleTitle", "ARTWORK", "GameFontNormalSmall")
	textScale:SetHeight(20)
	textScale:SetPoint("TOP", sqHeightSlider, "BOTTOM", 0, -30)
	textScale:SetPoint("LEFT", textPrevNext, "LEFT", 0, 0)
	textScale:SetJustifyH("LEFT")
	textScale:SetText(L["Scale"]..":")

	local scaleMainSlider = CreateFrame("Slider", "AHsearch_ConfigurationFrame_Options_ScaleMain", contentOptions)
	TemplateSlider(scaleMainSlider, sizeGeneral.ConfigSliderWidth, 1, 75, 150, AHsearch_Options.scaleMain, AHsearch.SetupMainScale, true)
	scaleMainSlider:SetPoint("TOP", textScale, "BOTTOM", 0, -5)
	scaleMainSlider:SetPoint("LEFT", textScale, "LEFT", 10, 0)
	-- scale END
	-- Options END ----------------------------------------

	-- ConfirmPopup START ----------------------------------------
	local ConfirmPopup = CreateFrame("Frame", "AHsearch_ConfigurationFrame_ConfirmPopup", configframe)
	ConfirmPopup:SetWidth(mainWidth)
	ConfirmPopup:SetHeight(mainHeight-topheight)
	ConfirmPopup:SetPoint("TOPLEFT", selection, "BOTTOMLEFT", 0, 1)
	ConfirmPopup:EnableMouse(true)
	ConfirmPopup:SetToplevel(true)
	ConfirmPopup:Hide()

	local texture = ConfirmPopup:CreateTexture(nil, "BACKGROUND")
	texture:SetPoint("TOPLEFT", ConfirmPopup, "TOPLEFT", 1, -1)
	texture:SetPoint("BOTTOMRIGHT", ConfirmPopup, "BOTTOMRIGHT", -1, 1)
	texture:SetTexture(0, 0, 0, 0.8)

	local text1 = ConfirmPopup:CreateFontString("AHsearch_ConfigurationFrame_ConfirmPopupText1", "OVERLAY", "GameFontNormal")
	text1:SetWidth(mainWidth-50)
	text1:SetPoint("CENTER",0, 0)
	text1:SetJustifyH("CENTER")
	local text2 = ConfirmPopup:CreateFontString("AHsearch_ConfigurationFrame_ConfirmPopupText2", "OVERLAY", "GameFontNormal")
	text2:SetWidth(mainWidth-50)
	text2:SetPoint("TOP", text1, "BOTTOM", 0, -5)
	text2:SetJustifyH("CENTER")
	local text3 = ConfirmPopup:CreateFontString("AHsearch_ConfigurationFrame_ConfirmPopupText3", "OVERLAY", "GameFontNormal")
	text3:SetWidth(mainWidth-50)
	text3:SetPoint("TOP", text2, "BOTTOM", 0, -5)
	text3:SetJustifyH("CENTER")

	local OkayButton = CreateFrame("Button", "AHsearch_ConfigurationFrame_ConfirmPopupOkayButton", ConfirmPopup)
	TemplateTextButton(OkayButton, L["YES"], 1)
	OkayButton:SetWidth(200)
	OkayButton:SetHeight(24)
	OkayButton:SetPoint("TOP", text3, "BOTTOM", 0, -30)
	OkayButton:SetPoint("LEFT", ConfirmPopup, "LEFT", (mainWidth-200-10-200)/2, 0)

	local CancelButton = CreateFrame("Button", "AHsearch_ConfigurationFrame_ConfirmPopupCancelButton", ConfirmPopup)
	TemplateTextButton(CancelButton, L["NO"], 1)
	CancelButton:SetWidth(200)
	CancelButton:SetHeight(24)
	CancelButton:SetPoint("LEFT", OkayButton, "RIGHT", 10, 0)
	CancelButton:SetScript("OnClick", function() AHsearch_ConfigurationFrame_ConfirmPopup:Hide() confirmIsShown = false end)
	-- ConfirmPopup END ----------------------------------------

	-- Mover Frame START ----------------------------------------
	local moverTop = CreateFrame("Frame", "AHsearch_ConfigurationFrame_MoverTopFrame", configframe)
	TemplateBorderTRBL(moverTop, 1)
	moverTop:SetWidth(mainWidth)
	moverTop:SetHeight(20)
	moverTop:SetPoint("BOTTOM", configframe, "TOP", 0, -sizeGeneral.Border)
	moverTop:EnableMouse(true)
	moverTop:EnableMouseWheel(true)
	moverTop:SetScript("OnMouseWheel", NOOP)
	local texture = moverTop:CreateTexture(nil, "BACKGROUND")
	texture:SetAllPoints(moverTop)
	texture:SetTexture(0, 0, 0, 1)
	local movertextTop = moverTop:CreateFontString(nil, "ARTWORK", "GameFontWhiteSmall")
	movertextTop:SetPoint("CENTER", moverTop, "CENTER", 0, 0)
	movertextTop:SetJustifyH("CENTER")
	movertextTop:SetTextColor(0.3, 0.3, 0.3, 1)
	movertextTop:SetText(L["click & move"])

	local moverBottom = CreateFrame("Frame", "AHsearch_ConfigurationFrame_MoverBottomFrame", configframe)
	TemplateBorderTRBL(moverBottom, 1)
	moverBottom:SetWidth(mainWidth)
	moverBottom:SetHeight(20)
	moverBottom:SetPoint("TOP", configframe, "BOTTOM", 0, sizeGeneral.Border)
	moverBottom:EnableMouse(true)
	moverBottom:EnableMouseWheel(true)
	moverBottom:SetScript("OnMouseWheel", NOOP)
	local texture = moverBottom:CreateTexture(nil, "BACKGROUND")
	texture:SetAllPoints(moverBottom)
	texture:SetTexture(0, 0, 0, 1)
	local movertextBottom = moverBottom:CreateFontString(nil, "ARTWORK", "GameFontWhiteSmall")
	movertextBottom:SetPoint("CENTER", moverBottom, "CENTER", 0, 0)
	movertextBottom:SetJustifyH("CENTER")
	movertextBottom:SetTextColor(0.3, 0.3, 0.3, 1)
	movertextBottom:SetText(L["click & move"])

	-- - move scripts START
	moverTop:SetScript("OnEnter", function() movertextTop:SetTextColor(1, 1, 1, 1) end)
	moverTop:SetScript("OnLeave", function() movertextTop:SetTextColor(0.3, 0.3, 0.3, 1) end)
	moverTop:SetScript("OnMouseDown", function() AHsearch_ConfigurationFrame:StartMoving() end)
	moverTop:SetScript("OnMouseUp", function() AHsearch_ConfigurationFrame:StopMovingOrSizing() AHsearch:SaveConfigFramePos() end)

	moverBottom:SetScript("OnEnter", function() movertextBottom:SetTextColor(1, 1, 1, 1) end)
	moverBottom:SetScript("OnLeave", function() movertextBottom:SetTextColor(0.3, 0.3, 0.3, 1) end)
	moverBottom:SetScript("OnMouseDown", function() AHsearch_ConfigurationFrame:StartMoving() end)
	moverBottom:SetScript("OnMouseUp", function() AHsearch_ConfigurationFrame:StopMovingOrSizing() AHsearch:SaveConfigFramePos() end)
	-- - move scripts END
	-- Mover Frame END ----------------------------------------
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:ConfigurationFrame_ShowContent()
	if GUI_CatFrame_Count == 0 then
		DisableTabButton("AHsearch_ConfigurationFrame_SearchQueries")
	else
		EnableTabButton("AHsearch_ConfigurationFrame_SearchQueries")
	end
	if GUI_Tab == 1 then
		SetTabButton("AHsearch_ConfigurationFrame_Categories", true)
		SetTabButton("AHsearch_ConfigurationFrame_SearchQueries", false)
		SetTabButton("AHsearch_ConfigurationFrame_Options", false)
		AHsearch_ConfigurationFrame_Categories_Content:Show()
		AHsearch_ConfigurationFrame_SearchQueries_Content:Hide()
		AHsearch_ConfigurationFrame_Options_Content:Hide()
	elseif GUI_Tab == 2 then
		SetTabButton("AHsearch_ConfigurationFrame_Categories", false)
		SetTabButton("AHsearch_ConfigurationFrame_SearchQueries", true)
		SetTabButton("AHsearch_ConfigurationFrame_Options", false)
		AHsearch_ConfigurationFrame_Categories_Content:Hide()
		AHsearch_ConfigurationFrame_SearchQueries_Content:Show()
		AHsearch_ConfigurationFrame_Options_Content:Hide()
	elseif GUI_Tab == 3 then
		SetTabButton("AHsearch_ConfigurationFrame_Categories", false)
		SetTabButton("AHsearch_ConfigurationFrame_SearchQueries", false)
		SetTabButton("AHsearch_ConfigurationFrame_Options", true)
		AHsearch_ConfigurationFrame_Categories_Content:Hide()
		AHsearch_ConfigurationFrame_SearchQueries_Content:Hide()
		AHsearch_ConfigurationFrame_Options_Content:Show()
	end	
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
-- Categories Actions START  ----------------------------------------
function AHsearch:ConfigurationFrame_Categories_CreateNewCat() -- CREATENEW
	AHsearch_ConfigurationFrame_Categories.editType = "new"
	-- values
	local icon = AHsearch_ConfigurationFrame_Categories_Icon.value
	if not icon then
		icon = 1
		AHsearch_ConfigurationFrame_Categories_Icon.value = icon
	end
	local color = AHsearch_ConfigurationFrame_Categories_Color.value
	if not color then
		color = "|cffffffff"
		AHsearch_ConfigurationFrame_Categories_Color.value = color
	end
	local order = AHsearch_ConfigurationFrame_Categories_Order:GetNumber()
	AHsearch_ConfigurationFrame_Categories_Order.value = order
	local cat = AHsearch_ConfigurationFrame_Categories_NewCat:GetText()
	AHsearch_ConfigurationFrame_Categories_CurrentCatText.value = cat

	AHsearch_ConfigurationFrame_Categories.originalCat = nil

	-- setup display
	AHsearch_ConfigurationFrame_Categories_CurrentCatTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_Categories_CurrentCatTexture:SetTexture(categoryIcons[icon].texture)
	AHsearch_ConfigurationFrame_Categories_CurrentCatTexture:SetTexCoord(unpack(categoryIcons[icon].coords))
	AHsearch_ConfigurationFrame_Categories_CurrentCatText:SetText(color..cat.."|r")

	AHsearch_ConfigurationFrame_Categories_NameCatTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_Categories_NameCat:SetText(cat)
	DisableEditBox("AHsearch_ConfigurationFrame_Categories_NameCat")

	AHsearch_ConfigurationFrame_Categories_OrderTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_Categories_Order:SetNumber(order)
	EnableEditBox("AHsearch_ConfigurationFrame_Categories_Order")

	AHsearch_ConfigurationFrame_Categories_ColorTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_Categories_Color_PullDownMenuText:SetText(color.."###")
	EnablePullDownMenu("AHsearch_ConfigurationFrame_Categories_Color")

	AHsearch_ConfigurationFrame_Categories_IconTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_Categories_Icon_PullDownMenuTexture:SetTexture(categoryIcons[icon].texture)
	AHsearch_ConfigurationFrame_Categories_Icon_PullDownMenuTexture:SetTexCoord(unpack(categoryIcons[icon].coords))
	EnablePullDownMenu("AHsearch_ConfigurationFrame_Categories_Icon")

	for i = 1, maxCat do
		_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Hide()
	end

	-- action buttons
	EnableTextButton("AHsearch_ConfigurationFrame_Categories_ClearAllCat", 2)
	AHsearch_ConfigurationFrame_Categories_UpdateCat:SetText(L["Create New"])
	EnableTextButton("AHsearch_ConfigurationFrame_Categories_UpdateCat", 1)
	DisableTextButton("AHsearch_ConfigurationFrame_Categories_DeleteCat")
end

function AHsearch:ConfigurationFrame_Categories_SelectCat(cat)
	AHsearch_ConfigurationFrame_Categories.editType = "selected"
	-- values
	local icon = AHsearch_Data[cat].icon or 1
	local color = AHsearch_Data[cat].color or "|cffffffff"
	local order = AHsearch_Data[cat].order or 1
	AHsearch_ConfigurationFrame_Categories_Order.value = order
	AHsearch_ConfigurationFrame_Categories_Color.value = color
	AHsearch_ConfigurationFrame_Categories_Icon.value = icon
	AHsearch_ConfigurationFrame_Categories_CurrentCatText.value = cat

	AHsearch_ConfigurationFrame_Categories.originalCat = cat
	AHsearch_ConfigurationFrame_Categories.originalOrder = order
	AHsearch_ConfigurationFrame_Categories.originalColor = color
	AHsearch_ConfigurationFrame_Categories.originalIcon = icon

	-- setup display
	AHsearch_ConfigurationFrame_Categories_NewCat:ClearFocus()
	AHsearch_ConfigurationFrame_Categories_NewCat:SetText("")

	AHsearch_ConfigurationFrame_Categories_CurrentCatTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_Categories_CurrentCatTexture:SetTexture(categoryIcons[icon].texture)
	AHsearch_ConfigurationFrame_Categories_CurrentCatTexture:SetTexCoord(unpack(categoryIcons[icon].coords))
	AHsearch_ConfigurationFrame_Categories_CurrentCatText:SetText(color..cat.."|r")

	AHsearch_ConfigurationFrame_Categories_NameCatTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_Categories_NameCat:SetText(cat)
	AHsearch_ConfigurationFrame_Categories_NameCat:ClearFocus()
	EnableEditBox("AHsearch_ConfigurationFrame_Categories_NameCat")

	AHsearch_ConfigurationFrame_Categories_OrderTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_Categories_Order:SetNumber(order)
	AHsearch_ConfigurationFrame_Categories_Order:ClearFocus()
	EnableEditBox("AHsearch_ConfigurationFrame_Categories_Order")

	AHsearch_ConfigurationFrame_Categories_ColorTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_Categories_Color_PullDownMenuText:SetText(color.."###")
	EnablePullDownMenu("AHsearch_ConfigurationFrame_Categories_Color")

	AHsearch_ConfigurationFrame_Categories_IconTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_Categories_Icon_PullDownMenuTexture:SetTexture(categoryIcons[icon].texture)
	AHsearch_ConfigurationFrame_Categories_Icon_PullDownMenuTexture:SetTexCoord(unpack(categoryIcons[icon].coords))
	EnablePullDownMenu("AHsearch_ConfigurationFrame_Categories_Icon")

	-- action buttons
	EnableTextButton("AHsearch_ConfigurationFrame_Categories_ClearAllCat", 2)
	AHsearch_ConfigurationFrame_Categories_UpdateCat:SetText(L["Update"])
	DisableTextButton("AHsearch_ConfigurationFrame_Categories_UpdateCat")
	EnableTextButton("AHsearch_ConfigurationFrame_Categories_DeleteCat", 1)
end

function AHsearch:ConfigurationFrame_Categories_ClearAllCat()
	AHsearch_ConfigurationFrame_Categories.editType = nil
	-- values
	AHsearch_ConfigurationFrame_Categories_Order.value = nil
	AHsearch_ConfigurationFrame_Categories_Color.value = nil
	AHsearch_ConfigurationFrame_Categories_Icon.value = nil
	AHsearch_ConfigurationFrame_Categories_CurrentCatText.value = nil

	-- setup display
	AHsearch_ConfigurationFrame_Categories_NewCat:SetText("")
	AHsearch_ConfigurationFrame_Categories_NewCat:ClearFocus()

	AHsearch_ConfigurationFrame_Categories_CurrentCatTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_Categories_CurrentCatTexture:SetTexture(nil)
	AHsearch_ConfigurationFrame_Categories_CurrentCatText:SetText("")

	AHsearch_ConfigurationFrame_Categories_NameCatTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_Categories_NameCat:SetText("")
	DisableEditBox("AHsearch_ConfigurationFrame_Categories_NameCat")

	AHsearch_ConfigurationFrame_Categories_OrderTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_Categories_Order:SetNumber(1)
	DisableEditBox("AHsearch_ConfigurationFrame_Categories_Order")

	AHsearch_ConfigurationFrame_Categories_ColorTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_Categories_Color_PullDownMenuText:SetText("")
	DisablePullDownMenu("AHsearch_ConfigurationFrame_Categories_Color")

	AHsearch_ConfigurationFrame_Categories_IconTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_Categories_Icon_PullDownMenuTexture:SetTexture(nil)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_Categories_Icon")

	for i = 1, maxCat do
		_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_CatFrameScrollFrame_Button"..i.."SelectedArrow"]:Hide()
		_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Hide()
	end

	-- action buttons
	DisableTextButton("AHsearch_ConfigurationFrame_Categories_ClearAllCat")
	DisableTextButton("AHsearch_ConfigurationFrame_Categories_UpdateCat")
	DisableTextButton("AHsearch_ConfigurationFrame_Categories_DeleteCat")
end

function AHsearch:ConfigurationFrame_Categories_UpdateCat(confirm) --SAVEDATA
	local originalCat = AHsearch_ConfigurationFrame_Categories.originalCat
	local currentCat = AHsearch_ConfigurationFrame_Categories_CurrentCatText.value

	if originalCat then
		-- update existing category START
		if AHsearch_Data[currentCat] and originalCat ~= currentCat then
			-- - category exists START
			if confirm then
				-- - category exists confirm -> action START
				local oldqueries = AHsearch_Data[originalCat].queries
				
				AHsearch_Data[originalCat] = nil
				AHsearch_Data[currentCat] = nil
				AHsearch_Data[currentCat] = {}
				AHsearch_Data[currentCat].order = AHsearch_ConfigurationFrame_Categories_Order.value
				AHsearch_Data[currentCat].icon = AHsearch_ConfigurationFrame_Categories_Icon.value
				AHsearch_Data[currentCat].color = AHsearch_ConfigurationFrame_Categories_Color.value

				if oldqueries then
					AHsearch_Data[currentCat].queries = {}
					AHsearch_Data[currentCat].queries = oldqueries
				end

				GUI_curCat = currentCat --LLL
				AHsearch:BuildStandardTable()
				AHsearch:CatFrame_Show(true)
				AHsearch:QueryFrame_Show(GUI_curCat)
				AHsearch:FillList("AHsearch_ConfigurationFrame_Categories_SelectCat")
				AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectCat")
				AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenu")
				AHsearch:ConfigurationFrame_Categories_SelectCat(currentCat)
				AHsearch:ConfigurationFrame_SearchQueries_SelectCat(GUI_curCat)
				AHsearch_ConfigurationFrame_ConfirmPopup:Hide()
				confirmIsShown = false
				-- - category exists confirm -> action END
			else
				-- - category exists confirm popup START
				AHsearch_ConfigurationFrame_Categories_NewCat:ClearFocus()
				AHsearch_ConfigurationFrame_Categories_NameCat:ClearFocus()
				AHsearch_ConfigurationFrame_Categories_Order:ClearFocus()
				AHsearch_ConfigurationFrame_ConfirmPopupText1:SetText( string.format(L["Category '%s' exists!"], "|cffffffff"..currentCat.."|r") )
				AHsearch_ConfigurationFrame_ConfirmPopupText2:SetText( L["Overwrite?"] )
				AHsearch_ConfigurationFrame_ConfirmPopupText3:SetText( L["All search queries in the old category will be deleted!"] )
				AHsearch_ConfigurationFrame_ConfirmPopupOkayButton:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_UpdateCat(true) end)
				AHsearch_ConfigurationFrame_ConfirmPopup:Show()
				confirmIsShown = true
				-- - category exists confirm popup END
			end
			-- - category exists END
		else
			-- - category does not exists -> make new with old query data (if exists) -> simple rename START
			local oldqueries = AHsearch_Data[originalCat].queries

			AHsearch_Data[originalCat] = nil
			AHsearch_Data[currentCat] = nil
			AHsearch_Data[currentCat] = {}
			AHsearch_Data[currentCat].order = AHsearch_ConfigurationFrame_Categories_Order.value
			AHsearch_Data[currentCat].icon = AHsearch_ConfigurationFrame_Categories_Icon.value
			AHsearch_Data[currentCat].color = AHsearch_ConfigurationFrame_Categories_Color.value

			if oldqueries then
				AHsearch_Data[currentCat].queries = {}
				AHsearch_Data[currentCat].queries = oldqueries
			end

			GUI_curCat = currentCat --LLL
			AHsearch:BuildStandardTable()
			AHsearch:CatFrame_Show(true)
			AHsearch:QueryFrame_Show(GUI_curCat)
			AHsearch:FillList("AHsearch_ConfigurationFrame_Categories_SelectCat")
			AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectCat")
			AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenu")
			AHsearch:ConfigurationFrame_Categories_SelectCat(currentCat)
			AHsearch:ConfigurationFrame_SearchQueries_SelectCat(GUI_curCat)
			-- - category does not exists -> make new with old query data (if exists) -> simple rename END
		end
		-- update existing category END
	else
		-- create new category START
		if AHsearch_Data[currentCat] then
			-- - category exists START
			if confirm then
				-- - category exists confirm -> action START
				AHsearch_Data[currentCat] = nil
				AHsearch_Data[currentCat] = {}
				AHsearch_Data[currentCat].order = AHsearch_ConfigurationFrame_Categories_Order.value
				AHsearch_Data[currentCat].icon = AHsearch_ConfigurationFrame_Categories_Icon.value
				AHsearch_Data[currentCat].color = AHsearch_ConfigurationFrame_Categories_Color.value

				GUI_curCat = currentCat --LLL
				AHsearch:BuildStandardTable()
				AHsearch:CatFrame_Show(true)
				AHsearch:QueryFrame_Show(GUI_curCat)
				AHsearch:FillList("AHsearch_ConfigurationFrame_Categories_SelectCat")
				AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectCat")
				AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenu")
				AHsearch:ConfigurationFrame_Categories_SelectCat(GUI_curCat)
				AHsearch:ConfigurationFrame_SearchQueries_SelectCat(GUI_curCat)
				AHsearch_ConfigurationFrame_ConfirmPopup:Hide()
				confirmIsShown = false
				-- - category exists confirm -> action END
			else
				-- - category exists confirm popup START
				AHsearch_ConfigurationFrame_Categories_NewCat:ClearFocus()
				AHsearch_ConfigurationFrame_Categories_NameCat:ClearFocus()
				AHsearch_ConfigurationFrame_Categories_Order:ClearFocus()
				AHsearch_ConfigurationFrame_ConfirmPopupText1:SetText( string.format(L["Category '%s' exists!"], "|cffffffff"..currentCat.."|r") )
				AHsearch_ConfigurationFrame_ConfirmPopupText2:SetText( L["Overwrite?"] )
				AHsearch_ConfigurationFrame_ConfirmPopupText3:SetText( L["All search queries in the old category will be deleted!"] )
				AHsearch_ConfigurationFrame_ConfirmPopupOkayButton:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_UpdateCat(true) end)
				AHsearch_ConfigurationFrame_ConfirmPopup:Show()
				confirmIsShown = true
				-- - category exists confirm popup END
			end
			-- - category exists END
		else
			-- - category does not exists -> make new START
			AHsearch_Data[currentCat] = {}
			AHsearch_Data[currentCat].order = AHsearch_ConfigurationFrame_Categories_Order.value
			AHsearch_Data[currentCat].icon = AHsearch_ConfigurationFrame_Categories_Icon.value
			AHsearch_Data[currentCat].color = AHsearch_ConfigurationFrame_Categories_Color.value

			GUI_curCat = currentCat --LLL
			AHsearch:BuildStandardTable()
			AHsearch:CatFrame_Show(true)
			AHsearch:QueryFrame_Show(GUI_curCat)
			AHsearch:FillList("AHsearch_ConfigurationFrame_Categories_SelectCat")
			AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectCat")
			AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenu")
			AHsearch:ConfigurationFrame_Categories_SelectCat(GUI_curCat)
			AHsearch:ConfigurationFrame_SearchQueries_SelectCat(GUI_curCat)
			-- - category does not exists -> make new END
		end
		-- create new category END
	end
	AHsearch:ConfigurationFrame_ShowContent() -- update tabs
end

function AHsearch:ConfigurationFrame_Categories_DeleteCat(confirm)
	local originalCat = AHsearch_ConfigurationFrame_Categories.originalCat
	if confirm then
		if originalCat and originalCat ~= "" then
			AHsearch_Data[originalCat] = nil
			AHsearch:BuildStandardTable()
			AHsearch:CatFrame_Show(true)
			AHsearch:FillList("AHsearch_ConfigurationFrame_Categories_SelectCat")
			AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectCat")
			AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenu")
			AHsearch:QueryFrame_Show(nil)
			AHsearch:ConfigurationFrame_ClearAll()
			AHsearch_ConfigurationFrame_ConfirmPopup:Hide()
			confirmIsShown = false
			AHsearch:ConfigurationFrame_ShowContent() -- update tabs
		end
	else
		AHsearch_ConfigurationFrame_Categories_NewCat:ClearFocus()
		AHsearch_ConfigurationFrame_Categories_NameCat:ClearFocus()
		AHsearch_ConfigurationFrame_Categories_Order:ClearFocus()
		AHsearch_ConfigurationFrame_ConfirmPopupText1:SetText( string.format(L["Delete category '%s'?"], "|cffffffff"..originalCat.."|r") )
		AHsearch_ConfigurationFrame_ConfirmPopupText2:SetText( L["All search queries in that category will be deleted!"] )
		AHsearch_ConfigurationFrame_ConfirmPopupText3:SetText( "" )
		AHsearch_ConfigurationFrame_ConfirmPopupOkayButton:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_DeleteCat(true) end)
		AHsearch_ConfigurationFrame_ConfirmPopup:Show()
		confirmIsShown = true
	end
end
-- Categories Actions END  ----------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:ConfigurationFrame_SearchQueries_TestSearch()
	if not AHsearch_ConfigurationFrame_SearchQueries_SortBy.value then return end -- brainbug paranoia

	BrowseName:SetText(AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value)
	UIDropDownMenu_SetSelectedValue(BrowseDropDown, AHsearch_ConfigurationFrame_SearchQueries_Rarity.value)

	-- filter1
	AuctionFrameBrowse.selectedClassIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter1.value
	AuctionFrameBrowse.selectedClass = CLASS_FILTERS[AHsearch_ConfigurationFrame_SearchQueries_Filter1.value]
	-- filter2
	AuctionFrameBrowse.selectedSubclassIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter2.value
	local subx
	if curFilter1 and curFilter2 then
		for i = 1, #Filter2Table do
			if Filter2Table[i].index == AHsearch_ConfigurationFrame_SearchQueries_Filter2.value then
				subx = "|cffffffff"..Filter2Table[i].name.."|r"
				break
			end
		end
	end
	AuctionFrameBrowse.selectedSubclass = subx
	-- filter3
	AuctionFrameBrowse.selectedInvtypeIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter3.value
	-- min/max
	BrowseMinLevel:SetNumber( AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value or "" )
	BrowseMaxLevel:SetNumber( AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value or "" )
	-- sort
	local sortOrder
	if AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value then
		sortOrder = 1
	end
	AuctionFrame_SetSort("list", AHsearch_ConfigurationFrame_SearchQueries_SortBy.value, sortOrder)
	-- misc
	IsUsableCheckButton:SetChecked(AHsearch_ConfigurationFrame_SearchQueries_Usable.value)
	ShowOnPlayerCheckButton:SetChecked(AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value)

	-- update AuctionFrame
	UIDropDownMenu_Initialize(BrowseDropDown, BrowseDropDown_Initialize)
	AuctionFrameFilters_Update()
	UIDropDownMenu_Refresh(BrowseDropDown, false)
	-- search
	AuctionFrameBrowse_Search()
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:UpdateFilter()
	table.wipe(Filter1Table)
	table.wipe(Filter2Table)
	table.wipe(Filter3Table)

	local x = 1
	Filter1Table[x] = {}
	Filter1Table[x].index = -1
	Filter1Table[x].name = "-"
	x = x + 1

	for i = 1, #FilterTable do
		Filter1Table[x] = {}
		Filter1Table[x].index = FilterTable[i].itemClassIndex
		Filter1Table[x].name = FilterTable[i].itemClass
		x = x + 1
	end

	if curFilter1 then
		local x = 1
		for i = 1, #FilterTable do
			if curFilter1 == FilterTable[i].itemClassIndex then
				if FilterTable[i].subClasses then

					Filter2Table[x] = {}
					Filter2Table[x].index = -1
					Filter2Table[x].name = "-"
					x = x + 1

					for j = 1, #FilterTable[i].subClasses do
						Filter2Table[x] = {}
						Filter2Table[x].index = FilterTable[i].subClasses[j].subClassIndex
						Filter2Table[x].name = FilterTable[i].subClasses[j].subClass
						x = x + 1
					end
				end
			end
		end
	end

	if curFilter1 and curFilter2 then

		local x = 1
		for i = 1, #FilterTable do
			--print(i, "#", FilterTable[i].itemClassIndex, FilterTable[i].itemClass) -- TEST
			if FilterTable[i].subClasses then
				for j = 1, #FilterTable[i].subClasses do
					if curFilter1 == FilterTable[i].itemClassIndex then
						--print(i, j, "#", "-", FilterTable[i].subClasses[j].subClassIndex, FilterTable[i].subClasses[j].subClass) -- TEST
						if FilterTable[i].subClasses[j].subClassInvTypes then
							if curFilter2 == FilterTable[i].subClasses[j].subClassIndex then

								Filter3Table[x] = {}
								Filter3Table[x].index = -1
								Filter3Table[x].name = "-"
								x = x + 1

								for k = 1, #FilterTable[i].subClasses[j].subClassInvTypes do
									Filter3Table[x] = {}
									Filter3Table[x].index = FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTypIndex
									Filter3Table[x].name = FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTyp
									x = x + 1
									--print(i, j, k, "#", "--", FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTypIndex, FilterTable[i].subClasses[j].subClassInvTypes[k].subClassInvTyp) -- TEST
								end
							end
						end
					end
				end
			end
		end

	end

	local locFilter1index, locFilter2index, locFilter3index
	local locFilter1name, locFilter2name, locFilter3name

	-- FILTER 1
	if curFilter1 then
		for i = 1, #Filter1Table do
			if Filter1Table[i].index == curFilter1 then
				locFilter1index = Filter1Table[i].index
				locFilter1name = Filter1Table[i].name
				break
			end
		end
		AHsearch_ConfigurationFrame_SearchQueries_Filter1_PullDownMenuText:SetText(locFilter1name)
	else
		AHsearch_ConfigurationFrame_SearchQueries_Filter1_PullDownMenuText:SetText("-")
	end
	EnablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter1")

	-- FILTER 2
	if curFilter1 and curFilter1 >= 0 then
		EnablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter2")
	else
		DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter2")
	end
	if curFilter1 and curFilter1 >= 0 and curFilter2 then
		for i = 1, #Filter2Table do
			if Filter2Table[i].index == curFilter2 then
				locFilter2index = Filter2Table[i].index
				locFilter2name = Filter2Table[i].name
				break
			end
		end
		AHsearch_ConfigurationFrame_SearchQueries_Filter2_PullDownMenuText:SetText(locFilter2name)
	else
		AHsearch_ConfigurationFrame_SearchQueries_Filter2_PullDownMenuText:SetText("-")
	end

	-- FILTER 3
	if curFilter1 and curFilter1 >= 0 and curFilter2 and curFilter2 >= 0 then
		EnablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter3")
	else
		DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter3")
	end
	if curFilter1 and curFilter1 >= 0 and curFilter2 and curFilter2 >= 0 and curFilter3 then
		for i = 1, #Filter3Table do
			if Filter3Table[i].index == curFilter3 then
				locFilter3index = Filter3Table[i].index
				locFilter3name = Filter3Table[i].name
				break
			end
		end
		AHsearch_ConfigurationFrame_SearchQueries_Filter3_PullDownMenuText:SetText(locFilter3name)
	else
		AHsearch_ConfigurationFrame_SearchQueries_Filter3_PullDownMenuText:SetText("-")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
-- Search Queries Actions START  ----------------------------------------
function AHsearch:ConfigurationFrame_SearchQueries_CreateNewSQ() -- CREATENEW
	-- values
	--namequery
	local namequery
	if AHsearch_ConfigurationFrame_SearchQueries.editType == "new" then
		namequery = AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value
	else
		namequery = BrowseName:GetText()
		AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value = namequery
	end
	--namequery

	--order
	local order
	if AHsearch_ConfigurationFrame_SearchQueries.editType == "new" then
		order = AHsearch_ConfigurationFrame_SearchQueries_Order:GetNumber()
	else
		order = 1
	end
	AHsearch_ConfigurationFrame_SearchQueries_Order.value = order
	--order

	--rarity
	local qualityIndex = AHsearch_ConfigurationFrame_SearchQueries_Rarity.value
	if not qualityIndex then
		qualityIndex = UIDropDownMenu_GetSelectedValue(BrowseDropDown)
		AHsearch_ConfigurationFrame_SearchQueries_Rarity.value = qualityIndex
	end
	--rarity

	--filter1
	local classIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter1.value
	if not classIndex then
		classIndex = AuctionFrameBrowse.selectedClassIndex
		AHsearch_ConfigurationFrame_SearchQueries_Filter1.value = classIndex
	end
	curFilter1 = classIndex
	--filter1

	--filter2
	local subclassIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter2.value
	if not subclassIndex then
		subclassIndex = AuctionFrameBrowse.selectedSubclassIndex
		AHsearch_ConfigurationFrame_SearchQueries_Filter2.value = subclassIndex
	end
	curFilter2 = subclassIndex
	--local subclass = AuctionFrameBrowse.selectedSubclass -- setup in save action
	--filter2

	--filter3
	local invTypeIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter3.value
	if not invTypeIndex then
		invTypeIndex = AuctionFrameBrowse.selectedInvtypeIndex
		AHsearch_ConfigurationFrame_SearchQueries_Filter3.value = invTypeIndex
	end
	curFilter3 = invTypeIndex
	--filter3

	--level
	local minLevel
	local maxLevel
	if AHsearch_ConfigurationFrame_SearchQueries.editType == "new" then
		minLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value
		maxLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value
	else
		minLevel = BrowseMinLevel:GetNumber()
		if minLevel == 0 then
			minLevel = nil
		end
		AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value = minLevel
		maxLevel = BrowseMaxLevel:GetNumber()
		if maxLevel == 0 then
			maxLevel = nil
		end
		AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value = maxLevel
	end
	--level

	--sort by
	local primaryColumnPRE, reversedPRE = GetAuctionSort("list", 1)
	local primaryColumn = AHsearch_ConfigurationFrame_SearchQueries_SortBy.value
	if not primaryColumn then
		primaryColumn = primaryColumnPRE
		AHsearch_ConfigurationFrame_SearchQueries_SortBy.value = primaryColumn
	end
	--sort by

	--sort order
	local reversed
	local sortOrder1 = AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value
	local sortOrder2 = AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc.value
	if not sortOrder1 and not sortOrder2 then
		reversed = reversedPRE
		if reversed then
			AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value = true
			AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc.value = false
		else
			AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value = false
			AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc.value = true
		end
	else
		if sortOrder1 then
			reversed = 1
		end
	end
	--sort order

	--misc
	local isUsable = AHsearch_ConfigurationFrame_SearchQueries_Usable.value
	if isUsable == nil then
		isUsable = IsUsableCheckButton:GetChecked()
	end
	AHsearch_ConfigurationFrame_SearchQueries_Usable.value = isUsable
	local showOnPlayer = AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value
	if showOnPlayer == nil then
		showOnPlayer = ShowOnPlayerCheckButton:GetChecked()
	end
	AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value = showOnPlayer
	--misc

	local title = AHsearch_ConfigurationFrame_SearchQueries_NewSQ:GetText()
	AHsearch_ConfigurationFrame_SearchQueries.newSQtitle      = title
	AHsearch_ConfigurationFrame_SearchQueries.originalSQtitle = nil
	AHsearch_ConfigurationFrame_SearchQueries.currentSQtitle  = nil

	AHsearch_ConfigurationFrame_SearchQueries.editType = "new"
	AHsearch_ConfigurationFrame_SearchQueries_Move.value = nil

	-- setup display
	AHsearch_ConfigurationFrame_SearchQueries_MoveTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_MoveAlertIcon:Hide()
	AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenuText:SetText("")
	AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenuTexture:SetTexture(nil)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Move")
	
	AHsearch_ConfigurationFrame_SearchQueries_TitleSQTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_TitleSQ:SetText(title)
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_TitleSQ")

	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ:SetText(namequery)
	EnableEditBox("AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ")

	AHsearch_ConfigurationFrame_SearchQueries_OrderTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_Order:SetNumber(order)
	EnableEditBox("AHsearch_ConfigurationFrame_SearchQueries_Order")

	AHsearch_ConfigurationFrame_SearchQueries_RarityTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_Rarity_PullDownMenuText:SetText(itemRarity[qualityIndex+2].color..itemRarity[qualityIndex+2].name.."|r")
	EnablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Rarity")

	AHsearch_ConfigurationFrame_SearchQueries_FilterTitle:SetTextColor(1, 0.82, 0)
	AHsearch:UpdateFilter()
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Filter1_PullDownMenu")
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Filter2_PullDownMenu")
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Filter3_PullDownMenu")

	AHsearch_ConfigurationFrame_SearchQueries_LevelTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMinTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMin:SetNumber(minLevel or "")
	EnableEditBox("AHsearch_ConfigurationFrame_SearchQueries_LevelMin")
	AHsearch_ConfigurationFrame_SearchQueries_LevelMaxTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMax:SetNumber(maxLevel or "")
	EnableEditBox("AHsearch_ConfigurationFrame_SearchQueries_LevelMax")

	AHsearch_ConfigurationFrame_SearchQueries_SortTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_SortBy_PullDownMenuText:SetText(L[ primaryColumn ])
	EnablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_SortBy")
	if reversed then
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc:SetChecked(true)
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc:SetChecked(false)
	else
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc:SetChecked(false)
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc:SetChecked(true)
	end
	EnableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc", true)
	EnableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc", true)

	AHsearch_ConfigurationFrame_SearchQueries_MiscTitle:SetTextColor(1, 0.82, 0)
	EnableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_Usable")
	AHsearch_ConfigurationFrame_SearchQueries_Usable:SetChecked(isUsable)
	EnableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar")
	AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar:SetChecked(showOnPlayer)

	for i = 1, maxQuery do
		_G["AHsearch_QueryFrameScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_QueryFrameScrollFrame_Button"..i.."SelectedArrow"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."SelectedArrow"]:Hide()
	end

	-- action buttons
	EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_ClearAllSQ", 2)
	AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Create New"])
	EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ", 1)
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_DeleteSQ")
	EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_TestSearch", 3)
end

function AHsearch:ConfigurationFrame_SearchQueries_SelectCat(cat)
	AHsearch:ConfigurationFrame_SearchQueries_ClearAfterSQDelete()
	AHsearch_ConfigurationFrame_SearchQueries_SelectCat.cat = cat
	AHsearch_ConfigurationFrame_SearchQueries_SelectCat.sq = nil

	AHsearch_ConfigurationFrame_SearchQueries_NewSQTitle:SetTextColor(1, 0.82, 0)
	EnableEditBox("AHsearch_ConfigurationFrame_SearchQueries_NewSQ")

	-- action buttons
	EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_ClearAllSQ", 2)
	AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Create New"])
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ")
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_DeleteSQ")
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_TestSearch")
end

function AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, sq)
	AHsearch_ConfigurationFrame_SearchQueries.editType = "selected"

	AHsearch_ConfigurationFrame_SearchQueries_SelectCat.cat = cat
	AHsearch_ConfigurationFrame_SearchQueries_SelectCat.sq = sq

	AHsearch_ConfigurationFrame_SearchQueries.newSQtitle = nil
	AHsearch_ConfigurationFrame_SearchQueries.originalSQtitle = sq
	AHsearch_ConfigurationFrame_SearchQueries.currentSQtitle = sq

	-- values
	local tab = AHsearch_Data[cat].queries[sq]

	--namequery
	local namequery = tab.name
	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value = namequery
	AHsearch_ConfigurationFrame_SearchQueries.originalNameQuerySQ = namequery
	--namequery

	--order
	local order = tab.order
	AHsearch_ConfigurationFrame_SearchQueries_Order.value = order
	AHsearch_ConfigurationFrame_SearchQueries.originalOrder = order
	--order

	--rarity
	local qualityIndex = tab.qualityIndex
	AHsearch_ConfigurationFrame_SearchQueries_Rarity.value = qualityIndex
	AHsearch_ConfigurationFrame_SearchQueries.originalRarity = qualityIndex
	--rarity

	--filter1
	local classIndex = tab.classIndex
	AHsearch_ConfigurationFrame_SearchQueries_Filter1.value = classIndex
	AHsearch_ConfigurationFrame_SearchQueries.originalFilter1 = classIndex
	curFilter1 = classIndex
	--filter1

	--filter2
	local subclassIndex = tab.subclassIndex
	AHsearch_ConfigurationFrame_SearchQueries_Filter2.value = subclassIndex
	AHsearch_ConfigurationFrame_SearchQueries.originalFilter2 = subclassIndex
	curFilter2 = subclassIndex
	--filter2

	--filter3
	local invTypeIndex = tab.invTypeIndex
	AHsearch_ConfigurationFrame_SearchQueries_Filter3.value = invTypeIndex
	AHsearch_ConfigurationFrame_SearchQueries.originalFilter3 = invTypeIndex
	curFilter3 = invTypeIndex
	--filter3

	--level
	local minLevel = tab.minLevel
	local maxLevel = tab.maxLevel
	AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value = minLevel
	AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value = maxLevel
	AHsearch_ConfigurationFrame_SearchQueries.originalLevelMin = minLevel
	AHsearch_ConfigurationFrame_SearchQueries.originalLevelMax = maxLevel
	--level

	--sort by
	local primaryColumn = tab.SortBy
	AHsearch_ConfigurationFrame_SearchQueries_SortBy.value = primaryColumn
	AHsearch_ConfigurationFrame_SearchQueries.originalSortBy = primaryColumn
	--sort by

	--sort order
	local reversed = tab.SortOrder
	if reversed then
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value = true
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc.value = false
		AHsearch_ConfigurationFrame_SearchQueries.originalSortOrderAsc = true
		AHsearch_ConfigurationFrame_SearchQueries.originalSortOrderDesc = false
	else
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value = false
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc.value = true
		AHsearch_ConfigurationFrame_SearchQueries.originalSortOrderAsc = false
		AHsearch_ConfigurationFrame_SearchQueries.originalSortOrderDesc = true
	end
	--sort order

	--misc
	local isUsable = tab.isUsable
	if not isUsable then isUsable = 0 end
	AHsearch_ConfigurationFrame_SearchQueries_Usable.value = isUsable
	AHsearch_ConfigurationFrame_SearchQueries.originalUsable = isUsable
	local showOnPlayer = tab.showOnPlayer
	if not showOnPlayer then showOnPlayer = 0 end
	AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value = showOnPlayer
	AHsearch_ConfigurationFrame_SearchQueries.originalDisplayOnChar = showOnPlayer
	--misc

	-- setup display
	AHsearch_ConfigurationFrame_SearchQueries_MoveTitle:SetTextColor(1, 0.82, 0)
	EnablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Move")

	AHsearch_ConfigurationFrame_SearchQueries_NewSQ:SetText("")
	AHsearch_ConfigurationFrame_SearchQueries_NewSQ:ClearFocus()

	AHsearch_ConfigurationFrame_SearchQueries_TitleSQTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_TitleSQ:SetText(sq)
	AHsearch_ConfigurationFrame_SearchQueries_TitleSQ:ClearFocus()
	EnableEditBox("AHsearch_ConfigurationFrame_SearchQueries_TitleSQ")

	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ:SetText(namequery)
	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ:ClearFocus()
	EnableEditBox("AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ")

	AHsearch_ConfigurationFrame_SearchQueries_OrderTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_Order:SetNumber(order)
	AHsearch_ConfigurationFrame_SearchQueries_Order:ClearFocus()
	EnableEditBox("AHsearch_ConfigurationFrame_SearchQueries_Order")

	AHsearch_ConfigurationFrame_SearchQueries_RarityTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_Rarity_PullDownMenuText:SetText(itemRarity[qualityIndex+2].color..itemRarity[qualityIndex+2].name.."|r")
	EnablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Rarity")

	AHsearch_ConfigurationFrame_SearchQueries_FilterTitle:SetTextColor(1, 0.82, 0)
	AHsearch:UpdateFilter()
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Filter1_PullDownMenu")
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Filter2_PullDownMenu")
	AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_Filter3_PullDownMenu")

	AHsearch_ConfigurationFrame_SearchQueries_LevelTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMinTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMin:SetNumber(minLevel or "")
	AHsearch_ConfigurationFrame_SearchQueries_LevelMin:ClearFocus()
	EnableEditBox("AHsearch_ConfigurationFrame_SearchQueries_LevelMin")
	AHsearch_ConfigurationFrame_SearchQueries_LevelMaxTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMax:SetNumber(maxLevel or "")
	AHsearch_ConfigurationFrame_SearchQueries_LevelMax:ClearFocus()
	EnableEditBox("AHsearch_ConfigurationFrame_SearchQueries_LevelMax")

	AHsearch_ConfigurationFrame_SearchQueries_SortTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_SortBy_PullDownMenuText:SetText(L[ primaryColumn ])
	EnablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_SortBy")
	if reversed then
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc:SetChecked(true)
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc:SetChecked(false)
	else
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc:SetChecked(false)
		AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc:SetChecked(true)
	end
	EnableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc", true)
	EnableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc", true)

	AHsearch_ConfigurationFrame_SearchQueries_MiscTitle:SetTextColor(1, 0.82, 0)
	EnableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_Usable")
	AHsearch_ConfigurationFrame_SearchQueries_Usable:SetChecked(isUsable)
	EnableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar")
	AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar:SetChecked(showOnPlayer)

	-- action buttons
	EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_ClearAllSQ", 2)
	if AHsearch_ConfigurationFrame_SearchQueries_Move.value then
		if AHsearch_ConfigurationFrame_SearchQueries_Move.value == AHsearch_ConfigurationFrame_SearchQueries_SelectCat.cat then
			AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Update"])
			DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ")
		else
			AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Move"])
			EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ", 1)
		end
	else
		AHsearch_ConfigurationFrame_SearchQueries_SaveSQ:SetText(L["Update"])
		DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ")
	end
	EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_DeleteSQ", 1)
	EnableTextButton("AHsearch_ConfigurationFrame_SearchQueries_TestSearch", 3)
end

function AHsearch:ConfigurationFrame_SearchQueries_ClearAllSQ()
	AHsearch_ConfigurationFrame_SearchQueries.editType = nil

	-- values
	AHsearch_ConfigurationFrame_SearchQueries_SelectCat.cat = nil
	AHsearch_ConfigurationFrame_SearchQueries_SelectCat.sq = nil

	AHsearch_ConfigurationFrame_SearchQueries.newSQtitle = nil
	AHsearch_ConfigurationFrame_SearchQueries.originalSQtitle = nil
	AHsearch_ConfigurationFrame_SearchQueries.currentSQtitle = nil

	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Order.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Rarity.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Filter1.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Filter2.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Filter3.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_SortBy.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Usable.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Move.value = nil

	-- setup display
	AHsearch_ConfigurationFrame_SearchQueries_MoveTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_MoveAlertIcon:Hide()
	AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenuText:SetText("")
	AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenuTexture:SetTexture(nil)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Move")

	AHsearch_ConfigurationFrame_SearchQueries_NewSQTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_NewSQ:SetText("")
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_NewSQ")

	AHsearch_ConfigurationFrame_SearchQueries_TitleSQTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_TitleSQ:SetText("")
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_TitleSQ")

	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ:SetText("")
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ")

	AHsearch_ConfigurationFrame_SearchQueries_OrderTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_Order:SetNumber(1)
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_Order")

	AHsearch_ConfigurationFrame_SearchQueries_RarityTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_Rarity_PullDownMenuText:SetText("")
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Rarity")

	AHsearch_ConfigurationFrame_SearchQueries_FilterTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_Filter1_PullDownMenuText:SetText("")
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter1")
	AHsearch_ConfigurationFrame_SearchQueries_Filter2_PullDownMenuText:SetText("")
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter2")
	AHsearch_ConfigurationFrame_SearchQueries_Filter3_PullDownMenuText:SetText("")
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter3")

	AHsearch_ConfigurationFrame_SearchQueries_LevelTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMinTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMin:SetNumber("")
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_LevelMin")
	AHsearch_ConfigurationFrame_SearchQueries_LevelMaxTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMax:SetNumber("")
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_LevelMax")

	AHsearch_ConfigurationFrame_SearchQueries_SortTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_SortBy_PullDownMenuText:SetText("")
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_SortBy")
	AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc:SetChecked(false)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc", true)
	AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc:SetChecked(false)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc", true)

	AHsearch_ConfigurationFrame_SearchQueries_MiscTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_Usable:SetChecked(false)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_Usable")
	AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar:SetChecked(false)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar")

	for i = 1, maxCat do
		_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_CatFrameScrollFrame_Button"..i.."SelectedArrow"]:Hide()
		_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Hide()
	end

	for i = 1, maxQuery do
		_G["AHsearch_QueryFrameScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_QueryFrameScrollFrame_Button"..i.."SelectedArrow"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."SelectedArrow"]:Hide()
	end

	GUI_curSQ = nil
	GUI_curCat = nil

	AHsearch:QueryFrame_Show(nil)

	-- action buttons
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_ClearAllSQ")
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ")
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_DeleteSQ")
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_TestSearch")
end

function AHsearch:ConfigurationFrame_SearchQueries_ClearAfterSQDelete()
	AHsearch_ConfigurationFrame_SearchQueries.editType = nil

	-- values
	--AHsearch_ConfigurationFrame_SearchQueries_SelectCat.cat = nil
	AHsearch_ConfigurationFrame_SearchQueries_SelectCat.sq = nil

	AHsearch_ConfigurationFrame_SearchQueries.newSQtitle = nil
	AHsearch_ConfigurationFrame_SearchQueries.originalSQtitle = nil
	AHsearch_ConfigurationFrame_SearchQueries.currentSQtitle = nil

	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Order.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Rarity.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Filter1.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Filter2.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Filter3.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_SortBy.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Usable.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value = nil
	AHsearch_ConfigurationFrame_SearchQueries_Move.value = nil

	-- setup display
	AHsearch_ConfigurationFrame_SearchQueries_MoveTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_MoveAlertIcon:Hide()
	AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenuText:SetText("")
	AHsearch_ConfigurationFrame_SearchQueries_Move_PullDownMenuTexture:SetTexture(nil)
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Move")

	AHsearch_ConfigurationFrame_SearchQueries_NewSQTitle:SetTextColor(1, 0.82, 0)
	AHsearch_ConfigurationFrame_SearchQueries_NewSQ:SetText("")
	EnableEditBox("AHsearch_ConfigurationFrame_SearchQueries_NewSQ")

	AHsearch_ConfigurationFrame_SearchQueries_TitleSQTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_TitleSQ:SetText("")
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_TitleSQ")

	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ:SetText("")
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ")

	AHsearch_ConfigurationFrame_SearchQueries_OrderTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_Order:SetNumber(1)
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_Order")

	AHsearch_ConfigurationFrame_SearchQueries_RarityTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_Rarity_PullDownMenuText:SetText("")
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Rarity")

	AHsearch_ConfigurationFrame_SearchQueries_FilterTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_Filter1_PullDownMenuText:SetText("")
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter1")
	AHsearch_ConfigurationFrame_SearchQueries_Filter2_PullDownMenuText:SetText("")
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter2")
	AHsearch_ConfigurationFrame_SearchQueries_Filter3_PullDownMenuText:SetText("")
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_Filter3")

	AHsearch_ConfigurationFrame_SearchQueries_LevelTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMinTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMin:SetNumber("")
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_LevelMin")
	AHsearch_ConfigurationFrame_SearchQueries_LevelMaxTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_LevelMax:SetNumber("")
	DisableEditBox("AHsearch_ConfigurationFrame_SearchQueries_LevelMax")

	AHsearch_ConfigurationFrame_SearchQueries_SortTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_SortBy_PullDownMenuText:SetText("")
	DisablePullDownMenu("AHsearch_ConfigurationFrame_SearchQueries_SortBy")
	AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc:SetChecked(false)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc", true)
	AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc:SetChecked(false)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_SortOrderDesc", true)

	AHsearch_ConfigurationFrame_SearchQueries_MiscTitle:SetTextColor(0.5, 0.5, 0.5)
	AHsearch_ConfigurationFrame_SearchQueries_Usable:SetChecked(0)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_Usable")
	AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar:SetChecked(0)
	DisableCheckButton("AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar")

	for i = 1, maxQuery do
		_G["AHsearch_QueryFrameScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_QueryFrameScrollFrame_Button"..i.."SelectedArrow"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Selected"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."SelectedArrow"]:Hide()
	end

	GUI_curSQ = nil

	-- action buttons
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_ClearAllSQ")
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_SaveSQ")
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_DeleteSQ")
	DisableTextButton("AHsearch_ConfigurationFrame_SearchQueries_TestSearch")
end

function AHsearch:ConfigurationFrame_SearchQueries_SaveSQ(confirm) --SAVEDATA
	local cat = AHsearch_ConfigurationFrame_SearchQueries_SelectCat.cat
	local sq  = AHsearch_ConfigurationFrame_SearchQueries_SelectCat.sq
	local newSQ      = AHsearch_ConfigurationFrame_SearchQueries.newSQtitle
	local originalSQ = AHsearch_ConfigurationFrame_SearchQueries.originalSQtitle
	local currentSQ  = AHsearch_ConfigurationFrame_SearchQueries.currentSQtitle
	local newCat     = AHsearch_ConfigurationFrame_SearchQueries_Move.value

	if originalSQ then
		-- move START ----------
		if newCat and newCat ~= cat then
			if AHsearch_Data[newCat].queries and AHsearch_Data[newCat].queries[currentSQ] then
				if confirm then
					-- - searchquery exists in new cat confirm -> action START
					AHsearch_Data[cat].queries[originalSQ] = nil

					local x = 0
					for k in pairs(AHsearch_Data[cat].queries) do
						x = x + 1
					end
					if x == 0 then
						AHsearch_Data[cat].queries = nil
					end

					AHsearch_Data[newCat].queries[currentSQ] = {}
					AHsearch_Data[newCat].queries[currentSQ].name = AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value
					AHsearch_Data[newCat].queries[currentSQ].order = AHsearch_ConfigurationFrame_SearchQueries_Order.value
					AHsearch_Data[newCat].queries[currentSQ].qualityIndex = AHsearch_ConfigurationFrame_SearchQueries_Rarity.value
					AHsearch_Data[newCat].queries[currentSQ].classIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter1.value
					AHsearch_Data[newCat].queries[currentSQ].subclassIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter2.value
					local subx
					if curFilter1 and curFilter2 then
						for i = 1, #Filter2Table do
							if Filter2Table[i].index == AHsearch_ConfigurationFrame_SearchQueries_Filter2.value then
								subx = "|cffffffff"..Filter2Table[i].name.."|r"
								break
							end
						end
					end
					AHsearch_Data[newCat].queries[currentSQ].subclass = subx
					AHsearch_Data[newCat].queries[currentSQ].invTypeIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter3.value

					AHsearch_Data[newCat].queries[currentSQ].minLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value
					AHsearch_Data[newCat].queries[currentSQ].maxLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value

					AHsearch_Data[newCat].queries[currentSQ].SortBy = AHsearch_ConfigurationFrame_SearchQueries_SortBy.value
					local sortOrder
					if AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value then
						sortOrder = 1
					end
					AHsearch_Data[newCat].queries[currentSQ].SortOrder = sortOrder

					local Usable = AHsearch_ConfigurationFrame_SearchQueries_Usable.value
					if Usable == 0 or Usable == false then
						Usable = nil
					elseif Usable == true then
						Uasble = 1
					end
					AHsearch_Data[newCat].queries[currentSQ].isUsable = Usable

					local DisplayOnChar = AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value
					if DisplayOnChar == 0 or DisplayOnChar == false then
						DisplayOnChar = nil
					elseif DisplayOnChar == true then
						DisplayOnChar = 1
					end
					AHsearch_Data[newCat].queries[currentSQ].showOnPlayer = DisplayOnChar

					AHsearch:BuildStandardTable()
					AHsearch:CatFrame_Show(true)
					AHsearch:QueryFrame_Show(cat)

					AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectSQ")
					AHsearch:ConfigurationFrame_SearchQueries_SelectCat(cat)

					AHsearch_ConfigurationFrame_ConfirmPopup:Hide()
					confirmIsShown = false
					-- - searchquery exists in new cat confirm -> action END
				else
					-- - searchquery exists in new cat confirm popup START
					AHsearch_ConfigurationFrame_SearchQueries_NewSQ:ClearFocus()
					AHsearch_ConfigurationFrame_SearchQueries_TitleSQ:ClearFocus()
					AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ:ClearFocus()
					AHsearch_ConfigurationFrame_SearchQueries_Order:ClearFocus()
					AHsearch_ConfigurationFrame_SearchQueries_LevelMin:ClearFocus()
					AHsearch_ConfigurationFrame_SearchQueries_LevelMax:ClearFocus()
					AHsearch_ConfigurationFrame_ConfirmPopupText1:SetText( string.format(L["Search Query '%s' in category '%s' exists!"], "|cffffffff"..currentSQ.."|r", "|cffffffff"..newCat.."|r") )
					AHsearch_ConfigurationFrame_ConfirmPopupText2:SetText( L["Overwrite?"] )
					AHsearch_ConfigurationFrame_ConfirmPopupText3:SetText( "" )
					AHsearch_ConfigurationFrame_ConfirmPopupOkayButton:SetScript("OnClick", function() AHsearch:ConfigurationFrame_SearchQueries_SaveSQ(true) end)
					AHsearch_ConfigurationFrame_ConfirmPopup:Show()
					confirmIsShown = true
					-- - searchquery exists in new cat confirm popup END
				end
				-- - search query exists END
			else
				-- - search query does not exists -> make new -> simple move START
				if not AHsearch_Data[newCat].queries then AHsearch_Data[newCat].queries = {} end

				AHsearch_Data[cat].queries[originalSQ] = nil

				local x = 0
				for k in pairs(AHsearch_Data[cat].queries) do
					x = x + 1
				end
				if x == 0 then
					AHsearch_Data[cat].queries = nil
				end

				AHsearch_Data[newCat].queries[currentSQ] = {}
				AHsearch_Data[newCat].queries[currentSQ].name = AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value
				AHsearch_Data[newCat].queries[currentSQ].order = AHsearch_ConfigurationFrame_SearchQueries_Order.value
				AHsearch_Data[newCat].queries[currentSQ].qualityIndex = AHsearch_ConfigurationFrame_SearchQueries_Rarity.value
				AHsearch_Data[newCat].queries[currentSQ].classIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter1.value
				AHsearch_Data[newCat].queries[currentSQ].subclassIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter2.value
				local subx
				if curFilter1 and curFilter2 then
					for i = 1, #Filter2Table do
						if Filter2Table[i].index == AHsearch_ConfigurationFrame_SearchQueries_Filter2.value then
							subx = "|cffffffff"..Filter2Table[i].name.."|r"
							break
						end
					end
				end
				AHsearch_Data[newCat].queries[currentSQ].subclass = subx
				AHsearch_Data[newCat].queries[currentSQ].invTypeIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter3.value

				AHsearch_Data[newCat].queries[currentSQ].minLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value
				AHsearch_Data[newCat].queries[currentSQ].maxLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value

				AHsearch_Data[newCat].queries[currentSQ].SortBy = AHsearch_ConfigurationFrame_SearchQueries_SortBy.value
				local sortOrder
				if AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value then
					sortOrder = 1
				end
				AHsearch_Data[newCat].queries[currentSQ].SortOrder = sortOrder

				local Usable = AHsearch_ConfigurationFrame_SearchQueries_Usable.value
				if Usable == 0 or Usable == false then
					Usable = nil
				elseif Usable == true then
					Uasble = 1
				end
				AHsearch_Data[newCat].queries[currentSQ].isUsable = Usable

				local DisplayOnChar = AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value
				if DisplayOnChar == 0 or DisplayOnChar == false then
					DisplayOnChar = nil
				elseif DisplayOnChar == true then
					DisplayOnChar = 1
				end
				AHsearch_Data[newCat].queries[currentSQ].showOnPlayer = DisplayOnChar

				AHsearch:BuildStandardTable()
				AHsearch:CatFrame_Show(true)
				AHsearch:QueryFrame_Show(cat)

				AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectSQ")
				AHsearch:ConfigurationFrame_SearchQueries_SelectCat(cat)

				AHsearch_ConfigurationFrame_ConfirmPopup:Hide()
				confirmIsShown = false
				-- - search query does not exists -> make new -> simple move END
			end
		-- move END ----------
		else
			-- update existing search query START
			if AHsearch_Data[cat].queries[currentSQ] and originalSQ ~= currentSQ then
				-- - search query exists START
				if confirm then
					-- - searchquery exists confirm -> action START
					AHsearch_Data[cat].queries[originalSQ] = nil

					AHsearch_Data[cat].queries[currentSQ] = {}
					AHsearch_Data[cat].queries[currentSQ].name = AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value
					AHsearch_Data[cat].queries[currentSQ].order = AHsearch_ConfigurationFrame_SearchQueries_Order.value
					AHsearch_Data[cat].queries[currentSQ].qualityIndex = AHsearch_ConfigurationFrame_SearchQueries_Rarity.value
					AHsearch_Data[cat].queries[currentSQ].classIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter1.value
					AHsearch_Data[cat].queries[currentSQ].subclassIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter2.value
					local subx
					if curFilter1 and curFilter2 then
						for i = 1, #Filter2Table do
							if Filter2Table[i].index == AHsearch_ConfigurationFrame_SearchQueries_Filter2.value then
								subx = "|cffffffff"..Filter2Table[i].name.."|r"
								break
							end
						end
					end
					AHsearch_Data[cat].queries[currentSQ].subclass = subx
					AHsearch_Data[cat].queries[currentSQ].invTypeIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter3.value

					AHsearch_Data[cat].queries[currentSQ].minLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value
					AHsearch_Data[cat].queries[currentSQ].maxLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value

					AHsearch_Data[cat].queries[currentSQ].SortBy = AHsearch_ConfigurationFrame_SearchQueries_SortBy.value
					local sortOrder
					if AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value then
						sortOrder = 1
					end
					AHsearch_Data[cat].queries[currentSQ].SortOrder = sortOrder

					local Usable = AHsearch_ConfigurationFrame_SearchQueries_Usable.value
					if Usable == 0 or Usable == false then
						Usable = nil
					elseif Usable == true then
						Uasble = 1
					end
					AHsearch_Data[cat].queries[currentSQ].isUsable = Usable

					local DisplayOnChar = AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value
					if DisplayOnChar == 0 or DisplayOnChar == false then
						DisplayOnChar = nil
					elseif DisplayOnChar == true then
						DisplayOnChar = 1
					end
					AHsearch_Data[cat].queries[currentSQ].showOnPlayer = DisplayOnChar

					GUI_curSQ = currentSQ --LLL
					AHsearch:BuildStandardTable()
					AHsearch:CatFrame_Show(true)
					AHsearch:QueryFrame_Show(cat)
					AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectSQ")
					AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, currentSQ)

					AHsearch_ConfigurationFrame_ConfirmPopup:Hide()
					confirmIsShown = false
					-- - searchquery exists confirm -> action END
				else
					-- - searchquery exists confirm popup START
					AHsearch_ConfigurationFrame_SearchQueries_NewSQ:ClearFocus()
					AHsearch_ConfigurationFrame_SearchQueries_TitleSQ:ClearFocus()
					AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ:ClearFocus()
					AHsearch_ConfigurationFrame_SearchQueries_Order:ClearFocus()
					AHsearch_ConfigurationFrame_SearchQueries_LevelMin:ClearFocus()
					AHsearch_ConfigurationFrame_SearchQueries_LevelMax:ClearFocus()
					AHsearch_ConfigurationFrame_ConfirmPopupText1:SetText( string.format(L["Search Query '%s' in category '%s' exists!"], "|cffffffff"..currentSQ.."|r", "|cffffffff"..cat.."|r") )
					AHsearch_ConfigurationFrame_ConfirmPopupText2:SetText( L["Overwrite?"] )
					AHsearch_ConfigurationFrame_ConfirmPopupText3:SetText( "" )
					AHsearch_ConfigurationFrame_ConfirmPopupOkayButton:SetScript("OnClick", function() AHsearch:ConfigurationFrame_SearchQueries_SaveSQ(true) end)
					AHsearch_ConfigurationFrame_ConfirmPopup:Show()
					confirmIsShown = true
					-- - searchquery exists confirm popup END
				end
				-- - search query exists END
			else
				-- - search query does not exists -> make new -> simple rename START
				AHsearch_Data[cat].queries[originalSQ] = nil

				AHsearch_Data[cat].queries[currentSQ] = {}
				AHsearch_Data[cat].queries[currentSQ].name = AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value
				AHsearch_Data[cat].queries[currentSQ].order = AHsearch_ConfigurationFrame_SearchQueries_Order.value
				AHsearch_Data[cat].queries[currentSQ].qualityIndex = AHsearch_ConfigurationFrame_SearchQueries_Rarity.value
				AHsearch_Data[cat].queries[currentSQ].classIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter1.value
				AHsearch_Data[cat].queries[currentSQ].subclassIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter2.value
				local subx
				if curFilter1 and curFilter2 then
					for i = 1, #Filter2Table do
						if Filter2Table[i].index == AHsearch_ConfigurationFrame_SearchQueries_Filter2.value then
							subx = "|cffffffff"..Filter2Table[i].name.."|r"
							break
						end
					end
				end
				AHsearch_Data[cat].queries[currentSQ].subclass = subx
				AHsearch_Data[cat].queries[currentSQ].invTypeIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter3.value

				AHsearch_Data[cat].queries[currentSQ].minLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value
				AHsearch_Data[cat].queries[currentSQ].maxLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value

				AHsearch_Data[cat].queries[currentSQ].SortBy = AHsearch_ConfigurationFrame_SearchQueries_SortBy.value
				local sortOrder
				if AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value then
					sortOrder = 1
				end
				AHsearch_Data[cat].queries[currentSQ].SortOrder = sortOrder

				local Usable = AHsearch_ConfigurationFrame_SearchQueries_Usable.value
				if Usable == 0 or Usable == false then
					Usable = nil
				elseif Usable == true then
					Uasble = 1
				end
				AHsearch_Data[cat].queries[currentSQ].isUsable = Usable

				local DisplayOnChar = AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value
				if DisplayOnChar == 0 or DisplayOnChar == false then
					DisplayOnChar = nil
				elseif DisplayOnChar == true then
					DisplayOnChar = 1
				end
				AHsearch_Data[cat].queries[currentSQ].showOnPlayer = DisplayOnChar

				GUI_curSQ = currentSQ --LLL
				AHsearch:BuildStandardTable()
				AHsearch:CatFrame_Show(true)
				AHsearch:QueryFrame_Show(cat)
				AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectSQ")
				AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, currentSQ)
				-- - search query does not exists -> make new -> simple rename END
			end
			-- update existing search query END
		end
	else
		-- create new searchquery START
		if AHsearch_Data[cat].queries and AHsearch_Data[cat].queries[newSQ] then
			-- - searchquery exists START
			if confirm then
				-- - searchquery exists confirm -> action START
				AHsearch_Data[cat].queries[newSQ] = nil

				AHsearch_Data[cat].queries[newSQ] = {}
				AHsearch_Data[cat].queries[newSQ].name = AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value
				AHsearch_Data[cat].queries[newSQ].order = AHsearch_ConfigurationFrame_SearchQueries_Order.value
				AHsearch_Data[cat].queries[newSQ].qualityIndex = AHsearch_ConfigurationFrame_SearchQueries_Rarity.value
				AHsearch_Data[cat].queries[newSQ].classIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter1.value
				AHsearch_Data[cat].queries[newSQ].subclassIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter2.value
				local subx
				if curFilter1 and curFilter2 then
					for i = 1, #Filter2Table do
						if Filter2Table[i].index == AHsearch_ConfigurationFrame_SearchQueries_Filter2.value then
							subx = "|cffffffff"..Filter2Table[i].name.."|r"
							break
						end
					end
				end
				AHsearch_Data[cat].queries[newSQ].subclass = subx
				AHsearch_Data[cat].queries[newSQ].invTypeIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter3.value

				AHsearch_Data[cat].queries[newSQ].minLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value
				AHsearch_Data[cat].queries[newSQ].maxLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value

				AHsearch_Data[cat].queries[newSQ].SortBy = AHsearch_ConfigurationFrame_SearchQueries_SortBy.value
				local sortOrder
				if AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value then
					sortOrder = 1
				end
				AHsearch_Data[cat].queries[newSQ].SortOrder = sortOrder

				local Usable = AHsearch_ConfigurationFrame_SearchQueries_Usable.value
				if Usable == 0 or Usable == false then
					Usable = nil
				elseif Usable == true then
					Uasble = 1
				end
				AHsearch_Data[cat].queries[newSQ].isUsable = Usable

				local DisplayOnChar = AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value
				if DisplayOnChar == 0 or DisplayOnChar == false then
					DisplayOnChar = nil
				elseif DisplayOnChar == true then
					DisplayOnChar = 1
				end
				AHsearch_Data[cat].queries[newSQ].showOnPlayer = DisplayOnChar

				GUI_curSQ = newSQ --LLL
				AHsearch:BuildStandardTable()
				AHsearch:CatFrame_Show(true)
				AHsearch:QueryFrame_Show(cat)
				AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectSQ")
				AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, newSQ)

				AHsearch_ConfigurationFrame_ConfirmPopup:Hide()
				confirmIsShown = false
				-- - searchquery exists confirm -> action END
			else
				-- - searchquery exists confirm popup START
				AHsearch_ConfigurationFrame_SearchQueries_NewSQ:ClearFocus()
				AHsearch_ConfigurationFrame_SearchQueries_TitleSQ:ClearFocus()
				AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ:ClearFocus()
				AHsearch_ConfigurationFrame_SearchQueries_Order:ClearFocus()
				AHsearch_ConfigurationFrame_SearchQueries_LevelMin:ClearFocus()
				AHsearch_ConfigurationFrame_SearchQueries_LevelMax:ClearFocus()
				AHsearch_ConfigurationFrame_ConfirmPopupText1:SetText( string.format(L["Search Query '%s' in category '%s' exists!"], "|cffffffff"..newSQ.."|r", "|cffffffff"..cat.."|r") )
				AHsearch_ConfigurationFrame_ConfirmPopupText2:SetText( L["Overwrite?"] )
				AHsearch_ConfigurationFrame_ConfirmPopupText3:SetText( "" )
				AHsearch_ConfigurationFrame_ConfirmPopupOkayButton:SetScript("OnClick", function() AHsearch:ConfigurationFrame_SearchQueries_SaveSQ(true) end)
				AHsearch_ConfigurationFrame_ConfirmPopup:Show()
				confirmIsShown = true
				-- - searchquery exists confirm popup END
			end
			-- - searchquery exists END
		else
			-- - searchquery does not exists -> make new START
			if not AHsearch_Data[cat].queries then AHsearch_Data[cat].queries = {} end

			AHsearch_Data[cat].queries[newSQ] = {}
			AHsearch_Data[cat].queries[newSQ].name = AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ.value
			AHsearch_Data[cat].queries[newSQ].order = AHsearch_ConfigurationFrame_SearchQueries_Order.value
			AHsearch_Data[cat].queries[newSQ].qualityIndex = AHsearch_ConfigurationFrame_SearchQueries_Rarity.value
			AHsearch_Data[cat].queries[newSQ].classIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter1.value
			AHsearch_Data[cat].queries[newSQ].subclassIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter2.value
			local subx
			if curFilter1 and curFilter2 then
				for i = 1, #Filter2Table do
					if Filter2Table[i].index == AHsearch_ConfigurationFrame_SearchQueries_Filter2.value then
						subx = "|cffffffff"..Filter2Table[i].name.."|r"
						break
					end
				end
			end
			AHsearch_Data[cat].queries[newSQ].subclass = subx
			AHsearch_Data[cat].queries[newSQ].invTypeIndex = AHsearch_ConfigurationFrame_SearchQueries_Filter3.value

			AHsearch_Data[cat].queries[newSQ].minLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMin.value
			AHsearch_Data[cat].queries[newSQ].maxLevel = AHsearch_ConfigurationFrame_SearchQueries_LevelMax.value

			AHsearch_Data[cat].queries[newSQ].SortBy = AHsearch_ConfigurationFrame_SearchQueries_SortBy.value
			local sortOrder
			if AHsearch_ConfigurationFrame_SearchQueries_SortOrderAsc.value then
				sortOrder = 1
			end
			AHsearch_Data[cat].queries[newSQ].SortOrder = sortOrder

			local Usable = AHsearch_ConfigurationFrame_SearchQueries_Usable.value
			if Usable == 0 or Usable == false then
				Usable = nil
			elseif Usable == true then
				Uasble = 1
			end
			AHsearch_Data[cat].queries[newSQ].isUsable = Usable

			local DisplayOnChar = AHsearch_ConfigurationFrame_SearchQueries_DisplayOnChar.value
			if DisplayOnChar == 0 or DisplayOnChar == false then
				DisplayOnChar = nil
			elseif DisplayOnChar == true then
				DisplayOnChar = 1
			end
			AHsearch_Data[cat].queries[newSQ].showOnPlayer = DisplayOnChar

			GUI_curSQ = newSQ --LLL
			AHsearch:BuildStandardTable()
			AHsearch:CatFrame_Show(true)
			AHsearch:QueryFrame_Show(cat)
			AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectSQ")
			AHsearch:ConfigurationFrame_SearchQueries_SelectSQ(cat, newSQ)
			-- - searchquery does not exists -> make new END
		end
		-- create new searchquery END
	end
end

function AHsearch:ConfigurationFrame_Categories_DeleteSQ(confirm)
	local cat = AHsearch_ConfigurationFrame_SearchQueries_SelectCat.cat
	local sq  = AHsearch_ConfigurationFrame_SearchQueries_SelectCat.sq

	if confirm then

		AHsearch_Data[cat].queries[sq] = nil

		local x = 0
		for k in pairs(AHsearch_Data[cat].queries) do
			x = x + 1
		end
		if x == 0 then
			AHsearch_Data[cat].queries = nil
		end

		AHsearch:BuildStandardTable()
		AHsearch:CatFrame_Show(true)
		AHsearch:QueryFrame_Show(cat)
		AHsearch:FillList("AHsearch_ConfigurationFrame_SearchQueries_SelectSQ")
		AHsearch:ConfigurationFrame_SearchQueries_ClearAfterSQDelete()
		AHsearch_ConfigurationFrame_ConfirmPopup:Hide()
		confirmIsShown = false
	else
		AHsearch_ConfigurationFrame_SearchQueries_NewSQ:ClearFocus()
		AHsearch_ConfigurationFrame_SearchQueries_TitleSQ:ClearFocus()
		AHsearch_ConfigurationFrame_SearchQueries_NameQuerySQ:ClearFocus()
		AHsearch_ConfigurationFrame_SearchQueries_Order:ClearFocus()
		AHsearch_ConfigurationFrame_SearchQueries_LevelMin:ClearFocus()
		AHsearch_ConfigurationFrame_SearchQueries_LevelMax:ClearFocus()
		AHsearch_ConfigurationFrame_ConfirmPopupText1:SetText( string.format(L["Delete search query '%s' in category '%s'?"], "|cffffffff"..sq.."|r", "|cffffffff"..cat.."|r") )
		AHsearch_ConfigurationFrame_ConfirmPopupText2:SetText( "" )
		AHsearch_ConfigurationFrame_ConfirmPopupText3:SetText( "" )
		AHsearch_ConfigurationFrame_ConfirmPopupOkayButton:SetScript("OnClick", function() AHsearch:ConfigurationFrame_Categories_DeleteSQ(true) end)
		AHsearch_ConfigurationFrame_ConfirmPopup:Show()
		confirmIsShown = true
	end
end
-- Search Queries Actions END  ----------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:ConfigurationFrame_ClearAll()
	AHsearch:ConfigurationFrame_Categories_ClearAllCat()
	AHsearch:ConfigurationFrame_SearchQueries_ClearAllSQ()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:SaveConfigFramePos()
	local point, relativeTo, relativePoint, x, y = AHsearch_ConfigurationFrame:GetPoint()
	AHsearch_Options.ConfigFrame_posX = x
	AHsearch_Options.ConfigFrame_posY = y
	AHsearch_Options.ConfigFrame_relP = relativePoint
end

function AHsearch:ConfigFramePos()
	if AHsearch_Options.ConfigFrame_relP then
		AHsearch_ConfigurationFrame:ClearAllPoints()
		AHsearch_ConfigurationFrame:SetPoint(AHsearch_Options.ConfigFrame_relP, AHsearch_Options.ConfigFrame_posX, AHsearch_Options.ConfigFrame_posY)
	else
		AHsearch_ConfigurationFrame:ClearAllPoints()
		AHsearch_ConfigurationFrame:SetPoint("TOPLEFT", AHsearchFrame, "BOTTOMLEFT", AHsearch_Options.curCatWidth+AHsearch_Options.curQueryWidth-20-(2*sizeGeneral.Border), 47)-- -12) --14+(30-border)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
-- Options Actions START  ----------------------------------------
function AHsearch:SetupCatWidth(value)
	AHsearch_Options.curCatWidth = value
	CatSizeUpdate(AHsearch_Options.curCatWidth)
	AHsearch_CatFrame:SetWidth(AHsearch_Options.curCatWidth)
	AHsearch:FillList("AHsearch_CatFrame")
end

function AHsearch:SetupCatHeight(value)
	AHsearch_Options.curCat = value
	local height = AHsearch_Options.curCat
	if AHsearch_Options.curCat > GUI_CatFrame_Count then
		height = GUI_CatFrame_Count
	end
	if GUI_CatFrame_Count == 0 then
		AHsearch_CatFrame:SetHeight(sizeGeneral.Offset+(10*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	else
		AHsearch_CatFrame:SetHeight(sizeGeneral.Offset+(height*sizeGeneral.BarHeight)+sizeGeneral.Offset)
	end
	AHsearch:FillList("AHsearch_CatFrame")
	AHsearch:QueryFrame_Show(GUI_curCat) -- VODOO
end

function AHsearch:SetupSQWidth(value)
	AHsearch_Options.curQueryWidth = value
	QuerySizeUpdate(AHsearch_Options.curQueryWidth)
	AHsearch_QueryFrame:SetWidth(AHsearch_Options.curQueryWidth)
	if not GUI_curCat then return end
	if not GUI_SQFrame_Count[GUI_curCat] then return end
	if GUI_CatFrame_Count > 0 then
		AHsearch:FillList("AHsearch_QueryFrame")
	end
end

function AHsearch:SetupSQHeight(value)
	AHsearch_Options.curQuery = value
	if not GUI_curCat then return end
	if not GUI_SQFrame_Count[GUI_curCat] then return end
	if GUI_CatFrame_Count > 0 then
		local height = AHsearch_Options.curQuery
		if AHsearch_Options.curQuery > GUI_SQFrame_Count[GUI_curCat] then
			height = GUI_SQFrame_Count[GUI_curCat]
		end
		AHsearch_QueryFrame:SetHeight(sizeGeneral.Offset+(height*sizeGeneral.BarHeight)+sizeGeneral.Offset)
		AHsearch:FillList("AHsearch_QueryFrame")
	end
end

function AHsearch:SetupMainScale(value)
	AHsearch_Options.scaleMain = value
	AHsearch_CatFrame:SetScale(value/100)
	AHsearch_CatFrame:ClearAllPoints()
	AHsearch_CatFrame:SetPoint("TOPLEFT", AHsearchFrame, "BOTTOMLEFT", -(20/(value/100)), -(12/(value/100)))
end
-- Options Actions END  ----------------------------------------
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:Search(cat, sq)
	-- save data for cycle search
	cycleCat = cat
	cycleSQ = sq
	-- load fields in AuctionFrame
	local tab = AHsearch_Data[cat].queries[sq]

	BrowseName:SetText(tab.name)

	BrowseMinLevel:SetNumber(tab.minLevel or "")
	BrowseMaxLevel:SetNumber(tab.maxLevel or "")

	AuctionFrameBrowse.selectedClassIndex = tab.classIndex
	AuctionFrameBrowse.selectedClass = CLASS_FILTERS[tab.classIndex]
	AuctionFrameBrowse.selectedSubclassIndex = tab.subclassIndex
	AuctionFrameBrowse.selectedSubclass = tab.subclass
	AuctionFrameBrowse.selectedInvtypeIndex = tab.invTypeIndex

	AuctionFrame_SetSort("list", tab.SortBy, tab.SortOrder)

	IsUsableCheckButton:SetChecked(tab.isUsable)
	ShowOnPlayerCheckButton:SetChecked(tab.showOnPlayer)
	UIDropDownMenu_SetSelectedValue(BrowseDropDown, tab.qualityIndex)
	-- update AuctionFrame
	UIDropDownMenu_Initialize(BrowseDropDown, BrowseDropDown_Initialize)
	AuctionFrameFilters_Update()
	UIDropDownMenu_Refresh(BrowseDropDown, false)
	-- search
	AuctionFrameBrowse_Search()
	AHsearch_CatFrame:Hide()
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:CycleSearch(dir)
	if not cycleChk then return end --> no AH GUI = no Action (Key Bindings)
	if not SearchQueryGlobalCheck() then return end --> no SQ = no Action

	if not cycleCat then
		local firstCat, firstSQ
		local lastCat, lastSQ
		for i = 1, GUI_CatFrame_Count do
			if GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] > 0 then
				for j = 1, GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] do
					if not firstCat then
						firstCat = GUI_CatFrame_data[i].Cat
						firstSQ  = GUI_SQFrame_data[ GUI_CatFrame_data[i].Cat ][j].Query
					end
					lastCat = GUI_CatFrame_data[i].Cat
					lastSQ  = GUI_SQFrame_data[ GUI_CatFrame_data[i].Cat ][j].Query
				end
			end
		end
		if dir == 1 then -- first cat
			cycleCat = firstCat
			cycleSQ  = firstSQ
		else             -- last cat
			cycleCat = lastCat
			cycleSQ  = lastSQ
		end
	else
		if not cycleCat and not cycleSQ then return end

		local nextCat, nextSQ
		local prevCat, prevSQ
		local num = 1
		local nextBingo, prevBingo
		for i = 1, GUI_CatFrame_Count do
			if GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] > 0 then
				for j = 1, GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] do
					if cycleCat == GUI_CatFrame_data[i].Cat and cycleSQ == GUI_SQFrame_data[ GUI_CatFrame_data[i].Cat ][j].Query then
						nextBingo = num
						prevBingo = num
					end
					num = num + 1
				end
			end
		end

		if prevBingo == 1 then
			prevBingo = num
		end
		if nextBingo == num-1 then
			nextBingo = 0
		end

		num = 1
		for i = 1, GUI_CatFrame_Count do
			if GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] > 0 then
				for j = 1, GUI_SQFrame_Count[ GUI_CatFrame_data[i].Cat ] do
					if num == (prevBingo-1) then
						prevCat = GUI_CatFrame_data[i].Cat
						prevSQ  = GUI_SQFrame_data[ GUI_CatFrame_data[i].Cat ][j].Query
					end
					if num == (nextBingo+1) then
						nextCat = GUI_CatFrame_data[i].Cat
						nextSQ  = GUI_SQFrame_data[ GUI_CatFrame_data[i].Cat ][j].Query
					end
					num = num + 1
				end
			end
		end

		if dir == 1 then -- next cat
			cycleCat = nextCat
			cycleSQ  = nextSQ
		else             -- prev cat
			cycleCat = prevCat
			cycleSQ  = prevSQ
		end
	end
	if not cycleCat and not cycleSQ then return end
	AHsearch:Search(cycleCat, cycleSQ)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:RandomSearch()
	if not cycleChk then return end --> no AH GUI = no Action (Key Bindings)
	if GUI_CatFrame_Count == 0 then return end

	local cycleCat_temp
	cycleCat_temp = math.random(GUI_CatFrame_Count)
	cycleCat_temp = GUI_CatFrame_data[cycleCat_temp].Cat

	if AHsearch_Data[cycleCat_temp].queries then
		cycleCat = cycleCat_temp
		cycleSQ  = math.random(GUI_SQFrame_Count[cycleCat])
		cycleSQ  = GUI_SQFrame_data[cycleCat][cycleSQ].Query
		AHsearch:Search(cycleCat, cycleSQ)
	else
		if SearchQueryGlobalCheck() then
			AHsearch:RandomSearch()
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AHsearch:ConfigEnable()
	configMode = true

	AHsearchPrevButton:Disable()
	AHsearchNextButton:Disable()
	AHsearchRandomButton:Disable()

	AHsearch_ConfigurationFrame_SelectionBorder:SetTexture(unpack(configBorderColor))
	AHsearch_ConfigurationFrame_Categories_ContentBorder:SetTexture(unpack(configBorderColor))
	AHsearch_ConfigurationFrame_SearchQueries_ContentBorder:SetTexture(unpack(configBorderColor))
	AHsearch_ConfigurationFrame_Options_ContentBorder:SetTexture(unpack(configBorderColor))

	AHsearch_QueryFrameBorder:SetTexture(unpack(configBorderColor))
	AHsearch_CatFrameBorder:SetTexture(unpack(configBorderColor))

	AHsearch:ConfigurationFrame_ShowContent()
	AHsearch_ConfigurationFrame:Show()

	if GUI_CatFrame_Count == 0 then
		AHsearch_ConfigurationFrame_Categories_NewCat:SetFocus()
		AHsearch_ConfigurationFrame_Categories_SelectCatTitle:SetTextColor(0.5, 0.5, 0.5)
		AHsearch_ConfigurationFrame_Categories_SelectCatBorder:SetTexture(0.4, 0.4, 0.4, 1)
	else
		AHsearch_ConfigurationFrame_Categories_SelectCatTitle:SetTextColor(1, 0.82, 0)
		AHsearch_ConfigurationFrame_Categories_SelectCatBorder:SetTexture(0.8, 0.2, 0.2, 1)
	end

	for i = 1, maxCat do
		_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:SetTexture(1, 0, 0, 0.3)
		_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Selected"]:SetTexture(1, 0, 0, 0.3)
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Selected"]:SetTexture(1, 0, 0, 0.3)
		if GUI_curCat then
			if _G["AHsearch_CatFrameScrollFrame_Button"..i].cat == GUI_curCat then
				_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:Show()
				_G["AHsearch_CatFrameScrollFrame_Button"..i.."SelectedArrow"]:Show()
			end
			if _G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i].cat == GUI_curCat then
				_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Selected"]:Show()
				_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Show()
			end
			if _G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i].cat == GUI_curCat then
				_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Selected"]:Show()
				_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Show()
			end
		end
	end
	for i = 1, maxQuery do
		_G["AHsearch_QueryFrameScrollFrame_Button"..i.."Selected"]:SetTexture(1, 0, 0, 0.3)
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Selected"]:SetTexture(1, 0, 0, 0.3)
		if GUI_curSQ then
			if _G["AHsearch_QueryFrameScrollFrame_Button"..i].sq == GUI_curSQ then
				_G["AHsearch_QueryFrameScrollFrame_Button"..i.."Selected"]:Show()
				_G["AHsearch_QueryFrameScrollFrame_Button"..i.."SelectedArrow"]:Show()
			end
			if _G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i].sq == GUI_curSQ then
				_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Selected"]:Show()
				_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."SelectedArrow"]:Show()
			end
		end
	end

	if GUI_curCat then
		AHsearch:CatFrame_ButtonEventUpdate(GUI_curCat)
		if GUI_curSQ then
			AHsearch:QueryFrame_ButtonEventUpdate(GUI_curCat, GUI_curSQ)
		end
	end
	AHsearch:CatFrame_Show(true)
end

function AHsearch:ConfigDisable(AHevent)
	configMode = false

	if not AHsearch_Data[cycleCat] or
	  (AHsearch_Data[cycleCat] and not AHsearch_Data[cycleCat].queries) or
	  (AHsearch_Data[cycleCat] and AHsearch_Data[cycleCat].queries and not AHsearch_Data[cycleCat].queries[cycleSQ])
	then
		cycleCat = false
	end

	AHsearchPrevButton:Enable()
	AHsearchNextButton:Enable()
	AHsearchRandomButton:Enable()

	AHsearch_ConfigurationFrame_SelectionBorder:SetTexture(unpack(normalBorderColor))
	AHsearch_ConfigurationFrame_Categories_ContentBorder:SetTexture(unpack(normalBorderColor))
	AHsearch_ConfigurationFrame_SearchQueries_ContentBorder:SetTexture(unpack(normalBorderColor))
	AHsearch_ConfigurationFrame_Options_ContentBorder:SetTexture(unpack(normalBorderColor))

	AHsearch_QueryFrameBorder:SetTexture(unpack(normalBorderColor))
	AHsearch_CatFrameBorder:SetTexture(unpack(normalBorderColor))

	AHsearch_ConfigurationFrame:Hide()
	AHsearch_ConfigurationFrame_ConfirmPopup:Hide()
	confirmIsShown = false

	for i = 1, maxCat do
		_G["AHsearch_CatFrameScrollFrame_Button"..i.."Selected"]:SetTexture(1, 1, 1, 0.1)
		_G["AHsearch_CatFrameScrollFrame_Button"..i.."SelectedArrow"]:Hide()
		_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."Selected"]:SetTexture(1, 1, 1, 0.1)
		_G["AHsearch_ConfigurationFrame_Categories_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."Selected"]:SetTexture(1, 1, 1, 0.1)
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectCatScrollFrame_Button"..i.."SelectedArrow"]:Hide()
	end
	for i = 1, maxQuery do
		_G["AHsearch_QueryFrameScrollFrame_Button"..i.."Selected"]:SetTexture(1, 1, 1, 0)
		_G["AHsearch_QueryFrameScrollFrame_Button"..i.."SelectedArrow"]:Hide()
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."Selected"]:SetTexture(1, 1, 1, 0.1)
		_G["AHsearch_ConfigurationFrame_SearchQueries_SelectSQScrollFrame_Button"..i.."SelectedArrow"]:Hide()
	end

	if not AHevent then
		VisibilityCheck(2)
	end
end

function AHsearch:CloseAllFrames()
	configMode = false
	AHsearch_ConfigurationFrame:Hide()
	AHsearch_QueryFrame:Hide()
	AHsearch_CatFrame:Hide()
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
local function OnEvent(self, event)
	if event == "AUCTION_HOUSE_SHOW" then
		if not AHsearchFrame then
			AHsearch:BuildStandardTable()
			AHsearch:BuildFilterTable()
			AHsearch:AHsearchFrame_Create()
			AHsearch:CatFrame_CreateFrame()
			AHsearch:QueryFrame_CreateFrame()
			AHsearch:ConfigurationFrame_CreateFrame()
			AHsearch:InitFrames()
		end
		AHsearch:CycleButtons_Toggle()
		cycleChk = true
	elseif event == "AUCTION_HOUSE_CLOSED" then
		AHsearch:ConfigDisable(1)
		AHsearch:CloseAllFrames()
		cycleChk = false
		cycleCat = false
	elseif event == "PLAYER_LOGIN" then
		AHsearch:InitOptions()
		AHsearch:UnregisterEvent("PLAYER_LOGIN")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

AHsearch:RegisterEvent("PLAYER_LOGIN")
AHsearch:RegisterEvent("AUCTION_HOUSE_SHOW")
AHsearch:RegisterEvent("AUCTION_HOUSE_CLOSED")
AHsearch:SetScript("OnEvent", OnEvent) -- start the Mystical Machine